@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 
 CConnectCommand::CConnectCommand(CServer const& server, ServerHandle const& handle, Credentials const& credentials, bool retry_connecting)
 	: server_(server)
@@ -55,18 +55,18 @@ bool CListCommand::valid() const
 	return true;
 }
 
-CFileTransferCommand::CFileTransferCommand(std::wstring const& localFile, CServerPath const& remotePath,
-										   std::wstring const& remoteFile, bool download,
-										   CFileTransferCommand::t_transferSettings const& transferSettings)
-	: m_localFile(localFile), m_remotePath(remotePath), m_remoteFile(remoteFile)
-	, m_download(download)
-	, m_transferSettings(transferSettings)
+CFileTransferCommand::CFileTransferCommand(reader_factory_holder const& reader, CServerPath const& remotePath,
+										   std::wstring const& remoteFile, transfer_flags const& flags)
+	: reader_(reader), m_remotePath(remotePath), m_remoteFile(remoteFile)
+	, flags_(flags)
 {
 }
 
-std::wstring CFileTransferCommand::GetLocalFile() const
+CFileTransferCommand::CFileTransferCommand(writer_factory_holder const& writer, CServerPath const& remotePath,
+										   std::wstring const& remoteFile, transfer_flags const& flags)
+	: writer_(writer), m_remotePath(remotePath), m_remoteFile(remoteFile)
+	, flags_(flags)
 {
-	return m_localFile;
 }
 
 CServerPath CFileTransferCommand::GetRemotePath() const
@@ -79,9 +79,17 @@ std::wstring CFileTransferCommand::GetRemoteFile() const
 	return m_remoteFile;
 }
 
-bool CFileTransferCommand::Download() const
+bool CFileTransferCommand::valid() const
 {
-	return m_download;
+	if (!reader_ && !writer_) {
+		return false;
+	}
+
+	if (m_remotePath.empty() || m_remoteFile.empty()) {
+		return false;
+	}
+
+	return true;
 }
 
 CRawCommand::CRawCommand(std::wstring const& command)
