@@ -1,6 +1,7 @@
 #ifndef FILEZILLA_ENGINE_CONTROLSOCKET_HEADER
 #define FILEZILLA_ENGINE_CONTROLSOCKET_HEADER
 
+#include "../include/activity_logger.h"
 #include "../include/directorylisting.h"
 #include "../include/server.h"
 #include "../include/serverpath.h"
@@ -60,6 +61,7 @@ public:
 		, engine_(controlSocket.engine_)
 		, currentServer_(controlSocket.currentServer_)
 		, currentPath_(controlSocket.currentPath_)
+		, options_(engine_.GetOptions())
 	{
 	}
 
@@ -74,6 +76,7 @@ public:
 	CFileZillaEnginePrivate & engine_;
 	CServer & currentServer_;
 	CServerPath& currentPath_;
+	COptionsBase& options_;
 };
 
 class ResultOpData : public COpData
@@ -180,20 +183,6 @@ public:
 
 };
 
-enum class TransferEndReason
-{
-	none,
-	successful,
-	timeout,
-	transfer_failure,					// Error during transfer, like lost connection. Retry automatically
-	transfer_failure_critical,			// Error during transfer like lack of diskspace. Needs user interaction
-	pre_transfer_command_failure,		// If a command fails prior to sending the transfer command
-	transfer_command_failure_immediate,	// Used if server does not send the 150 reply after the transfer command
-	transfer_command_failure,			// Used if the transfer command fails, but after receiving a 150 first
-	failure,							// Other unspecific failure
-	failed_resumetest
-};
-
 namespace fz {
 class socket_layer;
 }
@@ -225,8 +214,6 @@ public:
 	virtual void Chmod(CChmodCommand const& command);
 	void Sleep(fz::duration const& delay);
 
-	virtual bool Connected() const = 0;
-
 	Command GetCurrentCommandId() const;
 
 	void SendAsyncRequest(std::unique_ptr<CAsyncRequestNotification> && notification);
@@ -239,7 +226,13 @@ public:
 	std::wstring ConvToLocal(char const* buffer, size_t len);
 	std::string ConvToServer(std::wstring const&, bool force_utf8 = false);
 
-	void SetActive(CFileZillaEngine::_direction direction);
+	void RecordActivity(activity_logger::_direction direction, uint64_t amount);
+	template<typename T, std::enable_if_t<std::is_signed_v<T>, int> = 0>
+	inline void RecordActivity(activity_logger::_direction direction, int64_t amount) {
+		if (amount > 0) {
+			RecordActivity(direction, static_cast<uint64_t>(amount));
+		}
+	}
 
 	void SetHandle(ServerHandle const& handle) { handle_ = handle; }
 	ServerHandle const& GetHandle() const { return handle_; }
@@ -248,7 +241,7 @@ public:
 	// The following two functions control the timeout behaviour:
 	// ---
 
-	// Call this if data could be sent or retrieved
+	// Call this on activity that precludes a timeout
 	void SetAlive();
 
 	// Set to true if waiting for data
@@ -333,6 +326,7 @@ protected:
 	void OnObtainLock();
 };
 
+class activity_logger_layer;
 class CProxySocket;
 
 namespace fz {
@@ -347,8 +341,6 @@ public:
 
 	int DoConnect(std::wstring const& host, unsigned int port);
 
-	virtual bool Connected() const override;
-
 protected:
 	virtual int DoClose(int nErrorCode = FZ_REPLY_DISCONNECTED | FZ_REPLY_ERROR) override;
 	virtual void ResetSocket();
@@ -362,12 +354,15 @@ protected:
 	virtual int OnSend();
 	virtual void OnSocketError(int error);
 
+	virtual void SetSocketBufferSizes() {};
+
 	int Send(unsigned char const* buffer, unsigned int len);
 	int Send(char const* buffer, unsigned int len) {
 		return Send(reinterpret_cast<unsigned char const*>(buffer), len);
 	}
 
 	std::unique_ptr<fz::socket> socket_;
+	std::unique_ptr<activity_logger_layer> activity_logger_layer_;
 	std::unique_ptr<fz::rate_limited_layer> ratelimit_layer_;
 	std::unique_ptr<CProxySocket> proxy_layer_;
 	fz::socket_layer* active_layer_{};
