@@ -17,11 +17,8 @@
 
 #include <algorithm>
 
-#include <assert.h>
-
 fz::mutex CFileZillaEnginePrivate::global_mutex_{false};
 std::vector<CFileZillaEnginePrivate*> CFileZillaEnginePrivate::m_engineList;
-std::atomic_int CFileZillaEnginePrivate::m_activeStatus[2] = {{0}, {0}};
 std::list<CFileZillaEnginePrivate::t_failedLogins> CFileZillaEnginePrivate::m_failedLogins;
 
 struct invalid_current_working_dir_event_type{};
@@ -42,6 +39,7 @@ CFileZillaEnginePrivate::CFileZillaEnginePrivate(CFileZillaEngineContext& contex
 	: event_handler(context.GetEventLoop())
 	, transfer_status_(*this)
 	, opLockManager_(context.GetOpLockManager())
+	, activity_logger_(context.GetActivityLogger())
 	, notification_cb_(notification_cb)
 	, m_engine_id(get_next_engine_id())
 	, options_(context.GetOptions())
@@ -98,18 +96,35 @@ void erase_unordered(C & container, V const& v) noexcept
 }
 
 CFileZillaEnginePrivate::~CFileZillaEnginePrivate()
+{
+	shutdown();
+}
+
+void CFileZillaEnginePrivate::shutdown()
 {
 	options_.unwatch_all(this);
 	remove_handler();
 
-	m_maySendNotificationEvent = false;
+	{
+		fz::scoped_lock lock(notification_mutex_);
+		m_maySendNotificationEvent = false;
+
+		auto cb = std::move(notification_cb_);
+		notification_cb_ = nullptr;
+		// Must not hold mutex when destroying cb, or we'll deadlock
+		lock.unlock();
+	}
 
 	controlSocket_.reset();
 	currentCommand_.reset();
 
-	// Delete notification list
-	for (auto & notification : m_NotificationList) {
-		delete notification;
+	{
+		fz::scoped_lock lock(notification_mutex_);
+		// Delete notification list
+		for (auto & notification : m_NotificationList) {
+			delete notification;
+		}
+		m_NotificationList.clear();
 	}
 
 	// Remove ourself from the engine list
@@ -143,15 +158,14 @@ bool CFileZillaEnginePrivate::IsConnected() const
 	return controlSocket_ != nullptr;
 }
 
-void CFileZillaEnginePrivate::AddNotification(fz::scoped_lock& lock, std::unique_ptr<CNotification> && notification)
+void CFileZillaEnginePrivate::AddNotification(fz::scoped_lock&, std::unique_ptr<CNotification> && notification)
 {
 	if (notification) {
 		m_NotificationList.push_back(notification.release());
 	}
 
-	if (m_maySendNotificationEvent) {
+	if (m_maySendNotificationEvent && notification_cb_) {
 		m_maySendNotificationEvent = false;
-		lock.unlock();
 		notification_cb_(&parent_);
 	}
 }
@@ -187,21 +201,19 @@ void CFileZillaEnginePrivate::AddLogNotification(std::unique_ptr<CLogmsgNotifica
 
 void CFileZillaEnginePrivate::SendQueuedLogs(bool reset_flag)
 {
-	{
-		fz::scoped_lock lock(notification_mutex_);
-		m_NotificationList.insert(m_NotificationList.end(), queued_logs_.begin(), queued_logs_.end());
-		queued_logs_.clear();
+	fz::scoped_lock lock(notification_mutex_);
+	m_NotificationList.insert(m_NotificationList.end(), queued_logs_.begin(), queued_logs_.end());
+	queued_logs_.clear();
 
-		if (reset_flag) {
-			queue_logs_ = ShouldQueueLogsFromOptions();
-		}
-
-		if (!m_maySendNotificationEvent || m_NotificationList.empty()) {
-			return;
-		}
-		m_maySendNotificationEvent = false;
+	if (reset_flag) {
+		queue_logs_ = ShouldQueueLogsFromOptions();
 	}
 
+	if (!m_maySendNotificationEvent || m_NotificationList.empty() || !notification_cb_) {
+		return;
+	}
+	m_maySendNotificationEvent = false;
+
 	notification_cb_(&parent_);
 }
 
@@ -288,8 +300,6 @@ int CFileZillaEnginePrivate::Connect(CConnectCommand const& command)
 		return FZ_REPLY_ALREADYCONNECTED;
 	}
 
-	assert(!controlSocket_);
-
 	m_retryCount = 0;
 
 	auto const& server = command.GetServer();
@@ -339,7 +349,7 @@ int CFileZillaEnginePrivate::List(CListCommand const& command)
 						server.GetProtocol() == SWIFT || server.GetProtocol() == GOOGLE_CLOUD ||
 						server.GetProtocol() == GOOGLE_DRIVE || server.GetProtocol() == DROPBOX ||
 						server.GetProtocol() == ONEDRIVE || server.GetProtocol() == B2 ||
-						server.GetProtocol() == BOX || server.GetProtocol() == RACKSPACE) {
+						server.GetProtocol() == BOX || server.GetProtocol() == RACKSPACE || server.GetProtocol() == STORJ_GRANT) {
 					path = command.GetPath();
 					path.ChangePath(command.GetSubDir());
 				}
@@ -490,7 +500,6 @@ void CFileZillaEnginePrivate::OnTimer(fz::timer_id)
 
 	int res = ContinueConnect();
 	if (res == FZ_REPLY_CONTINUE) {
-		assert(controlSocket_);
 		controlSocket_->SendNextCommand();
 	}
 	else if (res != FZ_REPLY_WOULDBLOCK) {
@@ -534,6 +543,7 @@ int CFileZillaEnginePrivate::ContinueConnect()
 		break;
 #if ENABLE_STORJ
 	case STORJ:
+	case STORJ_GRANT:
 		controlSocket_ = std::make_unique<CStorjControlSocket>(*this);
 		break;
 #endif
@@ -672,8 +682,12 @@ void CFileZillaEnginePrivate::OnCommandEvent()
 		}
 
 		if (res == FZ_REPLY_CONTINUE) {
-			assert(controlSocket_);
-			controlSocket_->SendNextCommand();
+			if (controlSocket_) {
+				controlSocket_->SendNextCommand();
+			}
+			else {
+				ResetOperation(FZ_REPLY_INTERNALERROR);
+			}
 		}
 		else if (res != FZ_REPLY_WOULDBLOCK) {
 			ResetOperation(res);
@@ -689,8 +703,6 @@ void CFileZillaEnginePrivate::DoCancel()
 	}
 
 	if (m_retryTimer) {
-		assert(currentCommand_ && currentCommand_->GetId() == Command::connect);
-
 		controlSocket_.reset();
 
 		currentCommand_.reset();
@@ -713,23 +725,13 @@ void CFileZillaEnginePrivate::DoCancel()
 	}
 }
 
-bool CFileZillaEnginePrivate::CheckAsyncRequestReplyPreconditions(std::unique_ptr<CAsyncRequestNotification> const& reply)
-{
-	if (!reply) {
-		return false;
-	}
-	if (!IsBusy()) {
-		return false;
-	}
-
-	bool const match = reply->requestNumber == asyncRequestCounter_;
-	return match && controlSocket_;
-}
-
 void CFileZillaEnginePrivate::OnSetAsyncRequestReplyEvent(std::unique_ptr<CAsyncRequestNotification> const& reply)
 {
 	fz::scoped_lock lock(mutex_);
-	if (!CheckAsyncRequestReplyPreconditions(reply)) {
+	if (!controlSocket_) {
+		return;
+	}
+	if (!IsPendingAsyncRequestReply(reply)) {
 		return;
 	}
 
@@ -773,7 +775,7 @@ std::unique_ptr<CNotification> CFileZillaEnginePrivate::GetNextNotification()
 bool CFileZillaEnginePrivate::SetAsyncRequestReply(std::unique_ptr<CAsyncRequestNotification> && pNotification)
 {
 	fz::scoped_lock lock(mutex_);
-	if (!CheckAsyncRequestReplyPreconditions(pNotification)) {
+	if (!IsPendingAsyncRequestReply(pNotification)) {
 		return false;
 	}
 
@@ -795,26 +797,6 @@ bool CFileZillaEnginePrivate::IsPendingAsyncRequestReply(std::unique_ptr<CAsyncR
 	return pNotification->requestNumber == asyncRequestCounter_;
 }
 
-void CFileZillaEnginePrivate::SetActive(int direction)
-{
-	int const old_status = m_activeStatus[direction].fetch_or(0x1);
-	if (!old_status) {
-		AddNotification(std::make_unique<CActiveNotification>(direction));
-	}
-}
-
-bool CFileZillaEnginePrivate::IsActive(CFileZillaEngine::_direction direction)
-{
-	int const old = m_activeStatus[direction].exchange(0x2);
-	if (!(old & 0x1)) {
-		// Race: We might lose updates between the first exchange and this assignment.
-		// It is harmless though
-		m_activeStatus[direction] = 0;
-		return false;
-	}
-	return true;
-}
-
 CTransferStatus CFileZillaEnginePrivate::GetTransferStatus(bool &changed)
 {
 	return transfer_status_.Get(changed);
@@ -829,7 +811,9 @@ int CFileZillaEnginePrivate::CacheLookup(const CServerPath& path, CDirectoryList
 		return FZ_REPLY_ERROR;
 	}
 
-	assert(controlSocket_->GetCurrentServer());
+	if (!controlSocket_->GetCurrentServer()) {
+		return FZ_REPLY_INTERNALERROR;
+	}
 
 	bool is_outdated = false;
 	if (!directory_cache_.Lookup(listing, controlSocket_->GetCurrentServer(), path, true, is_outdated)) {
