@@ -3,8 +3,6 @@
 
 #include "ftpcontrolsocket.h"
 
-#include "../iothread.h"
-
 enum filetransferStates
 {
 	filetransfer_init = 0,
@@ -22,7 +20,7 @@ enum filetransferStates
 class CFtpFileTransferOpData final : public CFileTransferOpData, public CFtpTransferOpData, public CFtpOpData
 {
 public:
-	CFtpFileTransferOpData(CFtpControlSocket& controlSocket, bool is_download, std::wstring const& local_file, std::wstring const& remote_file, CServerPath const& remote_path, CFileTransferCommand::t_transferSettings const& settings);
+	CFtpFileTransferOpData(CFtpControlSocket& controlSocket, CFileTransferCommand const& cmd);
 
 	virtual int Send() override;
 	virtual int ParseResponse() override;
@@ -30,7 +28,6 @@ public:
 
 	int TestResumeCapability();
 
-	std::unique_ptr<CIOThread> ioThread_;
 	bool fileDidExist_{true};
 };
 
