@@ -3,12 +3,13 @@
 #include "logon.h"
 #include "../proxy.h"
 #include "../servercapabilities.h"
+#include "../tls.h"
+
+#include "../../include/misc.h"
 
 #include "../../include/engine_options.h"
 
-#include <libfilezilla/tls_layer.hpp>
-
-#include <assert.h>
+using namespace std::literals;
 
 CFtpLogonOpData::CFtpLogonOpData(CFtpControlSocket& controlSocket)
 	: COpData(Command::connect, L"CFtpLogonOpData")
@@ -47,9 +48,9 @@ int CFtpLogonOpData::Send()
 	case LOGON_CONNECT:
 		{
 			// Do not use FTP proxy if generic proxy is set
-			int const generic_proxy_type = engine_.GetOptions().get_int(OPTION_PROXY_TYPE);
+			int const generic_proxy_type = options_.get_int(OPTION_PROXY_TYPE);
 			if ((generic_proxy_type <= static_cast<int>(ProxyType::NONE) || generic_proxy_type >= static_cast<int>(ProxyType::count)) && !currentServer_.GetBypassProxy()) {
-				ftp_proxy_type_ = engine_.GetOptions().get_int(OPTION_FTP_PROXY_TYPE);
+				ftp_proxy_type_ = options_.get_int(OPTION_FTP_PROXY_TYPE);
 			}
 
 			if (!PrepareLoginSequence()) {
@@ -57,7 +58,7 @@ int CFtpLogonOpData::Send()
 			}
 
 			if (ftp_proxy_type_) {
-				host_ = engine_.GetOptions().get_string(OPTION_FTP_PROXY_HOST);
+				host_ = options_.get_string(OPTION_FTP_PROXY_HOST);
 
 				size_t pos = -1;
 				if (!host_.empty() && host_[0] == '[') {
@@ -110,7 +111,7 @@ int CFtpLogonOpData::Send()
 
 				// Enable SO_KEEPALIVE, lots of clueless users have broken routers and
 				// firewalls which terminate the control connection on long transfers.
-				int v = engine_.GetOptions().get_int(OPTION_TCP_KEEPALIVE_INTERVAL);
+				int v = options_.get_int(OPTION_TCP_KEEPALIVE_INTERVAL);
 				if (v >= 1 && v < 10000) {
 					controlSocket_.socket_->set_keepalive_interval(fz::duration::from_minutes(v));
 				}
@@ -145,7 +146,13 @@ int CFtpLogonOpData::Send()
 			{
 			case loginCommandType::user:
 				if (controlSocket_.credentials_.logonType_ == LogonType::interactive) {
-					waitChallenge = true;
+					challengeMode_ = always;
+				}
+				else if (controlSocket_.tls_layer_ && controlSocket_.tls_layer_->get_alpn() == "x-filezilla-ftp"sv) {
+					challengeMode_ = mfa;
+				}
+
+				if (challengeMode_) {
 					challenge.clear();
 				}
 
@@ -158,7 +165,8 @@ int CFtpLogonOpData::Send()
 				}
 			case loginCommandType::pass:
 				if (!challenge.empty()) {
-					auto notification = std::make_unique<CInteractiveLoginNotification>(CInteractiveLoginNotification::interactive, challenge, false);
+					auto type = otp_ ? CInteractiveLoginNotification::totp : CInteractiveLoginNotification::interactive;
+					auto notification = std::make_unique<CInteractiveLoginNotification>(type, challenge, false);
 					notification->server = currentServer_;
 					notification->handle_ = controlSocket_.handle_;
 					notification->credentials = controlSocket_.credentials_;
@@ -169,7 +177,16 @@ int CFtpLogonOpData::Send()
 					return FZ_REPLY_WOULDBLOCK;
 				}
 				else {
-					std::wstring pass = (controlSocket_.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous@example.com" : controlSocket_.credentials_.GetPass();
+					std::wstring pass;
+					if (controlSocket_.credentials_.logonType_ == LogonType::anonymous) {
+						pass = L"anonymous@example.com";
+					}
+					else {
+						pass = controlSocket_.credentials_.GetPass();
+						if (controlSocket_.credentials_.HasExtraParameter("otp_code")) {
+							pass = controlSocket_.credentials_.GetExtraParameter("otp_code") + L";" + pass;
+						}
+					}
 					if (cmd.command.empty()) {
 						return controlSocket_.SendCommand(L"PASS " + pass, true);
 					}
@@ -191,7 +208,9 @@ int CFtpLogonOpData::Send()
 				}
 				break;
 			case loginCommandType::other:
-				assert(!cmd.command.empty());
+				if (cmd.command.empty()) {
+					return FZ_REPLY_INTERNALERROR;
+				}
 				return controlSocket_.SendCommand(cmd.command, cmd.hide_arguments);
 			default:
 				return FZ_REPLY_INTERNALERROR;
@@ -275,6 +294,8 @@ int CFtpLogonOpData::ParseResponse()
 			controlSocket_.tls_layer_ = std::make_unique<fz::tls_layer>(controlSocket_.event_loop_, &controlSocket_, *controlSocket_.active_layer_, &engine_.GetContext().GetTlsSystemTrustStore(), controlSocket_.logger_);
 			controlSocket_.active_layer_ = controlSocket_.tls_layer_.get();
 
+			controlSocket_.tls_layer_->set_alpn({"ftp", "x-filezilla-ftp"});
+			controlSocket_.tls_layer_->set_min_tls_ver(get_min_tls_ver(options_));
 			if (!controlSocket_.tls_layer_->client_handshake(&controlSocket_)) {
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
@@ -288,6 +309,30 @@ int CFtpLogonOpData::ParseResponse()
 	else if (opState == LOGON_LOGON) {
 		t_loginCommand cmd = loginSequence.front();
 
+		if (cmd.type == loginCommandType::user) {
+			if (code == 3 && response.substr(0, 3) == L"336"sv) {
+				if (controlSocket_.tls_layer_ && controlSocket_.tls_layer_->get_alpn() == "x-filezilla-ftp"sv) {
+					size_t beg = challenge.rfind('[');
+					size_t end = challenge.rfind(']');
+					if (beg != std::string::npos && end != std::string::npos && beg < end) {
+						auto tokens = fz::strtok_view(std::wstring_view(challenge).substr(beg + 1, end - beg - 1), ',');
+						if (tokens.size() == 1 && tokens[0] == L"alg:totp"sv) {
+							otp_ = true;
+						}
+					}
+				}
+				else {
+					if (challengeMode_ != always) {
+						log(logmsg::status, _("Consider using the interactive login type."));
+						challenge.clear();
+					}
+				}
+			}
+			else if (challengeMode_ != always) {
+				challenge.clear();
+			}
+		}
+
 		if (code != 2 && code != 3) {
 			if (cmd.type == loginCommandType::user || cmd.type == loginCommandType::pass) {
 				auto const user = currentServer_.GetUser();
@@ -357,7 +402,7 @@ int CFtpLogonOpData::ParseResponse()
 		}
 
 		if (!loginSequence.empty()) {
-			waitChallenge = false;
+			challengeMode_ = none;
 
 			return FZ_REPLY_CONTINUE;
 		}
@@ -596,14 +641,14 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		}
 	}
 	else if (ftp_proxy_type_ == 1) {
-		std::wstring const proxyUser = engine_.GetOptions().get_string(OPTION_FTP_PROXY_USER);
+		std::wstring const proxyUser = options_.get_string(OPTION_FTP_PROXY_USER);
 		if (!proxyUser.empty()) {
 			// Proxy logon (if credendials are set)
 			t_loginCommand cmd = {false, false, loginCommandType::other, L"USER " + proxyUser};
 			loginSequence.push_back(cmd);
 			cmd.optional = true;
 			cmd.hide_arguments = true;
-			cmd.command = L"PASS " + engine_.GetOptions().get_string(OPTION_FTP_PROXY_PASS);
+			cmd.command = L"PASS " + options_.get_string(OPTION_FTP_PROXY_PASS);
 			loginSequence.push_back(cmd);
 		}
 		// User@host
@@ -626,14 +671,14 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		}
 	}
 	else if (ftp_proxy_type_ == 2 || ftp_proxy_type_ == 3) {
-		std::wstring const proxyUser = engine_.GetOptions().get_string(OPTION_FTP_PROXY_USER);
+		std::wstring const proxyUser = options_.get_string(OPTION_FTP_PROXY_USER);
 		if (!proxyUser.empty()) {
 			// Proxy logon (if credendials are set)
 			t_loginCommand cmd = {false, false, loginCommandType::other, L"USER " + proxyUser};
 			loginSequence.push_back(cmd);
 			cmd.optional = true;
 			cmd.hide_arguments = true;
-			cmd.command = L"PASS " + engine_.GetOptions().get_string(OPTION_FTP_PROXY_PASS);
+			cmd.command = L"PASS " + options_.get_string(OPTION_FTP_PROXY_PASS);
 			loginSequence.push_back(cmd);
 		}
 
@@ -666,8 +711,8 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		}
 	}
 	else if (ftp_proxy_type_ == 4) {
-		std::wstring proxyUser = engine_.GetOptions().get_string(OPTION_FTP_PROXY_USER);
-		std::wstring proxyPass = engine_.GetOptions().get_string(OPTION_FTP_PROXY_PASS);
+		std::wstring proxyUser = options_.get_string(OPTION_FTP_PROXY_USER);
+		std::wstring proxyPass = options_.get_string(OPTION_FTP_PROXY_PASS);
 		std::wstring host = currentServer_.Format(ServerFormat::with_optional_port, controlSocket_.credentials_);
 		std::wstring user = (controlSocket_.credentials_.logonType_ == LogonType::anonymous) ? L"anonymous" : currentServer_.GetUser();
 		std::wstring account = controlSocket_.credentials_.account_;
@@ -677,7 +722,7 @@ bool CFtpLogonOpData::PrepareLoginSequence()
 		fz::replace_substrings(user, L"%", L"%%");
 		fz::replace_substrings(account, L"%", L"%%");
 
-		std::wstring const loginSequenceStr = engine_.GetOptions().get_string(OPTION_FTP_PROXY_CUSTOMLOGINSEQUENCE);
+		std::wstring const loginSequenceStr = options_.get_string(OPTION_FTP_PROXY_CUSTOMLOGINSEQUENCE);
 		std::vector<std::wstring> const tokens = fz::strtok(loginSequenceStr, L"\r\n");
 
 		for (auto token : tokens) {
@@ -849,3 +894,18 @@ void CFtpLogonOpData::ParseFeat(std::wstring line)
 		CServerCapabilities::SetCapability(currentServer_, epsv_command, yes);
 	}
 }
+
+void CFtpLogonOpData::tls_handshake_finished()
+{
+	if (opState == LOGON_AUTH_WAIT) {
+		if (controlSocket_.tls_layer_ && controlSocket_.tls_layer_->get_alpn() == "x-filezilla-ftp") {
+			neededCommands[LOGON_SYST] = 0;
+			neededCommands[LOGON_PBSZ] = 0;
+			neededCommands[LOGON_PROT] = 0;
+			neededCommands[LOGON_CLNT] = 0;
+			neededCommands[LOGON_OPTSUTF8] = 0;
+			controlSocket_.m_protectDataChannel = true;
+		}
+		opState = LOGON_LOGON;
+	}
+}
