@@ -1,8 +1,10 @@
 #include "../filezilla.h"
+#include "../activity_logger_layer.h"
 #include "../directorylistingparser.h"
 #include "../engineprivate.h"
 #include "../proxy.h"
 #include "../servercapabilities.h"
+#include "../tls.h"
 
 #include "ftpcontrolsocket.h"
 #include "transfersocket.h"
@@ -10,10 +12,9 @@
 #include "../../include/engine_options.h"
 
 #include <libfilezilla/rate_limited_layer.hpp>
-#include <libfilezilla/tls_layer.hpp>
 #include <libfilezilla/util.hpp>
 
-#include <assert.h>
+using namespace std::literals;
 
 #ifndef FZ_WINDOWS
 #define HAVE_ASCII_TRANSFORM 1
@@ -246,6 +247,7 @@ void CTransferSocket::ResetSocket()
 	tls_layer_.reset();
 	proxy_layer_.reset();
 	ratelimit_layer_.reset();
+	activity_logger_layer_.reset();
 	socket_.reset();
 	buffer_.reset();
 }
@@ -384,6 +386,43 @@ void CTransferSocket::OnConnect()
 	}
 
 	if (tls_layer_) {
+		auto const cap = CServerCapabilities::GetCapability(controlSocket_.currentServer_, tls_resumption);
+
+		if (controlSocket_.tls_layer_->get_alpn() == "x-filezilla-ftp"sv) {
+			if (!tls_layer_->resumed_session()) {
+				TransferEnd(TransferEndReason::failed_tls_resumption);
+				return;
+			}
+			else if (tls_layer_->get_alpn() != "ftp-data"sv) {
+				controlSocket_.log(logmsg::error, _("Wrong ALPN on data connection"));
+				TransferEnd(TransferEndReason::wrong_tls_alpn);
+				return;
+			}
+
+			if (cap != yes) {
+				engine_.AddNotification(std::make_unique<FtpTlsResumptionNotification>(controlSocket_.currentServer_));
+				CServerCapabilities::SetCapability(controlSocket_.currentServer_, tls_resumption, yes);
+			}
+		}
+		else {
+			if (tls_layer_->resumed_session()) {
+				if (cap != yes) {
+					engine_.AddNotification(std::make_unique<FtpTlsResumptionNotification>(controlSocket_.currentServer_));
+					CServerCapabilities::SetCapability(controlSocket_.currentServer_, tls_resumption, yes);
+				}
+			}
+			else {
+				if (cap == yes) {
+					TransferEnd(TransferEndReason::failed_tls_resumption);
+					return;
+				}
+				else if (cap == unknown) {
+					// Ask whether to allow this insecure connection
+					++activity_block_;
+					controlSocket_.SendAsyncRequest(std::make_unique<FtpTlsNoResumptionNotification>(controlSocket_.currentServer_));
+				}
+			}
+		}
 		// Re-enable Nagle algorithm
 		socket_->set_flags(fz::socket::flag_nodelay, false);
 	}
@@ -395,16 +434,18 @@ void CTransferSocket::OnConnect()
 	}
 #endif
 
-	if (m_bActive) {
+	if (!activity_block_) {
 		TriggerPostponedEvents();
 	}
+
+	OnSend();
 }
 
 void CTransferSocket::OnReceive()
 {
 	controlSocket_.log(logmsg::debug_debug, L"CTransferSocket::OnReceive(), m_transferMode=%d", m_transferMode);
 
-	if (!m_bActive) {
+	if (activity_block_) {
 		controlSocket_.log(logmsg::debug_verbose, L"Postponing receive, m_bActive was false.");
 		m_postponedReceive = true;
 		return;
@@ -412,7 +453,8 @@ void CTransferSocket::OnReceive()
 
 	if (m_transferEndReason == TransferEndReason::none) {
 		if (m_transferMode == TransferMode::list) {
-			for (;;) {
+			// See comment in download loop
+			for (int i = 0; i < 100; ++i) {
 				char *pBuffer = new char[4096];
 				int error;
 				int numread = active_layer_->read(pBuffer, 4096, error);
@@ -431,7 +473,7 @@ void CTransferSocket::OnReceive()
 						return;
 					}
 
-					controlSocket_.SetActive(CFileZillaEngine::recv);
+					controlSocket_.SetAlive();
 					if (!m_madeProgress) {
 						m_madeProgress = 2;
 						engine_.transfer_status_.SetMadeProgress();
@@ -444,6 +486,7 @@ void CTransferSocket::OnReceive()
 					return;
 				}
 			}
+			send_event<fz::socket_event>(active_layer_, fz::socket_event_flag::read, 0);
 			return;
 		}
 		else if (m_transferMode == TransferMode::download) {
@@ -464,7 +507,7 @@ void CTransferSocket::OnReceive()
 					break;
 				}
 
-				controlSocket_.SetActive(CFileZillaEngine::recv);
+				controlSocket_.SetAlive();
 				if (!m_madeProgress) {
 					m_madeProgress = 2;
 					engine_.transfer_status_.SetMadeProgress();
@@ -552,7 +595,7 @@ void CTransferSocket::OnSend()
 		return;
 	}
 
-	if (!m_bActive) {
+	if (activity_block_) {
 		controlSocket_.log(logmsg::debug_verbose, L"Postponing send");
 		m_postponedSend = true;
 		return;
@@ -578,7 +621,7 @@ void CTransferSocket::OnSend()
 			break;
 		}
 
-		controlSocket_.SetActive(CFileZillaEngine::send);
+		controlSocket_.SetAlive();
 		if (m_madeProgress == 1) {
 			controlSocket_.log(logmsg::debug_debug, L"Made progress in CTransferSocket::OnSend()");
 			m_madeProgress = 2;
@@ -670,7 +713,8 @@ bool CTransferSocket::SetupPassiveTransfer(std::wstring const& host, int port)
 
 bool CTransferSocket::InitLayers(bool active)
 {
-	ratelimit_layer_ = std::make_unique<fz::rate_limited_layer>(nullptr, *socket_, &engine_.GetRateLimiter());
+	activity_logger_layer_ = std::make_unique<activity_logger_layer>(nullptr, *socket_, engine_.activity_logger_);
+	ratelimit_layer_ = std::make_unique<fz::rate_limited_layer>(nullptr, *activity_logger_layer_, &engine_.GetRateLimiter());
 	active_layer_ = ratelimit_layer_.get();
 
 	if (controlSocket_.proxy_layer_ && !active) {
@@ -693,7 +737,11 @@ bool CTransferSocket::InitLayers(bool active)
 
 		tls_layer_ = std::make_unique<fz::tls_layer>(controlSocket_.event_loop_, nullptr, *active_layer_, nullptr, controlSocket_.logger_);
 		active_layer_ = tls_layer_.get();
+		tls_layer_->set_min_tls_ver(get_min_tls_ver(engine_.GetOptions()));
 
+		if (controlSocket_.tls_layer_->get_alpn() == "x-filezilla-ftp"sv) {
+			tls_layer_->set_alpn("ftp-data"sv);
+		}
 		if (!tls_layer_->client_handshake(controlSocket_.tls_layer_->get_raw_certificate(), controlSocket_.tls_layer_->get_session_parameters(), controlSocket_.tls_layer_->peer_host())) {
 			return false;
 		}
@@ -710,7 +758,10 @@ void CTransferSocket::SetActive()
 		return;
 	}
 
-	m_bActive = true;
+	if (!activity_block_) {
+		return;
+	}
+	--activity_block_;
 	if (!socket_) {
 		return;
 	}
@@ -781,7 +832,6 @@ std::unique_ptr<fz::listen_socket> CTransferSocket::CreateSocketServer()
 
 	if (start < low || start > high) {
 		start = static_cast<decltype(start)>(fz::random_number(low, high));
-		assert(start >= low && start <= high);
 	}
 
 	std::unique_ptr<fz::listen_socket> server;
@@ -847,7 +897,7 @@ bool CTransferSocket::CheckGetNextReadBuffer()
 
 void CTransferSocket::OnInput(reader_base*)
 {
-	if (!m_bActive || m_transferEndReason != TransferEndReason::none) {
+	if (activity_block_ || m_transferEndReason != TransferEndReason::none) {
 		return;
 	}
 
@@ -858,7 +908,7 @@ void CTransferSocket::OnInput(reader_base*)
 
 void CTransferSocket::OnWrite(writer_base*)
 {
-	if (!m_bActive || m_transferEndReason != TransferEndReason::none) {
+	if (activity_block_ || m_transferEndReason != TransferEndReason::none) {
 		return;
 	}
 
@@ -889,7 +939,9 @@ void CTransferSocket::FinalizeWrite()
 
 void CTransferSocket::TriggerPostponedEvents()
 {
-	assert(m_bActive);
+	if (activity_block_) {
+		return;
+	}
 
 	if (m_postponedReceive) {
 		controlSocket_.log(logmsg::debug_verbose, L"Executing postponed receive");
@@ -928,10 +980,20 @@ void CTransferSocket::operator()(fz::event_base const& ev)
 
 void CTransferSocket::OnTimer(fz::timer_id)
 {
+#if FZ_WINDOWS
 	if (socket_ && socket_->is_connected()) {
 		int const ideal_send_buffer = socket_->ideal_send_buffer_size();
 		if (ideal_send_buffer != -1) {
 			socket_->set_buffer_sizes(-1, ideal_send_buffer);
 		}
 	}
+#endif
+}
+
+void CTransferSocket::ContinueWithoutSesssionResumption()
+{
+	if (activity_block_) {
+		--activity_block_;
+		TriggerPostponedEvents();
+	}
 }
