@@ -39,6 +39,8 @@ public:
 	void set_reader(std::unique_ptr<reader_base> && reader, bool ascii);
 	void set_writer(std::unique_ptr<writer_base> && writer, bool ascii);
 
+	void ContinueWithoutSesssionResumption();
+
 protected:
 	bool CheckGetNextWriteBuffer();
 	bool CheckGetNextReadBuffer();
@@ -74,7 +76,7 @@ protected:
 	CFileZillaEnginePrivate & engine_;
 	CFtpControlSocket & controlSocket_;
 
-	bool m_bActive{};
+	int activity_block_{1};
 	TransferEndReason m_transferEndReason{TransferEndReason::none};
 
 	TransferMode const m_transferMode;
@@ -84,6 +86,7 @@ protected:
 	void TriggerPostponedEvents();
 
 	std::unique_ptr<fz::socket> socket_;
+	std::unique_ptr<activity_logger_layer> activity_logger_layer_;
 	std::unique_ptr<fz::rate_limited_layer> ratelimit_layer_;
 	std::unique_ptr<CProxySocket> proxy_layer_;
 	std::unique_ptr<fz::tls_layer> tls_layer_;
