@@ -6,13 +6,15 @@
 #include "internalconnect.h"
 #include "request.h"
 
+#include "../../include/engine_options.h"
+
 #include "../controlsocket.h"
 #include "../engineprivate.h"
+#include "../tls.h"
 
 #include <libfilezilla/file.hpp>
 #include <libfilezilla/iputils.hpp>
 #include <libfilezilla/local_filesys.hpp>
-#include <libfilezilla/tls_layer.hpp>
 #include <libfilezilla/uri.hpp>
 
 #include <assert.h>
@@ -43,7 +45,7 @@ uint64_t HttpRequest::update_content_length()
 
 int HttpRequest::reset()
 {
-	flags_ &= flag_update_transferstatus;
+	flags_ &= (flag_update_transferstatus | flag_confidential_querystring);
 
 	if (body_) {
 		aio_result res = body_->rewind();
@@ -206,7 +208,7 @@ void CHttpControlSocket::OnReceive()
 
 void CHttpControlSocket::OnConnect()
 {
-	if (operations_.empty() || operations_.back()->opId != PrivCommand::http_connect) {
+	if (operations_.empty() || operations_.back()->opId != PrivCommand::http_connect || !socket_) {
 		log(logmsg::debug_warning, L"Discarding stale OnConnect");
 		return;
 	}
@@ -222,7 +224,9 @@ void CHttpControlSocket::OnConnect()
 			tls_layer_ = std::make_unique<fz::tls_layer>(event_loop_, this, *active_layer_, &engine_.GetContext().GetTlsSystemTrustStore(), logger_);
 			active_layer_ = tls_layer_.get();
 
+			tls_layer_->set_alpn("http/1.1");
 			if (!tls_layer_->client_handshake(&data)) {
+				tls_layer_->set_min_tls_ver(get_min_tls_ver(engine_.GetOptions()));
 				DoClose();
 			}
 		}
@@ -248,13 +252,13 @@ void CHttpControlSocket::FileTransfer(CFileTransferCommand const& cmd)
 	Push(std::make_unique<CHttpFileTransferOpData>(*this, cmd));
 }
 
-void CHttpControlSocket::FileTransfer(CHttpRequestCommand const& command)
+void CHttpControlSocket::FileTransfer(CHttpRequestCommand const& cmd)
 {
 	log(logmsg::debug_verbose, L"CHttpControlSocket::FileTransfer()");
 
-	log(logmsg::status, _("Requesting %s"), command.uri_.to_string());
+	log(logmsg::status, _("Requesting %s"), cmd.uri_.to_string(!cmd.confidential_qs_));
 
-	Push(std::make_unique<CHttpFileTransferOpData>(*this, command.uri_, command.verb_, command.body_, command.output_));
+	Push(std::make_unique<CHttpFileTransferOpData>(*this, cmd));
 }
 
 void CHttpControlSocket::Request(std::shared_ptr<HttpRequestResponseInterface> const& request)
@@ -285,7 +289,7 @@ int CHttpControlSocket::InternalConnect(std::wstring const& host, unsigned short
 {
 	log(logmsg::debug_verbose, L"CHttpControlSocket::InternalConnect()");
 
-	if (!Connected()) {
+	if (!currentServer_) {
 		return FZ_REPLY_INTERNALERROR;
 	}
 
@@ -357,27 +361,17 @@ int CHttpControlSocket::OnSend()
 	return res;
 }
 
-void CHttpControlSocket::OnWriteReady(writer_base* writer)
+void CHttpControlSocket::SetSocketBufferSizes()
 {
-	if (operations_.empty() || operations_.back()->opId != PrivCommand::http_request) {
-		log(logmsg::debug_warning, L"Stale writer event");
+	if (!socket_) {
 		return;
 	}
 
-	int res = static_cast<CHttpRequestOpData&>(*operations_.back()).OnReceive(writer);
-	if (res == FZ_REPLY_CONTINUE) {
-		SendNextCommand();
-	}
-	else if (res != FZ_REPLY_WOULDBLOCK) {
-		ResetOperation(res);
-	}
-}
-
-void CHttpControlSocket::operator()(fz::event_base const& ev)
-{
-	if (!fz::dispatch<write_ready_event>(ev, this,
-		&CHttpControlSocket::OnWriteReady))
-	{
-		CRealControlSocket::operator()(ev);
-	}
+	const int size_read = engine_.GetOptions().get_int(OPTION_SOCKET_BUFFERSIZE_RECV);
+#if FZ_WINDOWS
+	const int size_write = -1;
+#else
+	const int size_write = engine_.GetOptions().get_int(OPTION_SOCKET_BUFFERSIZE_SEND);
+#endif
+	socket_->set_buffer_sizes(size_read, size_write);
 }
