@@ -66,9 +66,11 @@ public:
 	std::string verb_;
 
 	enum flags {
-		flag_sent_header = 0x01,
-		flag_sent_body = 0x02,
-		flag_update_transferstatus = 0x04
+		flag_sending_header = 0x01,
+		flag_sent_header = 0x02,
+		flag_sent_body = 0x04,
+		flag_update_transferstatus = 0x08,
+		flag_confidential_querystring = 0x10
 	};
 	int flags_{};
 
@@ -183,8 +185,6 @@ public:
 	CHttpControlSocket(CFileZillaEnginePrivate & engine);
 	virtual ~CHttpControlSocket();
 
-	virtual bool Connected() const override { return static_cast<bool>(currentServer_); }
-
 	void FileTransfer(CHttpRequestCommand const& command);
 
 protected:
@@ -219,11 +219,10 @@ protected:
 	virtual void OnReceive() override;
 	virtual int OnSend() override;
 
-	virtual void operator()(fz::event_base const& ev) override;
-	void OnWriteReady(writer_base* writer);
-
 	virtual void ResetSocket() override;
 
+	virtual void SetSocketBufferSizes() override;
+
 	friend class CProtocolOpData<CHttpControlSocket>;
 	friend class CHttpFileTransferOpData;
 	friend class CHttpInternalConnectOpData;
