@@ -4,7 +4,6 @@
 
 #include <libfilezilla/encode.hpp>
 
-#include <assert.h>
 #include <string.h>
 
 CHttpRequestOpData::CHttpRequestOpData(CHttpControlSocket & controlSocket, std::shared_ptr<HttpRequestResponseInterface> const& request)
@@ -14,7 +13,7 @@ CHttpRequestOpData::CHttpRequestOpData(CHttpControlSocket & controlSocket, std::
 {
 	opState = request_init | request_reading;
 
-	request->request().flags_ &= HttpRequest::flag_update_transferstatus;
+	request->request().flags_ &= (HttpRequest::flag_update_transferstatus | HttpRequest::flag_confidential_querystring);
 	request->response().flags_ = 0;
 
 	requests_.emplace_back(request);
@@ -27,7 +26,7 @@ CHttpRequestOpData::CHttpRequestOpData(CHttpControlSocket & controlSocket, std::
 	, requests_(requests)
 {
 	for (auto & rr : requests_) {
-		rr->request().flags_ &= HttpRequest::flag_update_transferstatus;
+		rr->request().flags_ &= (HttpRequest::flag_update_transferstatus | HttpRequest::flag_confidential_querystring);
 		rr->response().flags_ = 0;
 	}
 	opState = request_init | request_reading;
@@ -42,6 +41,10 @@ CHttpRequestOpData::~CHttpRequestOpData()
 			}
 		}
 	}
+	if (!requests_.empty() && requests_.front() && requests_.front()->response().writer_) {
+		requests_.front()->response().writer_->set_handler(nullptr);
+	}
+
 	remove_handler();
 }
 
@@ -67,13 +70,17 @@ void CHttpRequestOpData::AddRequest(std::shared_ptr<HttpRequestResponseInterface
 			}
 		}
 	}
-	rr->request().flags_ &= HttpRequest::flag_update_transferstatus;
+	rr->request().flags_ &= (HttpRequest::flag_update_transferstatus | HttpRequest::flag_confidential_querystring);
 	rr->response().flags_ = 0;
 	requests_.push_back(rr);
 }
 
 int CHttpRequestOpData::Send()
 {
+	if (!controlSocket_.send_buffer_.empty()) {
+		return FZ_REPLY_WOULDBLOCK;
+	}
+
 	if (opState & request_init) {
 		if (send_pos_ >= requests_.size()) {
 			opState &= ~request_init;
@@ -169,30 +176,47 @@ int CHttpRequestOpData::Send()
 		else {
 			auto & req = requests_[send_pos_]->request();
 			if (!(req.flags_ & HttpRequest::flag_sent_header)) {
-				dataToSend_ = req.update_content_length();
+				if (!(req.flags_ & HttpRequest::flag_sending_header)) {
+					dataToSend_ = req.update_content_length();
 
-				if (dataToSend_ == aio_base::nosize) {
-					log(logmsg::error, _("Malformed request header: %s"), _("Invalid Content-Length"));
-					return FZ_REPLY_INTERNALERROR;
-				}
+					if (dataToSend_ == aio_base::nosize) {
+						log(logmsg::error, _("Malformed request header: %s"), _("Invalid Content-Length"));
+						return FZ_REPLY_INTERNALERROR;
+					}
 
-				// Assemble request and headers
-				std::string command = fz::sprintf("%s %s HTTP/1.1", req.verb_, req.uri_.get_request());
-				log(logmsg::command, "%s", command);
-				command += "\r\n";
+					req.flags_ |= HttpRequest::flag_sending_header;
 
-				for (auto const& header : req.headers_) {
-					std::string line = fz::sprintf("%s: %s", header.first, header.second);
-					if (header.first == "Authorization") {
-						log(logmsg::command, "%s: %s", header.first, std::string(header.second.size(), '*'));
+					// Assemble request and headers
+					std::string command = fz::sprintf("%s %s HTTP/1.1", req.verb_, req.uri_.get_request());
+					if (!(req.flags_ & HttpRequest::flag_confidential_querystring)) {
+						log(logmsg::command, "%s", command);
 					}
 					else {
-						log(logmsg::command, "%s", line);
+						log(logmsg::command, "%s %s HTTP/1.1", req.verb_, req.uri_.get_request(false));
 					}
-					command += line + "\r\n";
-				}
+					command += "\r\n";
 
-				command += "\r\n";
+					for (auto const& header : req.headers_) {
+						std::string line = fz::sprintf("%s: %s", header.first, header.second);
+						if (header.first == "Authorization") {
+							log(logmsg::command, "%s: %s", header.first, std::string(header.second.size(), '*'));
+						}
+						else {
+							log(logmsg::command, "%s", line);
+						}
+						command += line + "\r\n";
+					}
+
+					command += "\r\n";
+
+					auto result = controlSocket_.Send(command.c_str(), command.size());
+					if (result == FZ_REPLY_WOULDBLOCK && !controlSocket_.send_buffer_) {
+						result = FZ_REPLY_CONTINUE;
+					}
+					if (result != FZ_REPLY_CONTINUE) {
+						return result;
+					}
+				}
 
 				req.flags_ |= HttpRequest::flag_sent_header;
 				if (!req.body_) {
@@ -208,113 +232,99 @@ int CHttpRequestOpData::Send()
 							opState |= request_init;
 						}
 					}
+					return FZ_REPLY_CONTINUE;
+				}
+
+				log(logmsg::debug_info, "Finished sending request header.");
+				sendLogLevel_ = logmsg::debug_debug;
+
+				// Enable Nagle's algorithm if we have a beefy body
+				if (req.body_->size() > 536) { // TCPv4 minimum required MSS
+					controlSocket_.socket_->set_flags(fz::socket::flag_nodelay, false);
+				}
+#if FZ_WINDOWS
+				// TCP send buffer autotuning
+				if (!buffer_tuning_timer_) {
+					buffer_tuning_timer_ = add_timer(fz::duration::from_seconds(1), false);
+				}
+#endif
+			}
+
+			while (dataToSend_) {
+
+				if (req.body_buffer_.empty()) {
+					read_result r = req.body_->read();
+					if (r.type_ == aio_result::wait) {
+						return FZ_REPLY_WOULDBLOCK;
+					}
+					else if (r.type_ == aio_result::error) {
+						return FZ_REPLY_ERROR;
+					}
+					req.body_buffer_ = r.buffer_;
+
+					if (req.body_buffer_.empty() && dataToSend_) {
+						log(logmsg::error, _("Unexpected end-of-file on '%s'"), req.body_->name());
+						return FZ_REPLY_ERROR;
+					}
+					else if (req.body_buffer_.size() > dataToSend_) {
+						log(logmsg::error, _("Excess data read from '%s'"), req.body_->name());
+						return FZ_REPLY_ERROR;
+					}
+				}
+
+				int error;
+				int written = controlSocket_.active_layer_->write(req.body_buffer_.get(), req.body_buffer_.size(), error);
+				if (written < 0) {
+					if (error != EAGAIN) {
+						log(logmsg::error, _("Could not write to socket: %s"), fz::socket_error_description(error));
+						log(logmsg::error, _("Disconnected from server"));
+						return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
+					}
+					return FZ_REPLY_WOULDBLOCK;
+				}
+				else if (written) {
+					controlSocket_.SetAlive();
+					req.body_buffer_.consume(static_cast<size_t>(written));
+					dataToSend_ -= written;
+					if (req.flags_ & HttpRequest::flag_update_transferstatus) {
+						engine_.transfer_status_.Update(written);
+					}
+				}
+			}
+
+			log(logmsg::debug_info, "Finished sending request body");
+
+#if FZ_WINDOWS
+			stop_timer(buffer_tuning_timer_);
+#endif
+			controlSocket_.socket_->set_flags(fz::socket::flag_nodelay, true);
+
+			req.flags_ |= HttpRequest::flag_sent_body;
+
+			sendLogLevel_ = logmsg::debug_verbose;
+
+			opState &= ~request_send;
+			++send_pos_;
+
+			if (send_pos_ < requests_.size()) {
+				if (!req.keep_alive()) {
+					opState |= request_send_wait_for_read;
+					log(logmsg::debug_info, L"Request did not ask for keep-alive. Waiting for response to finish before sending next request a new connection.");
 				}
 				else {
-					log(logmsg::debug_info, "Finished sending request header.");
-					sendLogLevel_ = logmsg::debug_debug;
-
-					// Disable Nagle's algorithm if we have a beefy body
-					if (req.body_->size() > 536) { // TCPv4 minimum required MSS
-						controlSocket_.socket_->set_flags(fz::socket::flag_nodelay, false);
-					}
+					opState |= request_init;
 				}
-
-				auto result = controlSocket_.Send(command.c_str(), command.size());
-				if (result == FZ_REPLY_WOULDBLOCK && !controlSocket_.send_buffer_) {
-					result = FZ_REPLY_CONTINUE;
-				}
-
-				return result;
 			}
-			else {
-				while (controlSocket_.send_buffer_) {
-					int error;
-					int written = controlSocket_.active_layer_->write(controlSocket_.send_buffer_.get(), controlSocket_.send_buffer_.size(), error);
-					if (written < 0) {
-						if (error != EAGAIN) {
-							log(logmsg::error, _("Could not write to socket: %s"), fz::socket_error_description(error));
-							log(logmsg::error, _("Disconnected from server"));
-							return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
-						}
-						return FZ_REPLY_WOULDBLOCK;
-					}
-					else if (written) {
-						controlSocket_.SetActive(CFileZillaEngine::send);
-						controlSocket_.send_buffer_.consume(static_cast<size_t>(written));
-					}
-				}
 
-				while (dataToSend_) {
+			return FZ_REPLY_CONTINUE;
 
-					if (req.body_buffer_.empty()) {
-						read_result r = req.body_->read();
-						if (r.type_ == aio_result::wait) {
-							return FZ_REPLY_WOULDBLOCK;
-						}
-						else if (r.type_ == aio_result::error) {
-							return FZ_REPLY_ERROR;
-						}
-						req.body_buffer_ = r.buffer_;
-
-						if (req.body_buffer_.empty() && dataToSend_) {
-							log(logmsg::error, _("Unexpected end-of-file on '%s'"), req.body_->name());
-							return FZ_REPLY_ERROR;
-						}
-						else if (req.body_buffer_.size() > dataToSend_) {
-							log(logmsg::error, _("Excess data read from '%s'"), req.body_->name());
-							return FZ_REPLY_ERROR;
-						}
-					}
-
-					int error;
-					int written = controlSocket_.active_layer_->write(req.body_buffer_.get(), req.body_buffer_.size(), error);
-					if (written < 0) {
-						if (error != EAGAIN) {
-							log(logmsg::error, _("Could not write to socket: %s"), fz::socket_error_description(error));
-							log(logmsg::error, _("Disconnected from server"));
-							return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
-						}
-						return FZ_REPLY_WOULDBLOCK;
-					}
-					else if (written) {
-						controlSocket_.SetActive(CFileZillaEngine::send);
-						req.body_buffer_.consume(static_cast<size_t>(written));
-						dataToSend_ -= written;
-						if (req.flags_ & HttpRequest::flag_update_transferstatus) {
-							engine_.transfer_status_.Update(written);
-						}
-					}
-				}
-
-				log(logmsg::debug_info, "Finished sending request body");
-
-				controlSocket_.socket_->set_flags(fz::socket::flag_nodelay, true);
-
-				req.flags_ |= HttpRequest::flag_sent_body;
-
-				sendLogLevel_ = logmsg::debug_verbose;
-
-				opState &= ~request_send;
-				++send_pos_;
-
-				if (send_pos_ < requests_.size()) {
-					if (!req.keep_alive()) {
-						opState |= request_send_wait_for_read;
-						log(logmsg::debug_info, L"Request did not ask for keep-alive. Waiting for response to finish before sending next request a new connection.");
-					}
-					else {
-						opState |= request_init;
-					}
-				}
-				return FZ_REPLY_CONTINUE;
-			}
 		}
 	}
-	
+
 	if (opState & request_reading) {
 		return FZ_REPLY_WOULDBLOCK;
 	}
-	
+
 	return FZ_REPLY_INTERNALERROR;
 }
 
@@ -353,12 +363,16 @@ int CHttpRequestOpData::FinalizeResponseBody()
 
 int CHttpRequestOpData::ParseReceiveBuffer()
 {
+	if (read_state_.done_) {
+		return FinalizeResponseBody();
+	}
+
 	auto & shared_response = requests_.front();
 	if (shared_response) {
 		auto & request = shared_response->request();
 		if (!(request.flags_ & HttpRequest::flag_sent_header)) {
 			if (read_state_.eof_) {
-				log(logmsg::debug_verbose, L"Socket closed before request got sent");
+				log(logmsg::debug_verbose, L"Socket closed before request headers got sent");
 				log(logmsg::error, _("Connection closed by server"));
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
@@ -367,70 +381,56 @@ int CHttpRequestOpData::ParseReceiveBuffer()
 				return FZ_REPLY_ERROR;
 			}
 		}
+
 		auto & response = shared_response->response();
 
 		if (!response.got_header()) {
-			if (read_state_.eof_) {
+			int res = ParseHeader();
+			if (read_state_.eof_ && res == (FZ_REPLY_WOULDBLOCK | FZ_REPLY_CONTINUE)) {
 				log(logmsg::debug_verbose, L"Socket closed before headers got received");
 				log(logmsg::error, _("Connection closed by server"));
 				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
 			}
-
-			return ParseHeader();
-		}
-
-		if (response.flags_ & HttpResponse::flag_got_body) {
-			return FinalizeResponseBody();
-		}
-	}
-
-	if (read_state_.transfer_encoding_ == chunked) {
-		if (read_state_.eof_) {
-			log(logmsg::debug_verbose, L"Socket closed, chunk incomplete");
-			log(logmsg::error, _("Connection closed by server"));
-			return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
-		}
-		return ParseChunkedData();
-	}
-	else {
-		if (read_state_.eof_) {
-			assert(recv_buffer_.empty());
-
-			if (read_state_.responseContentLength_ != -1 && read_state_.receivedData_ != read_state_.responseContentLength_) {
-				log(logmsg::debug_verbose, L"Socket closed, content length not reached");
-				log(logmsg::error, _("Connection closed by server"));
-				return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
-			}
-			else {
-				return FinalizeResponseBody();
-			}
-		}
-		else {
-			size_t size = recv_buffer_.size();
-			if (!size) {
-				return FZ_REPLY_WOULDBLOCK|FZ_REPLY_CONTINUE;
-			}
-
-			if (read_state_.responseContentLength_ != -1 && read_state_.receivedData_ + static_cast<int64_t>(size) > read_state_.responseContentLength_) {
-				size = static_cast<size_t>(read_state_.responseContentLength_ - read_state_.receivedData_);
-			}
-
-			int res = ProcessData(recv_buffer_.get(), size);
-			recv_buffer_.consume(recv_buffer_.size() - size);
 			return res;
 		}
 	}
 
-	return FZ_REPLY_INTERNALERROR;
-}
-
-int CHttpRequestOpData::OnReceive(writer_base* writer)
-{
-	if (requests_.empty() || requests_.back()->response().writer_.get() != writer) {
-		log(logmsg::debug_warning, L"Stale writer event");
-		return FZ_REPLY_WOULDBLOCK;;
+	if (read_state_.transfer_encoding_ == chunked) {
+		int res = ParseChunkedData();
+		if (read_state_.eof_ && res == (FZ_REPLY_WOULDBLOCK | FZ_REPLY_CONTINUE)) {
+			log(logmsg::debug_verbose, L"Socket closed, chunk incomplete");
+			log(logmsg::error, _("Connection closed by server"));
+			return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
+		}
+		return res;
 	}
-	return OnReceive(true);
+	else {
+		size_t size = recv_buffer_.size();
+		if (!size) {
+			if (read_state_.eof_) {
+				if (read_state_.responseContentLength_ != -1 && read_state_.receivedData_ != read_state_.responseContentLength_) {
+					log(logmsg::debug_verbose, L"Socket closed, content length not reached");
+					log(logmsg::error, _("Connection closed by server"));
+					return FZ_REPLY_ERROR | FZ_REPLY_DISCONNECTED;
+				}
+				else {
+					read_state_.done_ = true;
+					return FinalizeResponseBody();
+				}
+			}
+			return FZ_REPLY_WOULDBLOCK|FZ_REPLY_CONTINUE;
+		}
+
+		if (read_state_.responseContentLength_ != -1 && read_state_.receivedData_ + static_cast<int64_t>(size) > read_state_.responseContentLength_) {
+			size = static_cast<size_t>(read_state_.responseContentLength_ - read_state_.receivedData_);
+		}
+
+		int res = ProcessData(recv_buffer_.get(), size);
+		recv_buffer_.consume(recv_buffer_.size() - size);
+		return res;
+	}
+
+	return FZ_REPLY_INTERNALERROR;
 }
 
 int CHttpRequestOpData::OnReceive(bool repeatedProcessing)
@@ -439,7 +439,7 @@ int CHttpRequestOpData::OnReceive(bool repeatedProcessing)
 		if (repeatedProcessing) {
 			repeatedProcessing = false;
 		}
-		else {
+		else if (!read_state_.eof_) {
 			int error;
 			size_t const recv_size = 1024 * 64;
 			int read = controlSocket_.active_layer_->read(recv_buffer_.get(recv_size), recv_size, error);
@@ -450,16 +450,15 @@ int CHttpRequestOpData::OnReceive(bool repeatedProcessing)
 				}
 				return FZ_REPLY_WOULDBLOCK;
 			}
-			recv_buffer_.add(static_cast<size_t>(read));
-
-			controlSocket_.SetActive(CFileZillaEngine::recv);
+			else if (read) {
+				recv_buffer_.add(static_cast<size_t>(read));
+				controlSocket_.SetAlive();
+			}
 
 			read_state_.eof_ = read == 0;
 		}
 
 		while (!requests_.empty()) {
-			assert(!requests_.empty());
-
 			int res = ParseReceiveBuffer();
 			if (res == FZ_REPLY_WOULDBLOCK) {
 				return res;
@@ -470,17 +469,31 @@ int CHttpRequestOpData::OnReceive(bool repeatedProcessing)
 
 			if (res == FZ_REPLY_OK) {
 				log(logmsg::debug_info, L"Finished a response");
-				if (requests_.front() && requests_.front()->request().body_) {
-					requests_.front()->request().body_->set_handler(nullptr);
+				if (requests_.front()) {
+					if (requests_.front()->request().body_) {
+						requests_.front()->request().body_->set_handler(nullptr);
+					}
+					if (requests_.front()->response().writer_) {
+						requests_.front()->response().writer_->set_handler(nullptr);
+					}
 				}
-				requests_.pop_front();
-				--send_pos_;
 
-				bool const socket_done = read_state_.eof_ || !read_state_.keep_alive_;
+				requests_.pop_front();
+
+				bool premature{};
+				if (!send_pos_) {
+					premature = true;
+				}
+				else {
+					--send_pos_;
+				}
+
+				bool const socket_done = read_state_.eof_ || !read_state_.keep_alive_ || premature;
 				if (socket_done) {
+					controlSocket_.active_layer_->shutdown();
 					if (!recv_buffer_.empty()) {
-						log(logmsg::error, _("Malformed response: %s"), _("Server sent too much data."));
-						return FZ_REPLY_ERROR;
+						log(logmsg::debug_verbose, "Discarding %u bytes from receive buffer", recv_buffer_.size());
+						recv_buffer_.clear();
 					}
 
 					controlSocket_.ResetSocket();
@@ -576,8 +589,21 @@ int CHttpRequestOpData::ParseHeader()
 				return FZ_REPLY_ERROR;
 			}
 
-			response.code_ = (recv_buffer_[9] - '0') * 100 + (recv_buffer_[10] - '0') * 10 + recv_buffer_[11] - '0';
-			response.flags_ |= HttpResponse::flag_got_code;
+			unsigned int code = response.code_ = (recv_buffer_[9] - '0') * 100 + (recv_buffer_[10] - '0') * 10 + recv_buffer_[11] - '0';
+			if (code != 100) {
+				response.code_ = code;
+				response.flags_ |= HttpResponse::flag_got_code;
+			}
+
+			if (!send_pos_) {
+				if (response.success()) {
+					log(logmsg::error, _("Broken server, it claims to have processed a request before it got fully set"));
+					return FZ_REPLY_ERROR;
+				}
+				else {
+					log(logmsg::debug_info, "Premature error response");
+				}
+			}
 		}
 		else {
 			if (!i) {
@@ -628,11 +654,6 @@ int CHttpRequestOpData::ProcessCompleteHeader()
 	auto & srr = requests_.front();
 	auto & request = srr->request();
 	auto & response = srr->response();
-	if (response.code_ == 100) {
-		// 100 Continue header. Ignore it and start over.
-		response.reset();
-		return FZ_REPLY_CONTINUE;
-	}
 
 	response.flags_ |= HttpResponse::flag_got_header;
 	if (request.verb_ == "HEAD" || response.code_prohobits_body()) {
@@ -650,7 +671,7 @@ int CHttpRequestOpData::ProcessCompleteHeader()
 		log(logmsg::error, _("Malformed response header: %s"), _("Unknown transfer encoding"));
 		return FZ_REPLY_ERROR;
 	}
-	
+
 	auto retry = response.get_header("Retry-After");
 	if (response.code_ >= 400 && !retry.empty()) {
 		// TODO: Retry-After for redirects
@@ -704,26 +725,26 @@ int CHttpRequestOpData::ProcessCompleteHeader()
 
 		if (res == FZ_REPLY_CONTINUE) {
 			if (response.writer_) {
-				response.writer_->set_handler(&controlSocket_);
+				response.writer_->set_handler(this);
 			}
-
 		}
 		else {
 			if (res == FZ_REPLY_OK) {
-				if (!request.body_ || request.flags_ & HttpRequest::flag_sent_body) {
+				if (send_pos_) {
 					// Clear the pointer, we no longer need the request to finish, all needed information is in read_state_
+					if (request.body_) {
+						request.body_->set_handler(nullptr);
+					}
 					srr.reset();
+					res = FZ_REPLY_CONTINUE;
 				}
-				else {
-					response.flags_ |= HttpResponse::flag_ignore_body;
-				}
-				res = FZ_REPLY_CONTINUE;
 			}
 		}
 	}
 
 	if (res == FZ_REPLY_CONTINUE) {
 		if (!read_state_.responseContentLength_) {
+			read_state_.done_ = true;
 			return FinalizeResponseBody();
 		}
 	}
@@ -791,6 +812,7 @@ int CHttpRequestOpData::ParseChunkedData()
 			if (!i) {
 				// We're done
 				recv_buffer_.consume(2);
+				read_state_.done_ = true;
 				return FinalizeResponseBody();
 			}
 
@@ -885,6 +907,7 @@ int CHttpRequestOpData::ProcessData(unsigned char* data, size_t & remaining)
 	read_state_.receivedData_ += initial - remaining;
 
 	if (res == FZ_REPLY_CONTINUE && read_state_.receivedData_ == read_state_.responseContentLength_) {
+		read_state_.done_ = true;
 		res = FinalizeResponseBody();
 	}
 
@@ -914,10 +937,10 @@ int CHttpRequestOpData::Reset(int result)
 
 void CHttpRequestOpData::operator()(fz::event_base const& ev)
 {
-	fz::dispatch<read_ready_event>(ev, this, &CHttpRequestOpData::OnLocalData);
+	fz::dispatch<read_ready_event, write_ready_event, fz::timer_event>(ev, this, &CHttpRequestOpData::OnReaderReady, &CHttpRequestOpData::OnWriterReady, &CHttpRequestOpData::OnTimer);
 }
 
-void CHttpRequestOpData::OnLocalData(reader_base * r)
+void CHttpRequestOpData::OnReaderReady(reader_base * r)
 {
 	if (requests_.empty() || !requests_[send_pos_]) {
 		return;
@@ -932,3 +955,32 @@ void CHttpRequestOpData::OnLocalData(reader_base * r)
 		controlSocket_.SendNextCommand();
 	}
 }
+
+void CHttpRequestOpData::OnWriterReady(writer_base* writer)
+{
+	if (requests_.empty() || requests_.back()->response().writer_.get() != writer) {
+		log(logmsg::debug_warning, L"Stale writer event");
+		return;
+	}
+
+	int res = OnReceive(true);
+	if (res == FZ_REPLY_CONTINUE) {
+		controlSocket_.SendNextCommand();
+	}
+	else if (res != FZ_REPLY_WOULDBLOCK) {
+		controlSocket_.ResetOperation(res);
+	}
+}
+
+void CHttpRequestOpData::OnTimer(fz::timer_id)
+{
+#if FZ_WINDOWS
+	if (controlSocket_.socket_ && controlSocket_.socket_->is_connected()) {
+		int const ideal_send_buffer = controlSocket_.socket_->ideal_send_buffer_size();
+		if (ideal_send_buffer != -1) {
+			controlSocket_.socket_->set_buffer_sizes(-1, ideal_send_buffer);
+		}
+	}
+#endif
+}
+
