@@ -35,11 +35,12 @@ public:
 	void AddRequest(std::shared_ptr<HttpRequestResponseInterface> const& rr);
 
 	int OnReceive(bool repeatedProcessing);
-	int OnReceive(writer_base* writer);
 
 private:
 	virtual void operator()(fz::event_base const& ev) override;
-	void OnLocalData(reader_base * r);
+	void OnReaderReady(reader_base * r);
+	void OnWriterReady(writer_base * writer);
+	void OnTimer(fz::timer_id);
 
 	int ParseReceiveBuffer();
 	int ParseHeader();
@@ -77,12 +78,17 @@ private:
 
 		fz::nonowning_buffer writer_buffer_;
 
+		bool done_{};
 		bool keep_alive_{};
 		bool eof_{};
 	};
 	read_state read_state_;
 
 	uint64_t dataToSend_{};
+
+#if FZ_WINDOWS
+	fz::timer_id buffer_tuning_timer_{};
+#endif
 };
 
 #endif
