@@ -124,12 +124,14 @@ void CStorjControlSocket::OnStorjEvent(storj_message const& message)
 	case storjEvent::Status:
 		log_raw(logmsg::status, message.text[0]);
 		break;
+/* Unused
 	case storjEvent::Recv:
-		SetActive(CFileZillaEngine::recv);
+		RecordActivity(activity_logger::recv, fz::to_integral<uint64_t>(message.text[0]);
 		break;
 	case storjEvent::Send:
-		SetActive(CFileZillaEngine::send);
+		RecordActivity(activity_logger::send, fz::to_integral<uint64_t>(message.text[0]);
 		break;
+*/
 	case storjEvent::Listentry:
 		if (operations_.empty() || operations_.back()->opId != Command::list) {
 			log(logmsg::debug_warning, L"storjEvent::Listentry outside list operation, ignoring.");
@@ -149,9 +151,9 @@ void CStorjControlSocket::OnStorjEvent(storj_message const& message)
 			if (!operations_.empty() && operations_.back()->opId == Command::transfer) {
 				auto & data = static_cast<CStorjFileTransferOpData &>(*operations_.back());
 
-				SetActive(data.download() ? CFileZillaEngine::recv : CFileZillaEngine::send);
+				RecordActivity(data.download() ? activity_logger::recv : activity_logger::send, value);
 
-				bool tmp;
+				bool tmp{};
 				CTransferStatus status = engine_.transfer_status_.Get(tmp);
 				if (!status.empty() && !status.madeProgress) {
 					if (data.download()) {
@@ -166,6 +168,9 @@ void CStorjControlSocket::OnStorjEvent(storj_message const& message)
 					}
 				}
 			}
+			else {
+				SetAlive();
+			}
 
 			engine_.transfer_status_.Update(value);
 		}
