@@ -49,7 +49,8 @@ enum class option_flags
 	predefined_priority = 4, // If that option is given in fzdefaults.xml, it overrides any user option
 	platform = 8, // A non-portable platform specific option, nodes have platform attribute
 	numeric_clamp = 16, // For numeric options, fixup input and clamp to allowed value range. If not set, setting invalid values discards
-	sensitive_data = 32 // Flag to mark sensitive data. Can be used to clear private data even without domain knowledge.
+	sensitive_data = 32, // Flag to mark sensitive data. Can be used to clear private data even without domain knowledge.
+	product = 64 // Product-specific options in common code
 };
 inline bool operator&(option_flags lhs, option_flags rhs) {
 	return (static_cast<std::underlying_type_t<option_flags>>(lhs) & static_cast<std::underlying_type_t<option_flags>>(rhs)) != 0;
@@ -227,15 +228,28 @@ public:
 		unwatch_all(get_option_watcher_notifier(handler));
 	}
 
+	template<typename T>
+	uint64_t change_count(T opt)
+	{
+		return change_count(mapOption(opt));
+	}
+
 	struct option_value final
 	{
 		std::wstring str_;
 		std::unique_ptr<pugi::xml_document> xml_{};
+		uint64_t change_counter_{};
 		int v_{};
 		bool predefined_{};
 	};
 
 protected:
+	template<typename T>
+	void set(T opt, std::wstring_view const& value, bool predefined)
+	{
+		set(mapOption(opt), value, predefined);
+	}
+
 	void add_missing(fz::scoped_write_lock & l);
 
 	int get_int(optionsIndex opt);
@@ -257,6 +271,8 @@ protected:
 	void unwatch(optionsIndex opt, std::tuple<void*, watcher_notifier> handler);
 	void unwatch_all(std::tuple<void*, watcher_notifier> handler);
 
+	uint64_t change_count(optionsIndex opt);
+
 	void set_default_value(optionsIndex opt);
 
 	fz::rwmutex mtx_;
