@@ -47,7 +47,9 @@ enum ServerProtocol
 
 	RACKSPACE,
 
-	MAX_VALUE = RACKSPACE
+	STORJ_GRANT,
+
+	MAX_VALUE = STORJ_GRANT
 };
 
 enum ServerType
@@ -101,11 +103,9 @@ enum class ProtocolFeature
 	EnterCommand,
 	DirectoryRename,
 	PostLoginCommands,
-	S3Lifecycle,
 	RecursiveDelete,
 	ServerAssignedHome,
 	TemporaryUrl,
-	S3Sse,
 	Security, // Encryption, integrity protection and authentication
 	UnixChmod
 };
@@ -238,6 +238,7 @@ enum class LogonType
 	interactive,
 	account,
 	key,
+	profile,
 
 	count
 };
@@ -253,7 +254,6 @@ namespace ParameterSection {
 		user,
 		credentials,
 		extra,
-		custom,
 
 		section_count
 	};
@@ -267,7 +267,9 @@ struct ParameterTraits
 
 	enum flags : unsigned char {
 		optional = 0x01,
-		numeric = 0x02
+		numeric = 0x02,
+		content_transparent = 0x04, // Parameter used for any purpose that does not affect site content
+		custom = 0x08 // Hint for the UI to not generate input fields, trait needs custom handling
 	};
 	unsigned char flags_;
 	std::wstring default_;
@@ -305,6 +307,7 @@ public:
 	std::map<std::string, std::wstring, std::less<>> const& GetExtraParameters() const;
 	bool HasExtraParameter(std::string_view const& name) const;
 	void SetExtraParameter(ServerProtocol protocol, std::string_view const& name, std::wstring const& value);
+	void SetExtraParameters(ServerProtocol protocol, std::map<std::string, std::wstring, std::less<>> const& extraParameters);
 
 protected:
 	std::wstring password_;
