@@ -6,7 +6,7 @@
 #include "LocalListView.h"
 #include "queue.h"
 #include "filezillaapp.h"
-#include "filter.h"
+#include "filter_manager.h"
 #include "file_utils.h"
 #include "infotext.h"
 #include "inputdialog.h"
@@ -143,7 +143,7 @@ public:
 		return true;
 	}
 
-	virtual int DisplayDropHighlight(wxPoint point)
+	virtual int DisplayDropHighlight(wxPoint const& point) override
 	{
 		DoDisplayDropHighlight(point);
 		return -1;
@@ -272,8 +272,8 @@ BEGIN_EVENT_TABLE(CLocalListView, CFileListCtrl<CLocalFileData>)
 	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CLocalListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
-CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue)
-	: CFileListCtrl<CLocalFileData>(pParent, pQueue),
+CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue, COptionsBase & options)
+	: CFileListCtrl<CLocalFileData>(pParent, pQueue, options),
 	CStateEventHandler(state),
 	m_parentView(pParent)
 {
@@ -309,7 +309,7 @@ CLocalListView::CLocalListView(CView* pParent, CState& state, CQueueView *pQueue
 CLocalListView::~CLocalListView()
 {
 	wxString str = wxString::Format(_T("%d %d"), m_sortDirection, m_sortColumn);
-	COptions::Get()->set(OPTION_LOCALFILELIST_SORTORDER, str.ToStdWstring());
+	options_.set(OPTION_LOCALFILELIST_SORTORDER, str.ToStdWstring());
 
 #ifdef __WXMSW__
 	volumeEnumeratorThread_.reset();
@@ -392,7 +392,7 @@ regular_dir:
 
 		auto result = local_filesys.begin_find_files(fz::to_native(m_dir.GetPath()), false);
 		if (!result) {
-			
+
 			if (result.error_ == fz::result::noperm) {
 				SetInfoText(_("You do not have permission to list this directory"));
 			}
@@ -549,7 +549,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 	}
 
 	if (data->dir) {
-		const int action = COptions::Get()->get_int(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
+		const int action = options_.get_int(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
 		if (action == 3) {
 			// No action
 			wxBell();
@@ -581,7 +581,7 @@ void CLocalListView::OnItemActivated(wxListEvent &event)
 		return;
 	}
 
-	const int action = COptions::Get()->get_int(OPTION_DOUBLECLICK_ACTION_FILE);
+	const int action = options_.get_int(OPTION_DOUBLECLICK_ACTION_FILE);
 	if (action == 3) {
 		// No action
 		wxBell();
@@ -763,37 +763,37 @@ bool CLocalListView::IsItemValid(unsigned int item) const
 	return true;
 }
 
-std::unique_ptr<CFileListCtrlSortBase> CLocalListView::GetSortComparisonObject()
+void CLocalListView::UpdateSortComparisonObject()
 {
 	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
 	NameSortMode nameSortMode = GetNameSortMode();
 
 	if (!m_sortDirection) {
 		if (m_sortColumn == 1) {
-			return std::make_unique<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 2) {
-			return std::make_unique<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 3) {
-			return std::make_unique<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else {
-			return std::make_unique<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 	}
 	else {
 		if (m_sortColumn == 1) {
-			return std::make_unique<CReverseSort<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortSize<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 2) {
-			return std::make_unique<CReverseSort<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortType<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 3) {
-			return std::make_unique<CReverseSort<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortTime<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else {
-			return std::make_unique<CReverseSort<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortName<std::vector<CLocalFileData>, CLocalFileData>, CLocalFileData>>(m_fileData, m_fileData, dirSortMode, nameSortMode, this);
 		}
 	}
 }
@@ -814,23 +814,23 @@ void CLocalListView::OnContextMenu(wxContextMenuEvent& event)
 	item->SetBitmap(wxArtProvider::GetBitmap(_T("ART_UPLOADADD"), wxART_MENU));
 	menu.Append(item);
 	menu.Append(XRCID("ID_ENTER"), _("E&nter directory"), _("Enter selected directory"));
-	
+
 	menu.AppendSeparator();
-	menu.Append(XRCID("ID_OPEN"), _("&Open"), _("Open the file."));		
+	menu.Append(XRCID("ID_OPEN"), _("&Open"), _("Open the file."));
 	menu.Append(XRCID("ID_EDIT"), _("&Edit"), _("Edit the file with the configured editor and upload changes to the server."));
-		
+
 	menu.AppendSeparator();
-	menu.Append(XRCID("ID_MKDIR"), _("&Create directory"), _("Create a new subdirectory in the current directory"));		
+	menu.Append(XRCID("ID_MKDIR"), _("&Create directory"), _("Create a new subdirectory in the current directory"));
 	menu.Append(XRCID("ID_MKDIR_CHGDIR"), _("Create director&y and enter it"), _("Create a new subdirectory in the current directory and change into it"));
 	menu.Append(XRCID("ID_CONTEXT_REFRESH"), _("Re&fresh"));
-		
+
 	menu.AppendSeparator();
 	menu.Append(XRCID("ID_DELETE"),_("&Delete"), _("Delete selected files and directories"));
 	menu.Append(XRCID("ID_RENAME"), _("&Rename"), _("Rename selected files and directories"));
-		
+
 	const bool connected = m_state.IsRemoteConnected();
 	if (!connected) {
-		menu.Enable(XRCID("ID_EDIT"), COptions::Get()->get_int(OPTION_EDIT_TRACK_LOCAL) == 0);
+		menu.Enable(XRCID("ID_EDIT"), options_.get_int(OPTION_EDIT_TRACK_LOCAL) == 0);
 		menu.Enable(XRCID("ID_UPLOAD"), false);
 		menu.Enable(XRCID("ID_ADDTOQUEUE"), false);
 	}
@@ -953,7 +953,7 @@ void CLocalListView::OnMenuUpload(wxCommandEvent& event)
 	if (!root.empty()) {
 		recursiveOperation->AddRecursionRoot(std::move(root));
 		CFilterManager filter;
-		recursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !queue_only);
+		recursiveOperation->StartRecursiveOperation(recursive_operation::recursive_transfer, filter.GetActiveFilters(), !queue_only);
 	}
 }
 
@@ -1184,7 +1184,7 @@ bool CLocalListView::OnAcceptRename(const wxListEvent& event)
 void CLocalListView::ApplyCurrentFilter()
 {
 	CStateFilterManager const& filter = m_state.GetStateFilterManager();
-	
+
 	if (!filter.HasSameLocalAndRemoteFilters() && IsComparing()) {
 		ExitComparisonMode();
 	}
@@ -1406,10 +1406,10 @@ void CLocalListView::OnStateChange(t_statechange_notifications notification, std
 	}
 	else if (notification == STATECHANGE_SERVER) {
 		if (m_windowTinter) {
-			m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+			m_windowTinter->SetBackgroundTint(site_colour_to_wx(m_state.GetSite().m_colour));
 		}
 		if (m_pInfoText) {
-			m_pInfoText->SetBackgroundTint(m_state.GetSite().m_colour);
+			m_pInfoText->SetBackgroundTint(site_colour_to_wx(m_state.GetSite().m_colour));
 		}
 	}
 	else {
@@ -1436,7 +1436,7 @@ void CLocalListView::SetInfoText(wxString const& text)
 
 void CLocalListView::OnBeginDrag(wxListEvent&)
 {
-	if (COptions::Get()->get_int(OPTION_DND_DISABLED) != 0) {
+	if (options_.get_int(OPTION_DND_DISABLED) != 0) {
 		return;
 	}
 
@@ -1629,7 +1629,7 @@ void CLocalListView::RefreshFile(std::wstring const& file)
 	if (m_hasParent) {
 		++start;
 	}
-	std::unique_ptr<CFileListCtrlSortBase> compare = GetSortComparisonObject();
+	auto& compare = GetSortComparisonObject();
 	std::vector<unsigned int>::iterator insertPos = std::lower_bound(start, m_indexMapping.end(), index, SortPredicate(compare));
 
 	const int item = insertPos - m_indexMapping.begin();
@@ -1806,7 +1806,7 @@ void CLocalListView::OnMenuEdit(wxCommandEvent&)
 	CServerPath path;
 
 	if (!m_state.GetSite()) {
-		if (COptions::Get()->get_int(OPTION_EDIT_TRACK_LOCAL)) {
+		if (options_.get_int(OPTION_EDIT_TRACK_LOCAL)) {
 			wxMessageBoxEx(_("Cannot edit file, not connected to any server."), _("Editing failed"), wxICON_EXCLAMATION);
 			return;
 		}
