@@ -4,7 +4,7 @@
 #include "dragdropmanager.h"
 #include "drop_target_ex.h"
 #include "filezillaapp.h"
-#include "filter.h"
+#include "filter_manager.h"
 #include "file_utils.h"
 #include "graphics.h"
 #include "inputdialog.h"
@@ -46,15 +46,6 @@ public:
 	{
 	}
 
-	void ClearDropHighlight()
-	{
-		const wxTreeItemId dropHighlight = m_pLocalTreeView->m_dropHighlight;
-		if (dropHighlight != wxTreeItemId()) {
-			m_pLocalTreeView->SetItemDropHighlight(dropHighlight, false);
-			m_pLocalTreeView->m_dropHighlight = wxTreeItemId();
-		}
-	}
-
 	std::wstring GetDirFromItem(wxTreeItemId const& item)
 	{
 		std::wstring const dir = m_pLocalTreeView->GetDirFromItem(item);
@@ -144,7 +135,7 @@ public:
 			return false;
 		}
 
-		ClearDropHighlight();
+		m_pLocalTreeView->ClearDropHighlight();
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
 		if (!hit) {
@@ -159,29 +150,24 @@ public:
 		return true;
 	}
 
-	wxTreeItemId DisplayDropHighlight(wxPoint point)
+	wxTreeItemId DisplayDropHighlight(wxPoint const& point) override
 	{
 		wxTreeItemId hit = GetHit(point);
 		if (!hit) {
-			ClearDropHighlight();
+			m_pLocalTreeView->ClearDropHighlight();
 			return hit;
 		}
 
 		std::wstring dir = GetDirFromItem(hit);
 
 		if (dir.empty()) {
-			ClearDropHighlight();
+			m_pLocalTreeView->ClearDropHighlight();
 			return wxTreeItemId();
 		}
-
-		const wxTreeItemId dropHighlight = m_pLocalTreeView->m_dropHighlight;
-		if (dropHighlight != wxTreeItemId()) {
-			m_pLocalTreeView->SetItemDropHighlight(dropHighlight, false);
+		else {
+			m_pLocalTreeView->DisplayDropHighlight(hit);
 		}
 
-		m_pLocalTreeView->SetItemDropHighlight(hit, true);
-		m_pLocalTreeView->m_dropHighlight = hit;
-
 		return hit;
 	}
 
@@ -193,7 +179,7 @@ public:
 			def == wxDragNone ||
 			def == wxDragCancel)
 		{
-			ClearDropHighlight();
+			m_pLocalTreeView->ClearDropHighlight();
 			return def;
 		}
 
@@ -212,7 +198,7 @@ public:
 	virtual void OnLeave()
 	{
 		CScrollableDropTarget<wxTreeCtrlEx>::OnLeave();
-		ClearDropHighlight();
+		m_pLocalTreeView->ClearDropHighlight();
 	}
 
 	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def)
@@ -641,26 +627,28 @@ std::wstring CLocalTreeView::GetDirFromItem(wxTreeItemId item)
 	while (item) {
 #ifdef __WXMSW__
 		if (item == m_desktop) {
-			wxChar path[MAX_PATH + 1];
-			if (SHGetFolderPath(0, CSIDL_DESKTOPDIRECTORY, 0, SHGFP_TYPE_CURRENT, path) != S_OK) {
-				if (SHGetFolderPath(0, CSIDL_DESKTOP, 0, SHGFP_TYPE_CURRENT, path) != S_OK) {
-					wxMessageBoxEx(_("Failed to get desktop path"));
-					return L"/";
-				}
+			wchar_t* out{};
+			if (SHGetKnownFolderPath(FOLDERID_Desktop, 0, 0, &out) != S_OK) {
+				wxMessageBoxEx(_("Failed to get desktop path"));
+				return L"/";
 			}
-			dir = path;
+			dir = out;
+			CoTaskMemFree(out);
+
 			if (dir.empty() || dir.back() != separator) {
 				dir += separator;
 			}
 			return dir;
 		}
 		else if (item == m_documents) {
-			wxChar path[MAX_PATH + 1];
-			if (SHGetFolderPath(0, CSIDL_PERSONAL, 0, SHGFP_TYPE_CURRENT, path) != S_OK) {
+			wchar_t* out{};
+			if (SHGetKnownFolderPath(FOLDERID_Documents, 0, 0, &out) != S_OK) {
 				wxMessageBoxEx(_("Failed to get 'My Documents' path"));
 				return L"/";
 			}
-			dir = path;
+			dir = out;
+			CoTaskMemFree(out);
+
 			if (dir.empty() || dir.back() != separator) {
 				dir += separator;
 			}
@@ -924,7 +912,7 @@ void CLocalTreeView::OnStateChange(t_statechange_notifications notification, std
 		SetDir(m_state.GetLocalDir().GetPath());
 	}
 	else if (notification == STATECHANGE_SERVER) {
-		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+		m_windowTinter->SetBackgroundTint(site_colour_to_wx(m_state.GetSite().m_colour));
 	}
 	else {
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
@@ -1184,7 +1172,7 @@ void CLocalTreeView::OnMenuUpload(wxCommandEvent& event)
 	bool const queue_only = event.GetId() == XRCID("ID_ADDTOQUEUE");
 
 	CFilterManager filter;
-	recursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !queue_only);
+	recursiveOperation->StartRecursiveOperation(recursive_operation::recursive_transfer, filter.GetActiveFilters(), !queue_only);
 }
 
 // Create a new Directory
@@ -1570,7 +1558,7 @@ wxTreeItemId CLocalTreeView::AddDrive(wxChar letter)
 	drive += _T(":");
 
 	long drivesToHide = CVolumeDescriptionEnumeratorThread::GetDrivesToHide();
-	if( CVolumeDescriptionEnumeratorThread::IsHidden(drive.wc_str(), drivesToHide) ) {
+	if ( CVolumeDescriptionEnumeratorThread::IsHidden(drive.wc_str(), drivesToHide) ) {
 		return wxTreeItemId();
 	}
 
