@@ -17,10 +17,9 @@
 #include "export.h"
 #include "filelist_statusbar.h"
 #include "filezillaapp.h"
-#include "filter.h"
+#include "filter_manager.h"
 #include "import.h"
 #include "inputdialog.h"
-#include "led.h"
 #include "list_search_panel.h"
 #include "local_recursive_operation.h"
 #include "LocalListView.h"
@@ -46,12 +45,12 @@
 #include "themeprovider.h"
 #include "toolbar.h"
 #include "update_dialog.h"
-#include "updater.h"
 #include "view.h"
 #include "viewheader.h"
 #include "welcome_dialog.h"
 #include "window_state_manager.h"
 #include "../include/version.h"
+#include "verifycertdialog.h"
 
 #if FZ_MANUALUPDATECHECK
 #include "overlay.h"
@@ -127,7 +126,6 @@ bool HandleKeyboardCommand(wxCommandEvent& event, wxWindow& parent)
 BEGIN_EVENT_TABLE(CMainFrame, wxNavigationEnabled<wxFrame>)
 	EVT_SIZE(CMainFrame::OnSize)
 	EVT_MENU(wxID_ANY, CMainFrame::OnMenuHandler)
-	EVT_COMMAND(wxID_ANY, fzEVT_UPDATE_LED_TOOLTIP, CMainFrame::OnUpdateLedTooltip)
 	EVT_TOOL(XRCID("ID_TOOLBAR_DISCONNECT"), CMainFrame::OnDisconnect)
 	EVT_MENU(XRCID("ID_MENU_SERVER_DISCONNECT"), CMainFrame::OnDisconnect)
 	EVT_TOOL(XRCID("ID_TOOLBAR_CANCEL"), CMainFrame::OnCancel)
@@ -259,9 +257,9 @@ protected:
 
 #if FZ_MANUALUPDATECHECK
 namespace {
-void ShowOverlay(std::wstring const& data, wxTopLevelWindow* parent, wxWindow* anchor, wxPoint const& offset)
+void ShowOverlay(COptions& options, std::wstring const& data, wxTopLevelWindow* parent, wxWindow* anchor, wxPoint const& offset)
 {
-	if (data.empty() || CBuildInfo::GetBuildType() != _T("official") || COptions::Get()->get_bool(OPTION_DISABLE_UPDATE_FOOTER)) {
+	if (data.empty() || CBuildInfo::GetBuildType() != _T("official") || options.get_bool(OPTION_DISABLE_UPDATE_FOOTER)) {
 		return;
 	}
 
@@ -275,7 +273,7 @@ void ShowOverlay(std::wstring const& data, wxTopLevelWindow* parent, wxWindow* a
 	}
 
 	int const id = fz::to_integral<int>(tokens[1]);
-	if (COptions::Get()->get_int(OPTION_SHOWN_OVERLAY) >= id) {
+	if (options.get_int(OPTION_SHOWN_OVERLAY) >= id) {
 		return;
 	}
 
@@ -344,8 +342,8 @@ void ShowOverlay(std::wstring const& data, wxTopLevelWindow* parent, wxWindow* a
 	outer->AddSpacer(0);
 
 	auto dismiss = new wxButton(p, nullID, L"Dismiss");
-	dismiss->Bind(wxEVT_BUTTON, [p, id](wxCommandEvent&) {
-		COptions::Get()->set(OPTION_SHOWN_OVERLAY, id);
+	dismiss->Bind(wxEVT_BUTTON, [&options, p, id](wxCommandEvent&) {
+		options.set(OPTION_SHOWN_OVERLAY, id);
 		p->Destroy();
 	});
 
@@ -358,13 +356,12 @@ void ShowOverlay(std::wstring const& data, wxTopLevelWindow* parent, wxWindow* a
 }
 #endif
 
-CMainFrame::CMainFrame()
+CMainFrame::CMainFrame(COptions& options)
 	: COptionChangeEventHandler(this)
-	, m_engineContext(*COptions::Get(), CustomEncodingConverter::Get())
+	, options_(options)
+	, m_engineContext(options, CustomEncodingConverter::Get())
 	, m_comparisonToggleAcceleratorId(wxNewId())
 {
-	m_pActivityLed[0] = m_pActivityLed[1] = 0;
-
 	wxGetApp().AddStartupProfileRecord("CMainFrame::CMainFrame");
 	wxRect screen_size = CWindowStateManager::GetScreenDimensions();
 
@@ -397,32 +394,27 @@ CMainFrame::CMainFrame()
 	// so that contextchange events can be processed in the right order.
 	m_pContextControl = new CContextControl(*this);
 
-	m_pStatusBar = new CStatusBar(this);
+	m_pStatusBar = new CStatusBar(this, m_engineContext.GetActivityLogger(), options_);
 	if (m_pStatusBar) {
-		m_pActivityLed[0] = new CLed(m_pStatusBar, 0);
-		m_pActivityLed[1] = new CLed(m_pStatusBar, 1);
-
-		m_pStatusBar->AddField(-1, widget_led_recv, m_pActivityLed[1]);
-		m_pStatusBar->AddField(-1, widget_led_send, m_pActivityLed[0]);
-
 		SetStatusBar(m_pStatusBar);
 	}
 
 	m_closeEventTimer.SetOwner(this);
 
 	if (CFilterManager::HasActiveFilters(true)) {
-		if (COptions::Get()->get_int(OPTION_FILTERTOGGLESTATE)) {
+		if (options_.get_int(OPTION_FILTERTOGGLESTATE)) {
 			CFilterManager::ToggleFilters();
 		}
 	}
 
 	CreateMenus();
 	CreateMainToolBar();
-	if (COptions::Get()->get_int(OPTION_SHOW_QUICKCONNECT)) {
+	if (options_.get_int(OPTION_SHOW_QUICKCONNECT)) {
 		CreateQuickconnectBar();
 	}
 
-	m_pAsyncRequestQueue = new CAsyncRequestQueue(this);
+	cert_store_ = std::make_unique<CertStore>();
+	async_request_queue_ = std::make_unique<CAsyncRequestQueue>(this, options_, *cert_store_);
 
 #ifdef __WXMSW__
 	long style = wxSP_NOBORDER | wxSP_LIVE_UPDATE;
@@ -441,11 +433,11 @@ CMainFrame::CMainFrame()
 	m_pBottomSplitter->SetMinimumPaneSize(20, 70);
 	m_pBottomSplitter->SetSashGravity(1.0);
 
-	const int message_log_position = COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION);
+	const int message_log_position = options_.get_int(OPTION_MESSAGELOG_POSITION);
 	m_pQueueLogSplitter = new CSplitterWindowEx(m_pBottomSplitter, -1, wxDefaultPosition, wxDefaultSize, wxSP_NOBORDER | wxSP_LIVE_UPDATE);
 	m_pQueueLogSplitter->SetMinimumPaneSize(50, 250);
 	m_pQueueLogSplitter->SetSashGravity(0.5);
-	m_pQueuePane = new CQueue(m_pQueueLogSplitter, this, m_pAsyncRequestQueue);
+	m_pQueuePane = new CQueue(m_pQueueLogSplitter, this, async_request_queue_.get(), *cert_store_);
 
 	if (message_log_position == 1) {
 		m_pStatusView = new CStatusView(m_pQueueLogSplitter, -1);
@@ -465,8 +457,8 @@ CMainFrame::CMainFrame()
 	switch (message_log_position) {
 	case 1:
 		m_pTopSplitter->Initialize(m_pBottomSplitter);
-		if (COptions::Get()->get_int(OPTION_SHOW_MESSAGELOG)) {
-			if (COptions::Get()->get_int(OPTION_SHOW_QUEUE)) {
+		if (options_.get_int(OPTION_SHOW_MESSAGELOG)) {
+			if (options_.get_int(OPTION_SHOW_QUEUE)) {
 				m_pQueueLogSplitter->SplitVertically(m_pQueuePane, m_pStatusView);
 			}
 			else {
@@ -475,7 +467,7 @@ CMainFrame::CMainFrame()
 			}
 		}
 		else {
-			if (COptions::Get()->get_int(OPTION_SHOW_QUEUE)) {
+			if (options_.get_int(OPTION_SHOW_QUEUE)) {
 				m_pStatusView->Hide();
 				m_pQueueLogSplitter->Initialize(m_pQueuePane);
 			}
@@ -488,7 +480,7 @@ CMainFrame::CMainFrame()
 		break;
 	case 2:
 		m_pTopSplitter->Initialize(m_pBottomSplitter);
-		if (COptions::Get()->get_int(OPTION_SHOW_QUEUE)) {
+		if (options_.get_int(OPTION_SHOW_QUEUE)) {
 			m_pQueueLogSplitter->Initialize(m_pQueuePane);
 		}
 		else {
@@ -498,14 +490,14 @@ CMainFrame::CMainFrame()
 		m_pQueuePane->AddPage(m_pStatusView, _("Message log"));
 		break;
 	default:
-		if (COptions::Get()->get_int(OPTION_SHOW_QUEUE)) {
+		if (options_.get_int(OPTION_SHOW_QUEUE)) {
 			m_pQueueLogSplitter->Initialize(m_pQueuePane);
 		}
 		else {
 			m_pQueuePane->Hide();
 			m_pQueueLogSplitter->Hide();
 		}
-		if (COptions::Get()->get_int(OPTION_SHOW_MESSAGELOG)) {
+		if (options_.get_int(OPTION_SHOW_MESSAGELOG)) {
 			m_pTopSplitter->SplitHorizontally(m_pStatusView, m_pBottomSplitter);
 		}
 		else {
@@ -545,16 +537,20 @@ CMainFrame::CMainFrame()
 
 	FixTabOrder();
 
-	COptions::Get()->watch(OPTION_ICONS_THEME, this);
-	COptions::Get()->watch(OPTION_ICONS_SCALE, this);
-	COptions::Get()->watch(OPTION_MESSAGELOG_POSITION, this);
-	COptions::Get()->watch(OPTION_FILEPANE_LAYOUT, this);
-	COptions::Get()->watch(OPTION_FILEPANE_SWAP, this);
+	options_.watch(OPTION_ICONS_THEME, this);
+	options_.watch(OPTION_ICONS_SCALE, this);
+	options_.watch(OPTION_MESSAGELOG_POSITION, this);
+	options_.watch(OPTION_FILEPANE_LAYOUT, this);
+	options_.watch(OPTION_FILEPANE_SWAP, this);
+
+	// Delay call to PostInitialize, otherwise macOS gets really confused about the right mouse button...
+	startupTimer_.SetOwner(this);
+	startupTimer_.Start(1, true);
 }
 
 CMainFrame::~CMainFrame()
 {
-	COptions::Get()->unwatch_all(this);
+	options_.unwatch_all(this);
 
 	CPowerManagement::Destroy();
 
@@ -564,7 +560,7 @@ CMainFrame::~CMainFrame()
 	m_pContextControl = 0;
 
 	CContextManager::Get()->DestroyAllStates();
-	delete m_pAsyncRequestQueue;
+	async_request_queue_.reset();
 #if FZ_MANUALUPDATECHECK
 	delete m_pUpdater;
 #endif
@@ -622,22 +618,15 @@ void CMainFrame::OnSize(wxSizeEvent &event)
 #endif
 }
 
-bool CMainFrame::CreateMenus()
+void CMainFrame::CreateMenus()
 {
 	wxGetApp().AddStartupProfileRecord("CMainFrame::CreateMenus");
 	CMenuBar* old = m_pMenuBar;
 
-	m_pMenuBar = CMenuBar::Load(this);
-
-	if (!m_pMenuBar) {
-		m_pMenuBar = old;
-		return false;
-	}
+	m_pMenuBar = new CMenuBar(*this, options_);
 
 	SetMenuBar(m_pMenuBar);
 	delete old;
-
-	return true;
 }
 
 void CMainFrame::CreateQuickconnectBar()
@@ -645,19 +634,14 @@ void CMainFrame::CreateQuickconnectBar()
 	wxGetApp().AddStartupProfileRecord("CMainFrame::CreateQuickconnectBar");
 	delete m_pQuickconnectBar;
 
-	m_pQuickconnectBar = new CQuickconnectBar();
-	if (!m_pQuickconnectBar->Create(this)) {
-		delete m_pQuickconnectBar;
-		m_pQuickconnectBar = 0;
-	}
-	else {
-		wxSize clientSize = GetClientSize();
-		if (m_pTopSplitter) {
-			wxSize panelSize = m_pQuickconnectBar->GetSize();
-			m_pTopSplitter->SetSize(-1, panelSize.GetHeight(), -1, clientSize.GetHeight() - panelSize.GetHeight(), wxSIZE_USE_EXISTING);
-		}
-		m_pQuickconnectBar->SetSize(0, 0, clientSize.GetWidth(), -1);
+	m_pQuickconnectBar = new CQuickconnectBar(*this);
+
+	wxSize clientSize = GetClientSize();
+	if (m_pTopSplitter) {
+		wxSize panelSize = m_pQuickconnectBar->GetSize();
+		m_pTopSplitter->SetSize(-1, panelSize.GetHeight(), -1, clientSize.GetHeight() - panelSize.GetHeight(), wxSIZE_USE_EXISTING);
 	}
+	m_pQuickconnectBar->SetSize(0, 0, clientSize.GetWidth(), -1);
 }
 
 void CMainFrame::OnMenuHandler(wxCommandEvent &event)
@@ -723,7 +707,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		OnMenuEditSettings(event);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_EDIT_NETCONFWIZARD")) {
-		CNetConfWizard wizard(this, COptions::Get(), m_engineContext);
+		CNetConfWizard wizard(this, &options_, m_engineContext);
 		wizard.Load();
 		wizard.Run();
 	}
@@ -743,9 +727,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 	else if (event.GetId() == XRCID("ID_CLEAR_UPDATER")) {
 #if FZ_MANUALUPDATECHECK
 		if (m_pUpdater) {
-			COptions::Get()->set(OPTION_UPDATECHECK_LASTDATE, std::wstring());
-			COptions::Get()->set(OPTION_UPDATECHECK_NEWVERSION, std::wstring());
-			m_pUpdater->Init();
+			m_pUpdater->Reset();
 		}
 #endif
 	}
@@ -770,7 +752,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		}
 	}
 	else if (event.GetId() == XRCID("ID_MENU_SERVER_VIEWHIDDEN")) {
-		bool showHidden = COptions::Get()->get_int(OPTION_VIEW_HIDDEN_FILES) ? 0 : 1;
+		bool showHidden = options_.get_int(OPTION_VIEW_HIDDEN_FILES) ? 0 : 1;
 		if (showHidden) {
 			CConditionalDialog dlg(this, CConditionalDialog::viewhidden, CConditionalDialog::ok, false);
 			dlg.SetTitle(_("Force showing hidden files"));
@@ -782,7 +764,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			(void)dlg.Run();
 		}
 
-		COptions::Get()->set(OPTION_VIEW_HIDDEN_FILES, showHidden ? 1 : 0);
+		options_.set(OPTION_VIEW_HIDDEN_FILES, showHidden ? 1 : 0);
 		const std::vector<CState*> *pStates = CContextManager::Get()->GetAllStates();
 		for (auto & pState : *pStates) {
 			CServerPath path = pState->GetRemotePath();
@@ -804,13 +786,13 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		dlg.ShowModal();
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_TYPE_AUTO")) {
-		COptions::Get()->set(OPTION_ASCIIBINARY, 0);
+		options_.set(OPTION_ASCIIBINARY, 0);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_TYPE_ASCII")) {
-		COptions::Get()->set(OPTION_ASCIIBINARY, 1);
+		options_.set(OPTION_ASCIIBINARY, 1);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_TYPE_BINARY")) {
-		COptions::Get()->set(OPTION_ASCIIBINARY, 2);
+		options_.set(OPTION_ASCIIBINARY, 2);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_PRESERVETIMES")) {
 		if (event.IsChecked()) {
@@ -819,7 +801,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			dlg.AddText(_("Please note that preserving timestamps on uploads on FTP, FTPS and FTPES servers only works if they support the MFMT command."));
 			dlg.Run();
 		}
-		COptions::Get()->set(OPTION_PRESERVE_TIMESTAMPS, event.IsChecked() ? 1 : 0);
+		options_.set(OPTION_PRESERVE_TIMESTAMPS, event.IsChecked() ? 1 : 0);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_PROCESSQUEUE")) {
 		if (m_pQueueView) {
@@ -836,16 +818,16 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		else {
 			url += _T("bugreport");
 		}
-		wxString version = GetFileZillaVersion();
-		if (version != _T("custom build")) {
+		auto const version = GetFileZillaVersion();
+		if (!version.empty()) {
 			url += _T("&version=");
 			// We need to urlencode version number
 
 			// Unbelievable, but wxWidgets does not have any method
 			// to urlencode strings.
 			// Do a crude approach: Drop everything unexpected...
-			for (unsigned int i = 0; i < version.Len(); i++) {
-				wxChar c = version.GetChar(i);
+			for (size_t i = 0; i < version.size(); ++i) {
+				auto c = version[i];
 				if ((c >= '0' && c <= '9') ||
 					(c >= 'a' && c <= 'z') ||
 					(c >= 'A' && c <= 'Z') ||
@@ -859,8 +841,8 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		wxLaunchDefaultBrowser(url);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_VIEW_FILELISTSTATUSBAR")) {
-		bool show = COptions::Get()->get_int(OPTION_FILELIST_STATUSBAR) == 0;
-		COptions::Get()->set(OPTION_FILELIST_STATUSBAR, show ? 1 : 0);
+		bool show = options_.get_int(OPTION_FILELIST_STATUSBAR) == 0;
+		options_.set(OPTION_FILELIST_STATUSBAR, show ? 1 : 0);
 		CContextControl::_context_controls* controls = m_pContextControl ? m_pContextControl->GetCurrentControls() : 0;
 		if (controls && controls->pLocalListViewPanel) {
 			wxStatusBar* pStatusBar = controls->pLocalListViewPanel->GetStatusBar();
@@ -889,7 +871,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			wxSize clientSize = GetClientSize();
 			m_pTopSplitter->SetSize(0, 0, clientSize.GetWidth(), clientSize.GetHeight());
 		}
-		COptions::Get()->set(OPTION_SHOW_QUICKCONNECT, m_pQuickconnectBar != 0);
+		options_.set(OPTION_SHOW_QUICKCONNECT, m_pQuickconnectBar != 0);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_MANUAL")) {
 		CState* pState = CContextManager::Get()->GetCurrentContext();
@@ -897,7 +879,7 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			wxBell();
 			return;
 		}
-		CManualTransfer dlg(m_pQueueView);
+		CManualTransfer dlg(options_, m_pQueueView);
 		dlg.Run(this, pState);
 	}
 	else if (event.GetId() == XRCID("ID_BOOKMARK_ADD") || event.GetId() == XRCID("ID_BOOKMARK_MANAGE")) {
@@ -939,20 +921,20 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 		}
 	}
 	else if (event.GetId() == XRCID("ID_MENU_HELP_WELCOME")) {
-		CWelcomeDialog dlg;
-		dlg.Run(this, true);
+		CWelcomeDialog dlg(options_, this);
+		dlg.Run(true);
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_SPEEDLIMITS_ENABLE")) {
-		bool enable = COptions::Get()->get_int(OPTION_SPEEDLIMIT_ENABLE) == 0;
+		bool enable = options_.get_int(OPTION_SPEEDLIMIT_ENABLE) == 0;
 
-		const int downloadLimit = COptions::Get()->get_int(OPTION_SPEEDLIMIT_INBOUND);
-		const int uploadLimit = COptions::Get()->get_int(OPTION_SPEEDLIMIT_OUTBOUND);
+		const int downloadLimit = options_.get_int(OPTION_SPEEDLIMIT_INBOUND);
+		const int uploadLimit = options_.get_int(OPTION_SPEEDLIMIT_OUTBOUND);
 		if (enable && !downloadLimit && !uploadLimit) {
 			CSpeedLimitsDialog dlg;
 			dlg.Run(this);
 		}
 		else {
-			COptions::Get()->set(OPTION_SPEEDLIMIT_ENABLE, enable ? 1 : 0);
+			options_.set(OPTION_SPEEDLIMIT_ENABLE, enable ? 1 : 0);
 		}
 	}
 	else if (event.GetId() == XRCID("ID_MENU_TRANSFER_SPEEDLIMITS_CONFIGURE")) {
@@ -965,8 +947,8 @@ void CMainFrame::OnMenuHandler(wxCommandEvent &event)
 			return;
 		}
 
-		int old_mode = COptions::Get()->get_int(OPTION_COMPARISONMODE);
-		COptions::Get()->set(OPTION_COMPARISONMODE, old_mode ? 0 : 1);
+		int old_mode = options_.get_int(OPTION_COMPARISONMODE);
+		options_.set(OPTION_COMPARISONMODE, old_mode ? 0 : 1);
 
 		CComparisonManager* pComparisonManager = pState->GetComparisonManager();
 		if (pComparisonManager && pComparisonManager->IsComparing()) {
@@ -1030,7 +1012,7 @@ void CMainFrame::OnEngineEvent(CFileZillaEngine* engine)
 			if (m_pStatusView) {
 				m_pStatusView->AddToLog(std::move(static_cast<CLogmsgNotification&>(*pNotification.get())));
 			}
-			if (COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION) == 2 && m_pQueuePane) {
+			if (options_.get_int(OPTION_MESSAGELOG_POSITION) == 2 && m_pQueuePane) {
 				m_pQueuePane->Highlight(3);
 			}
 			break;
@@ -1063,18 +1045,12 @@ void CMainFrame::OnEngineEvent(CFileZillaEngine* engine)
 					if (pAsyncRequest->GetRequestID() == reqId_certificate) {
 						pState->SetSecurityInfo(static_cast<CCertificateNotification&>(*pAsyncRequest));
 					}
-					if (m_pAsyncRequestQueue) {
-						m_pAsyncRequestQueue->AddRequest(pState->engine_.get(), std::move(pAsyncRequest));
+					if (async_request_queue_) {
+						async_request_queue_->AddRequest(pState->engine_.get(), std::move(pAsyncRequest));
 					}
 				}
 			}
 			break;
-		case nId_active:
-			{
-				CActiveNotification const& activeNotification = static_cast<CActiveNotification const&>(*pNotification.get());
-				UpdateActivityLed(activeNotification.GetDirection());
-			}
-			break;
 		case nId_transferstatus:
 			if (m_pQueueView) {
 				m_pQueueView->ProcessNotification(pState->engine_.get(), std::move(pNotification));
@@ -1097,6 +1073,11 @@ void CMainFrame::OnEngineEvent(CFileZillaEngine* engine)
 				pState->ChangeServer(notification.newServer_);
 			}
 			break;
+		case nId_ftp_tls_resumption: {
+			auto const& notification = static_cast<FtpTlsResumptionNotification const&>(*pNotification.get());
+			cert_store_->SetSessionResumptionSupport(fz::to_utf8(notification.server_.GetHost()), notification.server_.GetPort(), true, true);
+			break;
+		}
 		default:
 			break;
 		}
@@ -1105,37 +1086,13 @@ void CMainFrame::OnEngineEvent(CFileZillaEngine* engine)
 	}
 }
 
-void CMainFrame::OnUpdateLedTooltip(wxCommandEvent&)
-{
-	wxString tooltipText;
-
-	wxFileOffset downloadSpeed = m_pQueueView ? m_pQueueView->GetCurrentDownloadSpeed() : 0;
-	wxFileOffset uploadSpeed = m_pQueueView ? m_pQueueView->GetCurrentUploadSpeed() : 0;
-
-	CSizeFormat::_format format = static_cast<CSizeFormat::_format>(COptions::Get()->get_int(OPTION_SIZE_FORMAT));
-	if (format == CSizeFormat::bytes) {
-		format = CSizeFormat::iec;
-	}
-
-	const wxString downloadSpeedStr = CSizeFormat::Format(downloadSpeed, true, format,
-														  COptions::Get()->get_int(OPTION_SIZE_USETHOUSANDSEP) != 0,
-														  COptions::Get()->get_int(OPTION_SIZE_DECIMALPLACES));
-	const wxString uploadSpeedStr = CSizeFormat::Format(uploadSpeed, true, format,
-														COptions::Get()->get_int(OPTION_SIZE_USETHOUSANDSEP) != 0,
-														COptions::Get()->get_int(OPTION_SIZE_DECIMALPLACES));
-	tooltipText.Printf(_("Download speed: %s/s\nUpload speed: %s/s"), downloadSpeedStr, uploadSpeedStr);
-
-	m_pActivityLed[0]->SetToolTip(tooltipText);
-	m_pActivityLed[1]->SetToolTip(tooltipText);
-}
-
 bool CMainFrame::CreateMainToolBar()
 {
 	wxGetApp().AddStartupProfileRecord("CMainFrame::CreateMainToolBar");
 	if (m_pToolBar) {
 #ifdef __WXMAC__
 		if (m_pToolBar) {
-			COptions::Get()->set(OPTION_TOOLBAR_HIDDEN, m_pToolBar->IsShown() ? 0 : 1);
+			options_.set(OPTION_TOOLBAR_HIDDEN, m_pToolBar->IsShown() ? 0 : 1);
 		}
 #endif
 		SetToolBar(0);
@@ -1144,20 +1101,15 @@ bool CMainFrame::CreateMainToolBar()
 	}
 
 #ifndef __WXMAC__
-	if (COptions::Get()->get_int(OPTION_TOOLBAR_HIDDEN) != 0) {
+	if (options_.get_int(OPTION_TOOLBAR_HIDDEN) != 0) {
 		return true;
 	}
 #endif
 
-	m_pToolBar = CToolBar::Load(this);
-	if (!m_pToolBar) {
-		wxLogError(_("Cannot load toolbar from resource file"));
-		return false;
-	}
-	SetToolBar(m_pToolBar);
+	m_pToolBar = new CToolBar(*this, options_);
 
 #ifdef __WXMAC__
-	if (COptions::Get()->get_int(OPTION_TOOLBAR_HIDDEN) != 0) {
+	if (options_.get_int(OPTION_TOOLBAR_HIDDEN) != 0) {
 		m_pToolBar->Show(false);
 	}
 #endif
@@ -1358,7 +1310,7 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 
 #ifdef __WXMAC__
 		if (m_pToolBar) {
-			COptions::Get()->set(OPTION_TOOLBAR_HIDDEN, m_pToolBar->IsShown() ? 0 : 1);
+			options_.set(OPTION_TOOLBAR_HIDDEN, m_pToolBar->IsShown() ? 0 : 1);
 		}
 #endif
 		m_bQuit = true;
@@ -1369,10 +1321,6 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 		return;
 	}
 
-	// Getting deleted by wxWidgets
-	for (int i = 0; i < 2; ++i) {
-		m_pActivityLed[i] = 0;
-	}
 	m_pStatusBar = 0;
 	m_pMenuBar = 0;
 	m_pToolBar = 0;
@@ -1440,7 +1388,7 @@ void CMainFrame::OnClose(wxCloseEvent &event)
 	CSiteManager::ClearIdMap();
 
 	bool filters_toggled = CFilterManager::HasActiveFilters(true) && !CFilterManager::HasActiveFilters(false);
-	COptions::Get()->set(OPTION_FILTERTOGGLESTATE, filters_toggled ? 1 : 0);
+	options_.set(OPTION_FILTERTOGGLESTATE, filters_toggled ? 1 : 0);
 
 	Destroy();
 }
@@ -1492,10 +1440,13 @@ void CMainFrame::OnTimer(wxTimerEvent& event)
 		QueueEvent(evt);
 	}
 #if FZ_MANUALUPDATECHECK
-	else if( event.GetId() == update_dialog_timer_.GetId() ) {
+	else if ( event.GetId() == update_dialog_timer_.GetId() ) {
 		TriggerUpdateDialog();
 	}
 #endif
+	else if (event.GetId() == startupTimer_.GetId()) {
+		PostInitialize();
+	}
 }
 
 void CMainFrame::OpenSiteManager(Site const* site)
@@ -1505,7 +1456,7 @@ void CMainFrame::OpenSiteManager(Site const* site)
 		return;
 	}
 
-	CSiteManagerDialog dlg;
+	CSiteManagerDialog dlg(options_);
 
 	std::vector<CSiteManagerDialog::_connected_site> connected_sites;
 
@@ -1585,13 +1536,6 @@ void CMainFrame::OnSiteManager(wxCommandEvent& e)
 	OpenSiteManager();
 }
 
-void CMainFrame::UpdateActivityLed(int direction)
-{
-	if (m_pActivityLed[direction]) {
-		m_pActivityLed[direction]->Ping();
-	}
-}
-
 void CMainFrame::OnProcessQueue(wxCommandEvent& event)
 {
 	if (m_pQueueView) {
@@ -1606,21 +1550,19 @@ void CMainFrame::OnMenuEditSettings(wxCommandEvent&)
 		return;
 	}
 
-	COptions* pOptions = COptions::Get();
+	std::wstring const oldLang = options_.get_string(OPTION_LANGUAGE);
 
-	wxString oldLang = pOptions->get_string(OPTION_LANGUAGE);
-
-	int oldShowDebugMenu = pOptions->get_int(OPTION_DEBUG_MENU) != 0;
+	int oldShowDebugMenu = options_.get_int(OPTION_DEBUG_MENU) != 0;
 
 	int res = dlg.ShowModal();
 	if (res != wxID_OK) {
 		return;
 	}
 
-	wxString newLang = pOptions->get_string(OPTION_LANGUAGE);
+	std::wstring newLang = options_.get_string(OPTION_LANGUAGE);
 
 	if (oldLang != newLang ||
-		oldShowDebugMenu != pOptions->get_int(OPTION_DEBUG_MENU))
+		oldShowDebugMenu != options_.get_int(OPTION_DEBUG_MENU))
 	{
 		CreateMenus();
 	}
@@ -1639,7 +1581,7 @@ void CMainFrame::OnToggleLogView(wxCommandEvent&)
 
 	bool shown;
 
-	if (COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION) == 1) {
+	if (options_.get_int(OPTION_MESSAGELOG_POSITION) == 1) {
 		if (!m_pQueueLogSplitter) {
 			return;
 		}
@@ -1673,8 +1615,8 @@ void CMainFrame::OnToggleLogView(wxCommandEvent&)
 		}
 	}
 
-	if (COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION) != 2) {
-		COptions::Get()->set(OPTION_SHOW_MESSAGELOG, shown);
+	if (options_.get_int(OPTION_MESSAGELOG_POSITION) != 2) {
+		options_.set(OPTION_SHOW_MESSAGELOG, shown);
 	}
 }
 
@@ -1701,8 +1643,8 @@ void CMainFrame::ShowDirectoryTree(bool local, bool show)
 		return;
 	}
 
-	const int layout = COptions::Get()->get_int(OPTION_FILEPANE_LAYOUT);
-	const int swap = COptions::Get()->get_int(OPTION_FILEPANE_SWAP);
+	const int layout = options_.get_int(OPTION_FILEPANE_LAYOUT);
+	const int swap = options_.get_int(OPTION_FILEPANE_SWAP);
 	for (int i = 0; i < m_pContextControl->GetTabCount(); ++i) {
 		CContextControl::_context_controls* controls = m_pContextControl->GetControlsFromTabIndex(i);
 		if (!controls) {
@@ -1732,7 +1674,7 @@ void CMainFrame::ShowDirectoryTree(bool local, bool show)
 		}
 	}
 
-	COptions::Get()->set(local ? OPTION_SHOW_TREE_LOCAL : OPTION_SHOW_TREE_REMOTE, show);
+	options_.set(local ? OPTION_SHOW_TREE_LOCAL : OPTION_SHOW_TREE_REMOTE, show);
 }
 
 void CMainFrame::OnToggleQueueView(wxCommandEvent&)
@@ -1742,7 +1684,7 @@ void CMainFrame::OnToggleQueueView(wxCommandEvent&)
 	}
 
 	bool shown;
-	if (COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION) == 1) {
+	if (options_.get_int(OPTION_MESSAGELOG_POSITION) == 1) {
 		if (!m_pQueueLogSplitter) {
 			return;
 		}
@@ -1776,12 +1718,12 @@ void CMainFrame::OnToggleQueueView(wxCommandEvent&)
 		shown = m_pBottomSplitter->IsSplit();
 	}
 
-	COptions::Get()->set(OPTION_SHOW_QUEUE, shown);
+	options_.set(OPTION_SHOW_QUEUE, shown);
 }
 
 void CMainFrame::OnMenuHelpAbout(wxCommandEvent&)
 {
-	CAboutDialog dlg;
+	CAboutDialog dlg(options_);
 	if (!dlg.Create(this)) {
 		return;
 	}
@@ -1815,8 +1757,8 @@ void CMainFrame::OnCheckForUpdates(wxCommandEvent& event)
 		return;
 	}
 
-	if (event.GetId() == XRCID("ID_CHECKFORUPDATES") || (!COptions::Get()->get_int(OPTION_DEFAULT_DISABLEUPDATECHECK) && COptions::Get()->get_int(OPTION_UPDATECHECK) != 0)) {
-		m_pUpdater->RunIfNeeded();
+	if (event.GetId() == XRCID("ID_CHECKFORUPDATES") || (!options_.get_int(OPTION_DEFAULT_DISABLEUPDATECHECK) && options_.get_int(OPTION_UPDATECHECK) != 0)) {
+		m_pUpdater->Run(true);
 	}
 
 	update_dialog_timer_.Stop();
@@ -1828,43 +1770,45 @@ void CMainFrame::OnCheckForUpdates(wxCommandEvent& event)
 void CMainFrame::UpdaterStateChanged(UpdaterState s, build const& v)
 {
 #if FZ_AUTOUPDATECHECK
-	if (!m_pMenuBar) {
-		return;
-	}
+	CallAfter([this, s, v]() {
+		if (!m_pMenuBar) {
+			return;
+		}
 
-	if (s == UpdaterState::idle) {
-		wxMenu* m = 0;
-		wxMenuItem* pItem = m_pMenuBar->FindItem(GetAvailableUpdateMenuId(), &m);
-		if (pItem && m) {
-			for (size_t i = 0; i != m_pMenuBar->GetMenuCount(); ++i) {
-				if( m_pMenuBar->GetMenu(i) == m ) {
-					m_pMenuBar->Remove(i);
-					delete m;
-					break;
+		if (s == UpdaterState::idle) {
+			wxMenu* m = 0;
+			wxMenuItem* pItem = m_pMenuBar->FindItem(GetAvailableUpdateMenuId(), &m);
+			if (pItem && m) {
+				for (size_t i = 0; i != m_pMenuBar->GetMenuCount(); ++i) {
+					if (m_pMenuBar->GetMenu(i) == m) {
+						m_pMenuBar->Remove(i);
+						delete m;
+						break;
+					}
 				}
 			}
+			return;
 		}
-		return;
-	}
-	else if (s != UpdaterState::newversion && s != UpdaterState::newversion_ready && s != UpdaterState::newversion_stale) {
-		return;
-	}
-	
-	wxString const name = v.version_.empty() ? _("Unknown version") : wxString::Format(_("&Version %s"), v.version_);
-
-	wxMenuItem* pItem = m_pMenuBar->FindItem(GetAvailableUpdateMenuId());
-	if (!pItem) {
-		wxMenu* pMenu = new wxMenu();
-		pMenu->Append(GetAvailableUpdateMenuId(), name);
-		m_pMenuBar->Append(pMenu, _("&New version available!"));
-
-		if (!update_dialog_timer_.IsRunning()) {
-			update_dialog_timer_.Start(1, true);
+		else if (s != UpdaterState::newversion && s != UpdaterState::newversion_ready && s != UpdaterState::newversion_stale) {
+			return;
 		}
-	}
-	else {
-		pItem->SetItemLabel(name);
-	}
+
+		wxString const name = v.version_.empty() ? _("Unknown version") : wxString::Format(_("&Version %s"), v.version_);
+
+		wxMenuItem* pItem = m_pMenuBar->FindItem(GetAvailableUpdateMenuId());
+		if (!pItem) {
+			wxMenu* pMenu = new wxMenu();
+			pMenu->Append(GetAvailableUpdateMenuId(), name);
+			m_pMenuBar->Append(pMenu, _("&New version available!"));
+
+			if (!update_dialog_timer_.IsRunning()) {
+				update_dialog_timer_.Start(1, true);
+			}
+		}
+		else {
+			pItem->SetItemLabel(name);
+		}
+	});
 #endif
 }
 
@@ -1893,10 +1837,10 @@ void CMainFrame::TriggerUpdateDialog()
 
 void CMainFrame::UpdateLayout()
 {
-	int const layout = COptions::Get()->get_int(OPTION_FILEPANE_LAYOUT);
-	int const swap = COptions::Get()->get_int(OPTION_FILEPANE_SWAP);
+	int const layout = options_.get_int(OPTION_FILEPANE_LAYOUT);
+	int const swap = options_.get_int(OPTION_FILEPANE_SWAP);
 
-	int const messagelog_position = COptions::Get()->get_int(OPTION_MESSAGELOG_POSITION);
+	int const messagelog_position = options_.get_int(OPTION_MESSAGELOG_POSITION);
 
 	// First handle changes in message log position as it can make size of the other panes change
 	{
@@ -2101,7 +2045,7 @@ bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark, CState* pS
 	}
 
 	if (pState->IsRemoteConnected() || !pState->IsRemoteIdle()) {
-		int action = COptions::Get()->get_int(OPTION_ALREADYCONNECTED_CHOICE);
+		int action = options_.get_int(OPTION_ALREADYCONNECTED_CHOICE);
 		if (action < 2) {
 			wxDialogEx dlg;
 			if (!dlg.Load(this, _T("ID_ALREADYCONNECTED"))) {
@@ -2129,7 +2073,7 @@ bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark, CState* pS
 			if (XRCCTRL(dlg, "ID_REMEMBER", wxCheckBox)->IsChecked()) {
 				action |= 2;
 			}
-			COptions::Get()->set(OPTION_ALREADYCONNECTED_CHOICE, action);
+			options_.set(OPTION_ALREADYCONNECTED_CHOICE, action);
 		}
 
 		if (!(action & 1)) {
@@ -2165,13 +2109,13 @@ bool CMainFrame::ConnectToSite(Site & data, Bookmark const& bookmark, CState* pS
 
 void CMainFrame::CheckChangedSettings()
 {
-	m_pAsyncRequestQueue->RecheckDefaults();
+	async_request_queue_->RecheckDefaults();
 
 	CAutoAsciiFiles::SettingsChanged();
 
 #if FZ_MANUALUPDATECHECK
 	if (m_pUpdater) {
-		m_pUpdater->Init();
+		m_pUpdater->Run(false);
 	}
 #endif
 }
@@ -2306,7 +2250,7 @@ void CMainFrame::RememberSplitterPositions()
 		static_cast<int>(m_pQueueLogSplitter->GetRelativeSashPosition() * 1000000000)
 	);
 
-	COptions::Get()->set(OPTION_MAINWINDOW_SPLITTER_POSITION, posString);
+	options_.set(OPTION_MAINWINDOW_SPLITTER_POSITION, posString);
 }
 
 bool CMainFrame::RestoreSplitterPositions()
@@ -2316,7 +2260,7 @@ bool CMainFrame::RestoreSplitterPositions()
 	}
 
 	// top_pos bottom_height view_pos view_height_width local_pos remote_pos
-	std::wstring const positions = COptions::Get()->get_string(OPTION_MAINWINDOW_SPLITTER_POSITION);
+	std::wstring const positions = options_.get_string(OPTION_MAINWINDOW_SPLITTER_POSITION);
 	auto tokens = fz::strtok_view(positions, L" ");
 	if (tokens.size() < 6) {
 		return false;
@@ -2418,8 +2362,8 @@ void CMainFrame::OnActivate(wxActivateEvent& event)
 		pEditHandler->CheckForModifications(true);
 	}
 
-	if (m_pAsyncRequestQueue) {
-		m_pAsyncRequestQueue->TriggerProcessing();
+	if (async_request_queue_) {
+		async_request_queue_->TriggerProcessing();
 	}
 }
 
@@ -2436,7 +2380,7 @@ void CMainFrame::OnToolbarComparison(wxCommandEvent&)
 		return;
 	}
 
-	if (!COptions::Get()->get_int(OPTION_FILEPANE_LAYOUT)) {
+	if (!options_.get_int(OPTION_FILEPANE_LAYOUT)) {
 		CContextControl::_context_controls* controls = m_pContextControl->GetCurrentControls();
 		if (!controls) {
 			return;
@@ -2510,7 +2454,7 @@ void CMainFrame::OnToolbarComparisonDropdown(wxCommandEvent& event)
 	CComparisonManager* pComparisonManager = pState->GetComparisonManager();
 	menu->FindItem(XRCID("ID_TOOLBAR_COMPARISON"))->Check(pComparisonManager->IsComparing());
 
-	const int mode = COptions::Get()->get_int(OPTION_COMPARISONMODE);
+	const int mode = options_.get_int(OPTION_COMPARISONMODE);
 	if (mode == 0) {
 		menu->FindItem(XRCID("ID_COMPARE_SIZE"))->Check();
 	}
@@ -2518,7 +2462,7 @@ void CMainFrame::OnToolbarComparisonDropdown(wxCommandEvent& event)
 		menu->FindItem(XRCID("ID_COMPARE_DATE"))->Check();
 	}
 
-	menu->Check(XRCID("ID_COMPARE_HIDEIDENTICAL"), COptions::Get()->get_int(OPTION_COMPARE_HIDEIDENTICAL) != 0);
+	menu->Check(XRCID("ID_COMPARE_HIDEIDENTICAL"), options_.get_int(OPTION_COMPARE_HIDEIDENTICAL) != 0);
 
 	ShowDropdownMenu(menu, m_pToolBar, event);
 }
@@ -2553,9 +2497,9 @@ void CMainFrame::OnDropdownComparisonMode(wxCommandEvent& event)
 		return;
 	}
 
-	int old_mode = COptions::Get()->get_int(OPTION_COMPARISONMODE);
+	int old_mode = options_.get_int(OPTION_COMPARISONMODE);
 	int new_mode = (event.GetId() == XRCID("ID_COMPARE_SIZE")) ? 0 : 1;
-	COptions::Get()->set(OPTION_COMPARISONMODE, new_mode);
+	options_.set(OPTION_COMPARISONMODE, new_mode);
 
 	CComparisonManager* pComparisonManager = pState->GetComparisonManager();
 	if (old_mode != new_mode && pComparisonManager) {
@@ -2573,8 +2517,8 @@ void CMainFrame::OnDropdownComparisonHide(wxCommandEvent&)
 		return;
 	}
 
-	bool old_mode = COptions::Get()->get_int(OPTION_COMPARE_HIDEIDENTICAL) != 0;
-	COptions::Get()->set(OPTION_COMPARE_HIDEIDENTICAL, old_mode ? 0 : 1);
+	bool old_mode = options_.get_int(OPTION_COMPARE_HIDEIDENTICAL) != 0;
+	options_.set(OPTION_COMPARE_HIDEIDENTICAL, old_mode ? 0 : 1);
 
 	CComparisonManager* pComparisonManager = pState->GetComparisonManager();
 	if (pComparisonManager) {
@@ -2610,7 +2554,7 @@ void CMainFrame::ProcessCommandLine()
 
 	std::wstring site;
 	if (pCommandLine->HasSwitch(CCommandLine::sitemanager)) {
-		if (COptions::Get()->get_int(OPTION_STARTUP_ACTION) != 1) {
+		if (options_.get_int(OPTION_STARTUP_ACTION) != 1) {
 			OpenSiteManager();
 		}
 	}
@@ -2643,7 +2587,7 @@ void CMainFrame::ProcessCommandLine()
 			wxMessageBoxEx(error, _("Syntax error in command line"));
 		}
 
-		if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) && site.credentials.logonType_ == LogonType::normal) {
+		if (options_.get_int(OPTION_DEFAULT_KIOSKMODE) && site.credentials.logonType_ == LogonType::normal) {
 			site.SetLogonType(LogonType::ask);
 			CLoginManager::Get().RememberPassword(site);
 		}
@@ -2738,14 +2682,14 @@ void CMainFrame::OnIconize(wxIconizeEvent& event)
 		}
 		Show(true);
 
-		if (m_pAsyncRequestQueue) {
-			m_pAsyncRequestQueue->TriggerProcessing();
+		if (async_request_queue_) {
+			async_request_queue_->TriggerProcessing();
 		}
 
 		return;
 	}
 
-	if (!COptions::Get()->get_int(OPTION_MINIMIZE_TRAY)) {
+	if (!options_.get_int(OPTION_MINIMIZE_TRAY)) {
 		return;
 	}
 
@@ -2783,8 +2727,8 @@ void CMainFrame::OnTaskBarClick(wxTaskBarIconEvent&)
 	Show(true);
 	Iconize(false);
 
-	if (m_pAsyncRequestQueue)
-		m_pAsyncRequestQueue->TriggerProcessing();
+	if (async_request_queue_)
+		async_request_queue_->TriggerProcessing();
 
 #ifdef __WXGTK__
 	QueueEvent(new wxCommandEvent(fzEVT_TASKBAR_CLICK_DELAYED));
@@ -2807,7 +2751,7 @@ void CMainFrame::OnSearch(wxCommandEvent&)
 		return;
 	}
 
-	CSearchDialog dlg(this, *pState, m_pQueueView);
+	CSearchDialog dlg(this, *pState, m_pQueueView, options_);
 	if (!dlg.Load()) {
 		return;
 	}
@@ -2826,21 +2770,19 @@ void CMainFrame::PostInitialize()
 	// Need to do this after welcome screen to avoid simultaneous display of multiple dialogs
 	if (!m_pUpdater) {
 		update_dialog_timer_.SetOwner(this);
-		m_pUpdater = new CUpdater(*this, m_engineContext,
-			[this](CActiveNotification const& notification) {
-			UpdateActivityLed(notification.GetDirection());
-		}
-		);
-		m_pUpdater->Init();
-		ShowOverlay(m_pUpdater->GetResources(resource_type::overlay), this, m_pTopSplitter, wxPoint(-40, 30));
+		m_pUpdater = new CUpdater(m_engineContext);
+		m_pUpdater->AddHandler(*this);
+		ShowOverlay(options_, m_pUpdater->GetResources(resource_type::overlay), this, m_pTopSplitter, wxPoint(-40, 30));
 	}
 #endif
 
-	int const startupAction = COptions::Get()->get_int(OPTION_STARTUP_ACTION);
+	int const startupAction = options_.get_int(OPTION_STARTUP_ACTION);
 	bool startupReconnect = startupAction == 2;
 
 	if (startupAction == 1) {
-		OpenSiteManager();
+		if (wxDialogEx::CanShowPopupDialog()) {
+			OpenSiteManager();
+		}
 		startupReconnect = false;
 	}
 
@@ -2850,7 +2792,7 @@ void CMainFrame::PostInitialize()
 	}
 
 	if (m_pContextControl && startupReconnect) {
-		auto xml = COptions::Get()->get_xml(OPTION_TAB_DATA);
+		auto xml = options_.get_xml(OPTION_TAB_DATA);
 		pugi::xml_node tabs = xml.child("Tabs");
 		int i = 0;
 		for (auto tab = tabs.child("Tab"); tab; tab = tab.next_sibling("Tab")) {
@@ -2896,7 +2838,7 @@ void CMainFrame::OnMenuCloseTab(wxCommandEvent&)
 
 void CMainFrame::OnToggleToolBar(wxCommandEvent& event)
 {
-	COptions::Get()->set(OPTION_TOOLBAR_HIDDEN, event.IsChecked() ? 0 : 1);
+	options_.set(OPTION_TOOLBAR_HIDDEN, event.IsChecked() ? 0 : 1);
 #ifdef __WXMAC__
 	if (m_pToolBar) {
 		m_pToolBar->Show(event.IsChecked());
