@@ -6,6 +6,7 @@
 #include "option_change_event_handler.h"
 #include "queue.h"
 #include "queue_storage.h"
+#include "state.h"
 
 #include "../include/libfilezilla_engine.h"
 #include "../include/notification.h"
@@ -113,7 +114,7 @@ class CQueueView final : public CQueueViewBase, public COptionChangeEventHandler
 	friend class CActionAfterBlocker;
 
 public:
-	CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsyncRequestQueue* pAsyncRequestQueue);
+	CQueueView(CQueue* parent, int index, CMainFrame* pMainFrame, CAsyncRequestQueue* pAsyncRequestQueue, cert_store & certStore);
 	virtual ~CQueueView();
 
 	bool QueueFile(bool const queueOnly, bool const download,
@@ -139,7 +140,6 @@ public:
 	void RemoveAll();
 
 	void LoadQueue();
-	void LoadQueueFromXML();
 	void ImportQueue(pugi::xml_node element, bool updateSelections);
 
 	virtual void InsertItem(CServerItem* pServerItem, CQueueItem* pItem) override;
@@ -152,14 +152,6 @@ public:
 
 	static std::wstring ReplaceInvalidCharacters(std::wstring const& filename, bool includeQuotesAndBreaks = false);
 
-	// Get the current download speed as the sum of all active downloads.
-	// Unit is byte/s.
-	wxFileOffset GetCurrentDownloadSpeed();
-
-	// Get the current upload speed as the sum of all active uploads.
-	// Unit is byte/s.
-	wxFileOffset GetCurrentUploadSpeed();
-
 	std::shared_ptr<CActionAfterBlocker> GetActionAfterBlocker();
 
 protected:
@@ -259,6 +251,7 @@ protected:
 
 	CMainFrame* m_pMainFrame;
 	CAsyncRequestQueue* m_pAsyncRequestQueue;
+	cert_store & cert_store_;
 
 	std::list<CFileZillaEngine*> m_waitingForPassword;
 
@@ -282,10 +275,6 @@ protected:
 
 	CQueueStorage m_queue_storage;
 
-	// Get the current transfer speed.
-	// Unit is byte/s.
-	wxFileOffset GetCurrentSpeed(bool countDownload, bool countUpload);
-
 	void OnEngineEvent(CFileZillaEngine* engine);
 
 	void OnAskPassword();
