@@ -10,7 +10,7 @@
 #include "drop_target_ex.h"
 #include "edithandler.h"
 #include "filezillaapp.h"
-#include "filter.h"
+#include "filter_manager.h"
 #include "graphics.h"
 #include "infotext.h"
 #include "inputdialog.h"
@@ -195,7 +195,7 @@ public:
 		return true;
 	}
 
-	virtual int DisplayDropHighlight(wxPoint point)
+	virtual int DisplayDropHighlight(wxPoint const& point)
 	{
 		DoDisplayDropHighlight(point);
 		return -1;
@@ -316,10 +316,10 @@ BEGIN_EVENT_TABLE(CRemoteListView, CFileListCtrl<CGenericFileData>)
 	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CRemoteListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
-CRemoteListView::CRemoteListView(CView* pParent, CState& state, CQueueView* pQueue)
-	: CFileListCtrl<CGenericFileData>(pParent, pQueue),
-	CStateEventHandler(state),
-	m_parentView(pParent)
+CRemoteListView::CRemoteListView(CView* pParent, CState& state, CQueueView* pQueue, COptionsBase & options)
+	: CFileListCtrl<CGenericFileData>(pParent, pQueue, options)
+	, CStateEventHandler(state)
+	, m_parentView(pParent)
 {
 	state.RegisterHandler(this, STATECHANGE_REMOTE_DIR);
 	state.RegisterHandler(this, STATECHANGE_APPLYFILTER);
@@ -360,8 +360,8 @@ CRemoteListView::CRemoteListView(CView* pParent, CState& state, CQueueView* pQue
 
 CRemoteListView::~CRemoteListView()
 {
-	wxString str = wxString::Format(_T("%d %d"), m_sortDirection, m_sortColumn);
-	COptions::Get()->set(OPTION_REMOTEFILELIST_SORTORDER, str.ToStdWstring());
+	auto str = fz::sprintf(L"%d %d", m_sortDirection, m_sortColumn);
+	options_.set(OPTION_REMOTEFILELIST_SORTORDER, str);
 }
 
 // See comment to OnGetItemText
@@ -369,16 +369,23 @@ int CRemoteListView::OnGetItemImage(long item) const
 {
 	CRemoteListView *pThis = const_cast<CRemoteListView *>(this);
 	int index = GetItemIndex(item);
-	if (index == -1) {
+	if (index < 0) {
 		return -1;
 	}
 
+	if (index >= pThis->m_fileData.size()) {
+		return -1;
+	}
 	int &icon = pThis->m_fileData[index].icon;
 
 	if (icon != -2) {
 		return icon;
 	}
 
+	if (!m_pDirectoryListing || index >= m_pDirectoryListing->size()) {
+		return -1;
+	}
+
 	icon = pThis->GetIconIndex(iconType::file, (*m_pDirectoryListing)[index].name, false, (*m_pDirectoryListing)[index].is_dir());
 	return icon;
 }
@@ -415,6 +422,7 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 {
 	size_t const to_add = pDirectoryListing->size() - m_pDirectoryListing->size();
 	m_pDirectoryListing = pDirectoryListing;
+	UpdateSortComparisonObject();
 
 	m_indexMapping[0] = pDirectoryListing->size();
 
@@ -431,7 +439,7 @@ void CRemoteListView::UpdateDirectoryListing_Added(std::shared_ptr<CDirectoryLis
 		added_indexes.reserve(to_add);
 	}
 
-	std::unique_ptr<CFileListCtrlSortBase> compare = GetSortComparisonObject();
+	auto& compare = GetSortComparisonObject();
 	for (size_t i = pDirectoryListing->size() - to_add; i < pDirectoryListing->size(); ++i) {
 		CDirentry const& entry = (*pDirectoryListing)[i];
 		CGenericFileData data;
@@ -495,6 +503,7 @@ void CRemoteListView::UpdateDirectoryListing_Removed(std::shared_ptr<CDirectoryL
 	size_t const countRemoved = m_pDirectoryListing->size() - pDirectoryListing->size();
 	if (!countRemoved) {
 		m_pDirectoryListing = pDirectoryListing;
+		UpdateSortComparisonObject();
 		return;
 	}
 	wxASSERT(!IsComparing());
@@ -618,6 +627,7 @@ void CRemoteListView::UpdateDirectoryListing_Removed(std::shared_ptr<CDirectoryL
 	wxASSERT(m_indexMapping.size() == pDirectoryListing->size() + 1);
 
 	m_pDirectoryListing = pDirectoryListing;
+	UpdateSortComparisonObject();
 
 	if (m_pFilelistStatusBar) {
 		m_pFilelistStatusBar->SetHidden(m_pDirectoryListing->size() + 1 - m_indexMapping.size());
@@ -657,6 +667,7 @@ bool CRemoteListView::UpdateDirectoryListing(std::shared_ptr<CDirectoryListing>
 		}
 
 		m_pDirectoryListing = pDirectoryListing;
+		UpdateSortComparisonObject();
 
 		// We don't have to do anything
 		return true;
@@ -733,6 +744,7 @@ void CRemoteListView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 	}
 
 	m_pDirectoryListing = pDirectoryListing;
+	UpdateSortComparisonObject();
 
 	m_fileData.clear();
 	m_indexMapping.clear();
@@ -747,12 +759,16 @@ void CRemoteListView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 	if (m_pDirectoryListing) {
 		SetInfoText();
 
+		CFilterManager const& filter = m_state.GetStateFilterManager();
+		if (!filter.HasActiveRemoteFilters()) {
+			m_indexMapping.reserve(m_pDirectoryListing->size() + 1);
+			m_fileData.reserve(m_pDirectoryListing->size() + 1);
+		}
+
 		m_indexMapping.push_back(m_pDirectoryListing->size());
 
 		std::wstring const path = m_pDirectoryListing->path.GetPath();
 
-		CFilterManager const& filter = m_state.GetStateFilterManager();
-		
 		for (unsigned int i = 0; i < m_pDirectoryListing->size(); ++i) {
 			const CDirentry& entry = (*m_pDirectoryListing)[i];
 			CGenericFileData data;
@@ -764,7 +780,7 @@ void CRemoteListView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 				}
 #endif
 			}
-			m_fileData.push_back(data);
+			m_fileData.emplace_back(std::move(data));
 
 			if (filter.FilenameFiltered(entry.name, path, entry.is_dir(), entry.size, false, 0, entry.time)) {
 				++hidden;
@@ -789,7 +805,7 @@ void CRemoteListView::SetDirectoryListing(std::shared_ptr<CDirectoryListing> con
 
 		CGenericFileData data;
 		data.icon = m_dirIcon;
-		m_fileData.push_back(data);
+		m_fileData.emplace_back(std::move(data));
 	}
 	else {
 		eraseBackground = true;
@@ -876,7 +892,7 @@ std::wstring StripVMSRevision(std::wstring const& name)
 
 void CRemoteListView::OnItemActivated(wxListEvent &event)
 {
-	int const action = COptions::Get()->get_int(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
+	int const action = options_.get_int(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
 	if (!m_state.IsRemoteIdle(action ? false : true)) {
 		wxBell();
 		return;
@@ -961,7 +977,7 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 			}
 		}
 		else {
-			const int action = COptions::Get()->get_int(OPTION_DOUBLECLICK_ACTION_FILE);
+			const int action = options_.get_int(OPTION_DOUBLECLICK_ACTION_FILE);
 			if (action == 3) {
 				// No action
 				wxBell();
@@ -984,7 +1000,7 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 			}
 
 			std::wstring localFile = CQueueView::ReplaceInvalidCharacters(name);
-			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->get_int(OPTION_STRIP_VMS_REVISION)) {
+			if (m_pDirectoryListing->path.GetType() == VMS && options_.get_int(OPTION_STRIP_VMS_REVISION)) {
 				localFile = StripVMSRevision(localFile);
 			}
 			m_pQueue->QueueFile(queue_only, true, name,
@@ -1286,7 +1302,7 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 		}
 		else {
 			std::wstring localFile = CQueueView::ReplaceInvalidCharacters(name);
-			if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->get_int(OPTION_STRIP_VMS_REVISION)) {
+			if (m_pDirectoryListing->path.GetType() == VMS && options_.get_int(OPTION_STRIP_VMS_REVISION)) {
 				localFile = StripVMSRevision(localFile);
 			}
 			m_pQueue->QueueFile(queue_only, true,
@@ -1302,7 +1318,7 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 	if (!root.empty()) {
 		pRecursiveOperation->AddRecursionRoot(std::move(root));
 		CFilterManager filter;
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !queue_only);
+		pRecursiveOperation->StartRecursiveOperation(recursive_operation::recursive_transfer, filter.GetActiveFilters(), !queue_only);
 	}
 }
 
@@ -1533,7 +1549,7 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent&)
 
 		if (!root.empty()) {
 			pRecursiveOperation->AddRecursionRoot(std::move(root));
-			pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filter.GetActiveFilters());
+			pRecursiveOperation->StartRecursiveOperation(recursive_operation::recursive_delete, filter.GetActiveFilters());
 		}
 	}
 }
@@ -1697,7 +1713,7 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent&)
 {
 	Site const& site = m_state.GetSite();
 	auto protocol = site.server.GetProtocol();
-	
+
 	if (!m_state.IsRemoteConnected() || !m_state.IsRemoteIdle()) {
 		wxBell();
 		return;
@@ -1844,11 +1860,11 @@ void CRemoteListView::HandleGenericChmod(ChmodUICommand &command)
 		pRecursiveOperation->SetChmodData(std::move(chmodData));
 		pRecursiveOperation->AddRecursionRoot(std::move(root));
 		CFilterManager filter;
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, filter.GetActiveFilters());
+		pRecursiveOperation->StartRecursiveOperation(recursive_operation::recursive_chmod, filter.GetActiveFilters());
 
-		// Refresh listing. This gets done implicitely by the recursive operation, so
+		// Refresh listing. This gets done implicitly by the recursive operation, so
 		// only it if not recursing.
-		if (pRecursiveOperation->GetOperationMode() != CRecursiveOperation::recursive_chmod) {
+		if (pRecursiveOperation->GetOperationMode() != recursive_operation::recursive_chmod) {
 			m_state.ChangeRemoteDir(m_pDirectoryListing->path);
 		}
 	}
@@ -1895,7 +1911,7 @@ void CRemoteListView::ApplyCurrentFilter()
 			++hidden;
 			continue;
 		}
-	
+
 		if (entry.is_dir()) {
 			++totalDirCount;
 		}
@@ -1947,7 +1963,7 @@ std::vector<std::wstring> CRemoteListView::RememberSelectedItems(std::wstring& f
 			if (item < 0) {
 				break;
 			}
-			
+
 			if (!item) {
 				selectedNames.push_back(L"..");
 			}
@@ -2119,10 +2135,10 @@ void CRemoteListView::OnStateChange(t_statechange_notifications notification, st
 	}
 	else if (notification == STATECHANGE_SERVER) {
 		if (m_windowTinter) {
-			m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+			m_windowTinter->SetBackgroundTint(site_colour_to_wx(m_state.GetSite().m_colour));
 		}
 		if (m_pInfoText) {
-			m_pInfoText->SetBackgroundTint(m_state.GetSite().m_colour);
+			m_pInfoText->SetBackgroundTint(site_colour_to_wx(m_state.GetSite().m_colour));
 		}
 	}
 	else {
@@ -2163,7 +2179,7 @@ void CRemoteListView::SetInfoText()
 
 void CRemoteListView::OnBeginDrag(wxListEvent&)
 {
-	if (COptions::Get()->get_int(OPTION_DND_DISABLED) != 0) {
+	if (options_.get_int(OPTION_DND_DISABLED) != 0) {
 		return;
 	}
 
@@ -2506,10 +2522,10 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 	}
 
 	if (!column) {
-		if ((size_t)index == m_pDirectoryListing->size()) {
+		if (m_pDirectoryListing && (size_t)index == m_pDirectoryListing->size()) {
 			return _T("..");
 		}
-		else if ((size_t)index < m_pDirectoryListing->size()) {
+		else if (m_pDirectoryListing && (size_t)index < m_pDirectoryListing->size()) {
 			return (*m_pDirectoryListing)[index].name;
 		}
 		else {
@@ -2520,7 +2536,7 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 		return wxString(); //.. has no attributes
 	}
 
-	if ((size_t)index >= m_pDirectoryListing->size()) {
+	if (!m_pDirectoryListing || (size_t)index >= m_pDirectoryListing->size()) {
 		return wxString();
 	}
 
@@ -2560,50 +2576,51 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 	return wxString();
 }
 
-std::unique_ptr<CFileListCtrlSortBase> CRemoteListView::GetSortComparisonObject()
+void CRemoteListView::UpdateSortComparisonObject()
 {
 	CFileListCtrlSort<CDirectoryListing>::DirSortMode dirSortMode = GetDirSortMode();
 	NameSortMode nameSortMode = GetNameSortMode();
 
-	CDirectoryListing const& directoryListing = *m_pDirectoryListing;
+	static CDirectoryListing const empty;
+	CDirectoryListing const& directoryListing = m_pDirectoryListing ? *m_pDirectoryListing : empty;
 	if (!m_sortDirection) {
 		if (m_sortColumn == 1) {
-			return std::make_unique<CFileListCtrlSortSize<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortSize<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 2) {
-			return std::make_unique<CFileListCtrlSortType<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortType<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 3) {
-			return std::make_unique<CFileListCtrlSortTime<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortTime<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 4) {
-			return std::make_unique<CFileListCtrlSortPermissions<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortPermissions<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 5) {
-			return std::make_unique<CFileListCtrlSortOwnerGroup<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortOwnerGroup<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else {
-			return std::make_unique<CFileListCtrlSortName<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CFileListCtrlSortName<CDirectoryListing, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
 		}
 	}
 	else {
 		if (m_sortColumn == 1) {
-			return std::make_unique<CReverseSort<CFileListCtrlSortSize<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortSize<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 2) {
-			return std::make_unique<CReverseSort<CFileListCtrlSortType<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortType<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 3) {
-			return std::make_unique<CReverseSort<CFileListCtrlSortTime<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortTime<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 4) {
-			return std::make_unique<CReverseSort<CFileListCtrlSortPermissions<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortPermissions<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else if (m_sortColumn == 5) {
-			return std::make_unique<CReverseSort<CFileListCtrlSortOwnerGroup<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortOwnerGroup<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
 		}
 		else {
-			return std::make_unique<CReverseSort<CFileListCtrlSortName<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
+			sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortName<CDirectoryListing, CGenericFileData>, CGenericFileData>>(directoryListing, m_fileData, dirSortMode, nameSortMode, this);
 		}
 	}
 }
@@ -2628,7 +2645,7 @@ void CRemoteListView::LinkIsNotDir(CServerPath const& path, std::wstring const&
 {
 	if (m_pLinkResolveState && m_pLinkResolveState->remote_path == path && m_pLinkResolveState->link == link) {
 		std::wstring localFile = CQueueView::ReplaceInvalidCharacters(link);
-		if (m_pDirectoryListing->path.GetType() == VMS && COptions::Get()->get_int(OPTION_STRIP_VMS_REVISION)) {
+		if (m_pDirectoryListing->path.GetType() == VMS && options_.get_int(OPTION_STRIP_VMS_REVISION)) {
 			localFile = StripVMSRevision(localFile);
 		}
 		m_pQueue->QueueFile(false, true,
