@@ -38,15 +38,6 @@ public:
 	{
 	}
 
-	void ClearDropHighlight()
-	{
-		const wxTreeItemId dropHighlight = m_pRemoteTreeView->m_dropHighlight;
-		if (dropHighlight != wxTreeItemId()) {
-			m_pRemoteTreeView->SetItemDropHighlight(dropHighlight, false);
-			m_pRemoteTreeView->m_dropHighlight = wxTreeItemId();
-		}
-	}
-
 	wxTreeItemId GetHit(const wxPoint& point)
 	{
 		int flags = 0;
@@ -143,7 +134,7 @@ public:
 		if (!CScrollableDropTarget<wxTreeCtrlEx>::OnDrop(x, y)) {
 			return false;
 		}
-		ClearDropHighlight();
+		m_pRemoteTreeView->ClearDropHighlight();
 
 		wxTreeItemId hit = GetHit(wxPoint(x, y));
 		if (!hit) {
@@ -158,28 +149,22 @@ public:
 		return true;
 	}
 
-	wxTreeItemId DisplayDropHighlight(wxPoint point)
+	wxTreeItemId DisplayDropHighlight(wxPoint const& point) override
 	{
 		wxTreeItemId hit = GetHit(point);
 		if (!hit) {
-			ClearDropHighlight();
+			m_pRemoteTreeView->ClearDropHighlight();
 			return wxTreeItemId();
 		}
 
-		const CServerPath& path = m_pRemoteTreeView->GetPathFromItem(hit);
+		CServerPath const& path = m_pRemoteTreeView->GetPathFromItem(hit);
 
 		if (path.empty()) {
-			ClearDropHighlight();
+			m_pRemoteTreeView->ClearDropHighlight();
 			return wxTreeItemId();
 		}
 
-		const wxTreeItemId dropHighlight = m_pRemoteTreeView->m_dropHighlight;
-		if (dropHighlight != wxTreeItemId()) {
-			m_pRemoteTreeView->SetItemDropHighlight(dropHighlight, false);
-		}
-
-		m_pRemoteTreeView->SetItemDropHighlight(hit, true);
-		m_pRemoteTreeView->m_dropHighlight = hit;
+		m_pRemoteTreeView->DisplayDropHighlight(hit);
 
 		return hit;
 	}
@@ -192,7 +177,7 @@ public:
 			def == wxDragNone ||
 			def == wxDragCancel)
 		{
-			ClearDropHighlight();
+			m_pRemoteTreeView->ClearDropHighlight();
 			return def;
 		}
 
@@ -211,7 +196,7 @@ public:
 	virtual void OnLeave()
 	{
 		CScrollableDropTarget<wxTreeCtrlEx>::OnLeave();
-		ClearDropHighlight();
+		m_pRemoteTreeView->ClearDropHighlight();
 	}
 
 	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def)
@@ -289,7 +274,7 @@ void CRemoteTreeView::OnStateChange(t_statechange_notifications notification, st
 		ApplyFilters(false);
 	}
 	else if (notification == STATECHANGE_SERVER) {
-		m_windowTinter->SetBackgroundTint(m_state.GetSite().m_colour);
+		m_windowTinter->SetBackgroundTint(site_colour_to_wx(m_state.GetSite().m_colour));
 	}
 }
 
@@ -537,7 +522,7 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 
 	std::wstring const path = listing.path.GetPath();
 
-	CFilterDialog filter;
+	CFilterManager const& filter = m_state.GetStateFilterManager();
 
 	wxTreeItemId last;
 	for (size_t i = 0; i < listing.size(); ++i) {
@@ -604,7 +589,7 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 	std::vector<wxTreeItemId> toDelete;
 
 	bool inserted = false;
-	
+
 	wxTreeItemIdValue unused;
 	child = GetFirstChild(parent, unused);
 
@@ -1056,7 +1041,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent&)
 		pRecursiveOperation->SetChmodData(std::move(chmodData));
 
 		CFilterManager filter;
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_chmod, filter.GetActiveFilters());
+		pRecursiveOperation->StartRecursiveOperation(recursive_operation::recursive_chmod, filter.GetActiveFilters());
 	}
 }
 
@@ -1092,7 +1077,7 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 
 	const bool addOnly = event.GetId() == XRCID("ID_ADDTOQUEUE");
 	CFilterManager filter;
-	pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_transfer, filter.GetActiveFilters(), !addOnly);
+	pRecursiveOperation->StartRecursiveOperation(recursive_operation::recursive_transfer, filter.GetActiveFilters(), !addOnly);
 }
 
 void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
@@ -1155,7 +1140,7 @@ void CRemoteTreeView::OnMenuDelete(wxCommandEvent&)
 			m_state.ChangeRemoteDir(startDir);
 		}
 
-		pRecursiveOperation->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filter.GetActiveFilters());
+		pRecursiveOperation->StartRecursiveOperation(recursive_operation::recursive_delete, filter.GetActiveFilters());
 	}
 }
 
