@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "filezilla.h"
 #include "StatusView.h"
 #include "Options.h"
 #include "state.h"
@@ -6,6 +6,7 @@
 #include <libfilezilla/util.hpp>
 
 #include <wx/dcclient.h>
+#include <wx/menu.h>
 
 #define MAX_LINECOUNT 1000
 #define LINECOUNT_REMOVAL 10
@@ -91,6 +92,7 @@ END_EVENT_TABLE()
 
 
 CStatusView::CStatusView(wxWindow* parent, wxWindowID id)
+	: COptionChangeEventHandler(this)
 {
 	Create(parent, id, wxDefaultPosition, wxDefaultSize, wxSUNKEN_BORDER);
 	m_pTextCtrl = new CFastTextCtrl(this);
@@ -112,11 +114,12 @@ CStatusView::CStatusView(wxWindow* parent, wxWindowID id)
 
 	SetBackgroundStyle(wxBG_STYLE_SYSTEM);
 
-	RegisterOption(OPTION_MESSAGELOG_TIMESTAMP);
+	COptions::Get()->watch(OPTION_MESSAGELOG_TIMESTAMP, this);
 }
 
 CStatusView::~CStatusView()
 {
+	COptions::Get()->unwatch_all(this);
 }
 
 void CStatusView::OnSize(wxSizeEvent &)
@@ -127,12 +130,12 @@ void CStatusView::OnSize(wxSizeEvent &)
 	}
 }
 
-void CStatusView::AddToLog(CLogmsgNotification const& notification)
+void CStatusView::AddToLog(CLogmsgNotification && notification)
 {
-	AddToLog(notification.msgType, notification.msg, fz::datetime::now());
+	AddToLog(notification.msgType, std::move(notification.msg), std::move(notification.time_));
 }
 
-void CStatusView::AddToLog(logmsg::type messagetype, std::wstring const& message, fz::datetime const& time)
+void CStatusView::AddToLog(logmsg::type messagetype, std::wstring && message, fz::datetime const& time)
 {
 	if (!m_shown) {
 		if (m_hiddenLines.size() >= MAX_LINECOUNT) {
@@ -268,7 +271,7 @@ void CStatusView::AddToLog(logmsg::type messagetype, std::wstring const& message
 
 void CStatusView::InitDefAttr()
 {
-	m_showTimestamps = COptions::Get()->GetOptionVal(OPTION_MESSAGELOG_TIMESTAMP) != 0;
+	m_showTimestamps = COptions::Get()->get_int(OPTION_MESSAGELOG_TIMESTAMP) != 0;
 	m_lastTime = fz::datetime::now();
 	m_lastTimeString = m_lastTime.format(_T("%H:%M:%S\t"), fz::datetime::local);
 
@@ -435,7 +438,7 @@ void CStatusView::OnContextMenu(wxContextMenuEvent&)
 	menu.Append(XRCID("ID_COPYTOCLIPBOARD"), _("&Copy to clipboard"));
 	menu.Append(XRCID("ID_CLEARALL"), _("C&lear all"));
 
-	menu.Check(XRCID("ID_SHOW_DETAILED_LOG"), COptions::Get()->GetOptionVal(OPTION_LOGGING_SHOW_DETAILED_LOGS) != 0);
+	menu.Check(XRCID("ID_SHOW_DETAILED_LOG"), COptions::Get()->get_int(OPTION_LOGGING_SHOW_DETAILED_LOGS) != 0);
 
 	CState* pState = CContextManager::Get()->GetCurrentContext();
 	if (pState) {
@@ -451,7 +454,7 @@ void CStatusView::OnContextMenu(wxContextMenuEvent&)
 
 	PopupMenu(&menu);
 
-	COptions::Get()->SetOption(OPTION_LOGGING_SHOW_DETAILED_LOGS, menu.IsChecked(XRCID("ID_SHOW_DETAILED_LOG")) ? 1 : 0);
+	COptions::Get()->set(OPTION_LOGGING_SHOW_DETAILED_LOGS, menu.IsChecked(XRCID("ID_SHOW_DETAILED_LOG")) ? 1 : 0);
 }
 
 void CStatusView::OnClear(wxCommandEvent&)
@@ -465,13 +468,15 @@ void CStatusView::OnClear(wxCommandEvent&)
 
 void CStatusView::OnCopy(wxCommandEvent&)
 {
-	if (!m_pTextCtrl)
+	if (!m_pTextCtrl) {
 		return;
+	}
 
 	long from, to;
 	m_pTextCtrl->GetSelection(&from, &to);
-	if (from != to)
+	if (from != to) {
 		m_pTextCtrl->Copy();
+	}
 	else {
 		m_pTextCtrl->Freeze();
 		m_pTextCtrl->SetSelection(-1, -1);
@@ -486,7 +491,7 @@ void CStatusView::SetFocus()
 	m_pTextCtrl->SetFocus();
 }
 
-bool CStatusView::Show(bool show /*=true*/)
+bool CStatusView::Show(bool show)
 {
 	m_shown = show;
 
@@ -497,8 +502,8 @@ bool CStatusView::Show(bool show /*=true*/)
 			m_unusedLineLengths.splice(m_unusedLineLengths.end(), m_lineLengths, m_lineLengths.begin(), m_lineLengths.end());
 		}
 
-		for (auto const& line : m_hiddenLines) {
-			AddToLog(line.messagetype, line.message, line.time);
+		for (auto & line : m_hiddenLines) {
+			AddToLog(line.messagetype, std::move(line.message), line.time);
 		}
 		m_hiddenLines.clear();
 	}
@@ -506,7 +511,7 @@ bool CStatusView::Show(bool show /*=true*/)
 	return wxWindow::Show(show);
 }
 
-void CStatusView::OnOptionsChanged(changed_options_t const&)
+void CStatusView::OnOptionsChanged(watched_options const&)
 {
 	InitDefAttr();
 }
