@@ -12,6 +12,11 @@
 #include <wx/statbmp.h>
 #include <wx/statbox.h>
 
+CAboutDialog::CAboutDialog(COptionsBase& options)
+	: options_(options)
+{
+}
+
 bool CAboutDialog::Create(wxWindow* parent)
 {
 	wxDialogEx::Create(parent, -1, _("About FileZilla"));
@@ -22,7 +27,7 @@ bool CAboutDialog::Create(wxWindow* parent)
 	auto top = lay.createFlex(2);
 	main->Add(top);
 
-	top->Add(new wxStaticBitmap(this, -1, CThemeProvider::Get()->CreateBitmap("ART_FILEZILLA", wxString(), CThemeProvider::GetIconSize(iconSizeLarge))), 0, wxALL, lay.border);
+	top->Add(new wxStaticBitmap(this, -1, CThemeProvider::Get()->CreateBitmap("ART_FILEZILLA", wxString(), CThemeProvider::Get()->GetIconSize(iconSizeLarge))), 0, wxALL, lay.border);
 
 	auto topRight = lay.createFlex(1);
 	top->Add(topRight);
@@ -32,7 +37,7 @@ bool CAboutDialog::Create(wxWindow* parent)
 		version += L"-nightly";
 	}
 	topRight->Add(new wxStaticText(this, -1, version));
-	topRight->Add(new wxStaticText(this, -1, L"Copyright (C) 2004-2020  Tim Kosse"));
+	topRight->Add(new wxStaticText(this, -1, L"Copyright (C) 2004-2021  Tim Kosse"));
 
 	auto homepage = lay.createFlex(2);
 	homepage->Add(new wxStaticText(this, -1, _("Homepage:")), lay.valign);
@@ -87,16 +92,10 @@ bool CAboutDialog::Create(wxWindow* parent)
 			inner->Add(new wxStaticText(box, -1, os));
 		}
 
-		int major, minor;
-		if (GetRealOsVersion(major, minor)) {
+		auto sysver = GetSystemVersion();
+		if (sysver) {
 			inner->Add(new wxStaticText(box, -1, _("OS version:")));
-
-			wxString osVersion = wxString::Format(_T("%d.%d"), major, minor);
-			int fakeMajor, fakeMinor;
-			if (wxGetOsVersion(&fakeMajor, &fakeMinor) != wxOS_UNKNOWN && (fakeMajor != major || fakeMinor != minor)) {
-				osVersion += ' ';
-				osVersion += wxString::Format(_("(app-compat is set to %d.%d)"), fakeMajor, fakeMinor);
-			}
+			wxString osVersion = wxString::Format(_T("%u.%u"), sysver.major, sysver.minor);
 			inner->Add(new wxStaticText(box, -1, osVersion));
 		}
 #ifdef __WXMSW__
@@ -117,7 +116,7 @@ bool CAboutDialog::Create(wxWindow* parent)
 		}
 
 		inner->Add(new wxStaticText(box, -1, _("Settings directory:")));
-		inner->Add(new wxStaticText(box, -1, COptions::Get()->get_string(OPTION_DEFAULT_SETTINGSDIR)));
+		inner->Add(new wxStaticText(box, -1, options_.get_string(OPTION_DEFAULT_SETTINGSDIR)));
 	}
 
 	main->Add(new wxStaticLine(this), lay.grow);
@@ -192,15 +191,9 @@ void CAboutDialog::OnCopy()
 		text += _T("  Name:           ") + os + _T("\n");
 	}
 
-	int major, minor;
-	if (GetRealOsVersion(major, minor)) {
-		wxString version = wxString::Format(_T("%d.%d"), major, minor);
-		int fakeMajor, fakeMinor;
-		if (wxGetOsVersion(&fakeMajor, &fakeMinor) != wxOS_UNKNOWN && (fakeMajor != major || fakeMinor != minor)) {
-			version += _T(" ");
-			version += wxString::Format(_("(app-compat is set to %d.%d)"), fakeMajor, fakeMinor);
-		}
-		text += wxString::Format(_T("  Version:        %s\n"), version);
+	auto sysver = GetSystemVersion();
+	if (sysver) {
+		text += wxString::Format(_T("  Version:        %u.%u\n"), sysver.major, sysver.minor);
 	}
 
 #ifdef __WXMSW__
@@ -217,7 +210,7 @@ void CAboutDialog::OnCopy()
 		text += _T("  CPU features:   ") + cpuCaps + _T("\n");
 	}
 
-	text += _T("  Settings dir:   ") + COptions::Get()->get_string(OPTION_DEFAULT_SETTINGSDIR) + _T("\n");
+	text += _T("  Settings dir:   ") + options_.get_string(OPTION_DEFAULT_SETTINGSDIR) + _T("\n");
 
 #ifdef __WXMSW__
 	text.Replace(_T("\n"), _T("\r\n"));
