@@ -20,9 +20,10 @@ EVT_COMMAND(wxID_ANY, fzEVT_PROCESSASYNCREQUESTQUEUE, CAsyncRequestQueue::OnProc
 EVT_TIMER(wxID_ANY, CAsyncRequestQueue::OnTimer)
 END_EVENT_TABLE()
 
-CAsyncRequestQueue::CAsyncRequestQueue(wxTopLevelWindow *parent)
+CAsyncRequestQueue::CAsyncRequestQueue(wxTopLevelWindow *parent, COptionsBase & options, cert_store & certStore)
 	: parent_(parent)
-	, certStore_(std::make_unique<CertStore>())
+	, options_(options)
+	, certStore_(certStore)
 {
 	CContextManager::Get()->RegisterHandler(this, STATECHANGE_REMOVECONTEXT, false);
 	m_timer.SetOwner(this);
@@ -48,7 +49,7 @@ bool CAsyncRequestQueue::ProcessDefaults(CFileZillaEngine *pEngine, std::unique_
 				action = CDefaultFileExistsDlg::GetDefault(pFileExistsNotification->download);
 			}
 			if (action == CFileExistsNotification::unknown) {
-				int option = COptions::Get()->get_int(pFileExistsNotification->download ? OPTION_FILEEXISTS_DOWNLOAD : OPTION_FILEEXISTS_UPLOAD);
+				int option = options_.get_int(pFileExistsNotification->download ? OPTION_FILEEXISTS_DOWNLOAD : OPTION_FILEEXISTS_UPLOAD);
 				if (option < CFileExistsNotification::unknown || option >= CFileExistsNotification::ACTION_COUNT) {
 					action = CFileExistsNotification::unknown;
 				}
@@ -64,7 +65,7 @@ bool CAsyncRequestQueue::ProcessDefaults(CFileZillaEngine *pEngine, std::unique_
 
 			if (action == CFileExistsNotification::resume && pFileExistsNotification->ascii) {
 				// Check if resuming ascii files is allowed
-				if (!COptions::Get()->get_int(OPTION_ASCIIRESUME)) {
+				if (!options_.get_int(OPTION_ASCIIRESUME)) {
 					// Overwrite instead
 					action = CFileExistsNotification::overwrite;
 				}
@@ -96,8 +97,10 @@ bool CAsyncRequestQueue::ProcessDefaults(CFileZillaEngine *pEngine, std::unique_
 		{
 			auto & certNotification = static_cast<CCertificateNotification&>(*pNotification.get());
 
-			if (!certStore_->IsTrusted(certNotification.info_)) {
-				break;
+			if (!certNotification.info_.system_trust() || !options_.get_bool(OPTION_TRUST_SYSTEM_TRUST_STORE)) {
+				if (!certStore_.IsTrusted(certNotification.info_)) {
+					break;
+				}
 			}
 
 			certNotification.trusted_ = true;
@@ -109,13 +112,26 @@ bool CAsyncRequestQueue::ProcessDefaults(CFileZillaEngine *pEngine, std::unique_
 	case reqId_insecure_connection:
 		{
 			auto & insecureNotification = static_cast<CInsecureConnectionNotification&>(*pNotification.get());
-			if (!certStore_->IsInsecure(fz::to_utf8(insecureNotification.server_.GetHost()), insecureNotification.server_.GetPort())) {
+			if (!certStore_.IsInsecure(fz::to_utf8(insecureNotification.server_.GetHost()), insecureNotification.server_.GetPort())) {
 				break;
 			}
 
 			insecureNotification.allow_ = true;
 			pEngine->SetAsyncRequestReply(std::move(pNotification));
 
+			return true;
+		}
+	case reqId_tls_no_resumption:
+		{
+			auto & notification = static_cast<FtpTlsNoResumptionNotification&>(*pNotification.get());
+
+			auto v = certStore_.GetSessionResumptionSupport(fz::to_utf8(notification.server_.GetHost()), notification.server_.GetPort());
+			if (!v || *v) {
+				break;
+			}
+			
+			notification.allow_ = true;
+			pEngine->SetAsyncRequestReply(std::move(pNotification));
 			return true;
 		}
 	default:
@@ -167,9 +183,10 @@ bool CAsyncRequestQueue::ProcessNextRequest()
 			CLoginManager::Get().CachedPasswordFailed(notification.server, notification.GetChallenge());
 		}
 		bool canRemember = notification.GetType() == CInteractiveLoginNotification::keyfile;
+		bool otp = notification.GetType() == CInteractiveLoginNotification::totp;
 
 		Site site(notification.server, notification.handle_, notification.credentials);
-		if (CLoginManager::Get().GetPassword(site, true, notification.GetChallenge(), canRemember)) {
+		if (CLoginManager::Get().GetPassword(site, true, notification.GetChallenge(), otp, canRemember)) {
 			notification.credentials = site.credentials;
 			notification.passwordSet = true;
 		}
@@ -180,7 +197,7 @@ bool CAsyncRequestQueue::ProcessNextRequest()
 				return false;
 			}
 
-			if (CLoginManager::Get().GetPassword(site, false, notification.GetChallenge(), canRemember)) {
+			if (CLoginManager::Get().GetPassword(site, false, notification.GetChallenge(), otp, canRemember)) {
 				notification.credentials = site.credentials;
 				notification.passwordSet = true;
 			}
@@ -210,10 +227,8 @@ bool CAsyncRequestQueue::ProcessNextRequest()
 			return false;
 		}
 
-		if (certStore_) {
-			auto & notification = static_cast<CCertificateNotification&>(*entry.pNotification.get());
-			CVerifyCertDialog::ShowVerificationDialog(*certStore_, notification);
-		}
+		auto & notification = static_cast<CCertificateNotification&>(*entry.pNotification.get());
+		CVerifyCertDialog::ShowVerificationDialog(certStore_, notification);
 
 		SendReply(entry);
 	}
@@ -224,7 +239,24 @@ bool CAsyncRequestQueue::ProcessNextRequest()
 
 		auto & notification = static_cast<CInsecureConnectionNotification&>(*entry.pNotification.get());
 
-		ConfirmInsecureConection(parent_, *certStore_.get(), notification);
+		ConfirmInsecureConection(parent_, certStore_, notification);
+
+		SendReply(entry);
+	}
+	else if (entry.pNotification->GetRequestID() == reqId_tls_no_resumption) {
+		if (!CheckWindowState()) {
+			return false;
+		}
+
+		auto & notification = static_cast<FtpTlsNoResumptionNotification&>(*entry.pNotification.get());
+
+		auto v = certStore_.GetSessionResumptionSupport(fz::to_utf8(notification.server_.GetHost()), notification.server_.GetPort());
+		if (v && !*v) {
+			notification.allow_ = true;
+		}
+		else {
+			ConfirmFtpTlsNoResumptionNotification(parent_, certStore_, notification);
+		}
 
 		SendReply(entry);
 	}
@@ -247,7 +279,7 @@ bool CAsyncRequestQueue::ProcessFileExistsNotification(t_queueEntry &entry)
 		action = CDefaultFileExistsDlg::GetDefault(notification.download);
 	}
 	if (action == CFileExistsNotification::unknown) {
-		int option = COptions::Get()->get_int(notification.download ? OPTION_FILEEXISTS_DOWNLOAD : OPTION_FILEEXISTS_UPLOAD);
+		int option = options_.get_int(notification.download ? OPTION_FILEEXISTS_DOWNLOAD : OPTION_FILEEXISTS_UPLOAD);
 		if (option <= CFileExistsNotification::unknown || option >= CFileExistsNotification::ACTION_COUNT) {
 			action = CFileExistsNotification::ask;
 		}
@@ -322,7 +354,7 @@ bool CAsyncRequestQueue::ProcessFileExistsNotification(t_queueEntry &entry)
 
 	if (action == CFileExistsNotification::resume && notification.ascii) {
 		// Check if resuming ascii files is allowed
-		if (!COptions::Get()->get_int(OPTION_ASCIIRESUME)) {
+		if (!options_.get_int(OPTION_ASCIIRESUME)) {
 			// Overwrite instead
 			action = CFileExistsNotification::overwrite;
 		}
