@@ -1,7 +1,6 @@
 #include "filezilla.h"
 #include "clearprivatedata.h"
 #include "commandqueue.h"
-#include "ipcmutex.h"
 #include "local_recursive_operation.h"
 #include "Mainfrm.h"
 #include "Options.h"
@@ -11,6 +10,8 @@
 #include "remote_recursive_operation.h"
 #include "state.h"
 
+#include "../commonui/ipcmutex.h"
+
 #include <libfilezilla/file.hpp>
 
 BEGIN_EVENT_TABLE(CClearPrivateDataDialog, wxDialogEx)
@@ -109,9 +110,6 @@ void CClearPrivateDataDialog::Run()
 	if (pQueueCheck->GetValue()) {
 		m_pMainFrame->GetQueue()->SetActive(false);
 		m_pMainFrame->GetQueue()->RemoveAll();
-
-		CInterProcessMutex mutex(MUTEX_QUEUE);
-		RemoveXmlFile(L"queue");
 	}
 }
 
