@@ -117,7 +117,7 @@ bool CContextControl::CreateTab(CLocalPath const& localPath, Site const& site, C
 				return false;
 			}
 		}
-	
+
 		pState->SetLastSite(site, remotePath);
 
 		CreateContextControls(*pState);
@@ -185,7 +185,7 @@ void CContextControl::CreateContextControls(CState& state)
 		if (currentControls.pRemoteListView) {
 			currentControls.pRemoteListView->SaveColumnSettings(OPTION_REMOTEFILELIST_COLUMN_WIDTHS, OPTION_REMOTEFILELIST_COLUMN_SHOWN, OPTION_REMOTEFILELIST_COLUMN_ORDER);
 		}
-		
+
 		if (!m_tabs) {
 			m_tabs = new wxAuiNotebookEx();
 
@@ -197,7 +197,7 @@ void CContextControl::CreateContextControls(CState& state)
 			currentControls.pViewSplitter->Reparent(m_tabs);
 
 			m_tabs->AddPage(currentControls.pViewSplitter, currentControls.pState->GetTitle());
-			m_tabs->SetTabColour(0, currentControls.pState->GetSite().m_colour);
+			m_tabs->SetTabColour(0, site_colour_to_wx(currentControls.pState->GetSite().m_colour));
 			ReplaceWindow(currentControls.pViewSplitter, m_tabs);
 
 			m_tabs->Connect(wxEVT_COMMAND_AUINOTEBOOK_PAGE_CHANGED, wxAuiNotebookEventHandler(CContextControl::OnTabChanged), 0, this);
@@ -234,18 +234,18 @@ void CContextControl::CreateContextControls(CState& state)
 	context_controls.pLocalTreeViewPanel = new CView(context_controls.pLocalSplitter);
 	context_controls.pLocalListViewPanel = new CView(context_controls.pLocalSplitter);
 	context_controls.pLocalTreeView = new CLocalTreeView(context_controls.pLocalTreeViewPanel, -1, state, m_mainFrame.GetQueue());
-	context_controls.pLocalListView = new CLocalListView(context_controls.pLocalListViewPanel, state, m_mainFrame.GetQueue());
+	context_controls.pLocalListView = new CLocalListView(context_controls.pLocalListViewPanel, state, m_mainFrame.GetQueue(), m_mainFrame.GetOptions());
 	context_controls.pLocalTreeViewPanel->SetWindow(context_controls.pLocalTreeView);
 	context_controls.pLocalListViewPanel->SetWindow(context_controls.pLocalListView);
 
 	context_controls.pRemoteTreeViewPanel = new CView(context_controls.pRemoteSplitter);
 	context_controls.pRemoteListViewPanel = new CView(context_controls.pRemoteSplitter);
 	context_controls.pRemoteTreeView = new CRemoteTreeView(context_controls.pRemoteTreeViewPanel, -1, state, m_mainFrame.GetQueue());
-	context_controls.pRemoteListView = new CRemoteListView(context_controls.pRemoteListViewPanel, state, m_mainFrame.GetQueue());
+	context_controls.pRemoteListView = new CRemoteListView(context_controls.pRemoteListViewPanel, state, m_mainFrame.GetQueue(), m_mainFrame.GetOptions());
 	context_controls.pRemoteTreeViewPanel->SetWindow(context_controls.pRemoteTreeView);
 	context_controls.pRemoteListViewPanel->SetWindow(context_controls.pRemoteListView);
 
-	bool show_filelist_statusbars = COptions::Get()->get_int(OPTION_FILELIST_STATUSBAR) != 0;
+	bool show_filelist_statusbars = m_mainFrame.GetOptions().get_int(OPTION_FILELIST_STATUSBAR) != 0;
 
 	CFilelistStatusBar* pLocalFilelistStatusBar = new CFilelistStatusBar(context_controls.pLocalListViewPanel);
 	if (!show_filelist_statusbars) {
@@ -278,8 +278,8 @@ void CContextControl::CreateContextControls(CState& state)
 	context_controls.pRemoteListSearchPanel = new CListSearchPanel(context_controls.pRemoteListViewPanel, context_controls.pRemoteListView, context_controls.pState, false);
 	context_controls.pRemoteListViewPanel->SetSearchPanel(context_controls.pRemoteListSearchPanel);
 
-	const int layout = COptions::Get()->get_int(OPTION_FILEPANE_LAYOUT);
-	const int swap = COptions::Get()->get_int(OPTION_FILEPANE_SWAP);
+	const int layout = m_mainFrame.GetOptions().get_int(OPTION_FILEPANE_LAYOUT);
+	const int swap = m_mainFrame.GetOptions().get_int(OPTION_FILEPANE_SWAP);
 
 	if (layout == 1) {
 		if (swap) {
@@ -298,7 +298,7 @@ void CContextControl::CreateContextControls(CState& state)
 		}
 	}
 
-	if (COptions::Get()->get_int(OPTION_SHOW_TREE_LOCAL)) {
+	if (m_mainFrame.GetOptions().get_int(OPTION_SHOW_TREE_LOCAL)) {
 		context_controls.pLocalViewHeader = new CLocalViewHeader(context_controls.pLocalTreeViewPanel, state);
 		context_controls.pLocalTreeViewPanel->SetHeader(context_controls.pLocalViewHeader);
 		if (layout == 3 && swap) {
@@ -318,7 +318,7 @@ void CContextControl::CreateContextControls(CState& state)
 		context_controls.pLocalSplitter->Initialize(context_controls.pLocalListViewPanel);
 	}
 
-	if (COptions::Get()->get_int(OPTION_SHOW_TREE_REMOTE)) {
+	if (m_mainFrame.GetOptions().get_int(OPTION_SHOW_TREE_REMOTE)) {
 		context_controls.pRemoteViewHeader = new CRemoteViewHeader(context_controls.pRemoteTreeViewPanel, state);
 		context_controls.pRemoteTreeViewPanel->SetHeader(context_controls.pRemoteViewHeader);
 		if (layout == 3 && !swap) {
@@ -662,7 +662,7 @@ void CContextControl::OnStateChange(CState* pState, t_statechange_notifications
 		if (controls && controls->used()) {
 			int i = m_tabs->GetPageIndex(controls->pViewSplitter);
 			if (i != wxNOT_FOUND) {
-				m_tabs->SetTabColour(i, controls->pState->GetSite().m_colour);
+				m_tabs->SetTabColour(i, site_colour_to_wx(controls->pState->GetSite().m_colour));
 				m_tabs->SetPageText(i, controls->pState->GetTitle());
 			}
 		}
@@ -706,7 +706,7 @@ void CContextControl::SaveTabs()
 		}
 	}
 
-	COptions::Get()->set(OPTION_TAB_DATA, xml);
+	m_mainFrame.GetOptions().set(OPTION_TAB_DATA, xml);
 }
 
 void CContextControl::RestoreTabs()
@@ -717,9 +717,9 @@ void CContextControl::RestoreTabs()
 
 	int selected = 0;
 
-	auto xml = COptions::Get()->get_xml(OPTION_TAB_DATA);
+	auto xml = m_mainFrame.GetOptions().get_xml(OPTION_TAB_DATA);
 
-	bool selectedOnly = COptions::Get()->get_int(OPTION_STARTUP_ACTION) != 2;
+	bool selectedOnly = m_mainFrame.GetOptions().get_int(OPTION_STARTUP_ACTION) != 2;
 
 	CCommandLine const* pCommandLine = wxGetApp().GetCommandLine();
 	if (pCommandLine && pCommandLine->BlocksReconnectAtStartup()) {
