@@ -251,9 +251,9 @@ public:
 		}
 	}
 
-	inline int CmpStringNoCase(const wxString &data1, const wxString &data2) const
+	inline int CmpStringNoCase(std::wstring_view const& data1, std::wstring_view const& data2) const
 	{
-		return data1.CmpNoCase(data2);
+		return fz::stricmp(data1, data2);
 	}
 
 	inline int CmpTime(const value_type &data1, const value_type &data2) const
@@ -499,11 +499,12 @@ namespace genericTypes {
 	};
 }
 
+class COptionsBase;
 template<class CFileData> class CFileListCtrl : public wxListCtrlEx, public CComparableListing
 {
 	template<typename Listing, typename DataEntry> friend class CFileListCtrlSortType;
 public:
-	CFileListCtrl(wxWindow* pParent, CQueueView *pQueue, bool border = false);
+	CFileListCtrl(wxWindow* pParent, CQueueView *pQueue, COptionsBase & options, bool border = false);
 	virtual ~CFileListCtrl() = default;
 
 	void SetFilelistStatusBar(CFilelistStatusBar* pFilelistStatusBar) { m_pFilelistStatusBar = pFilelistStatusBar; }
@@ -525,7 +526,7 @@ protected:
 	virtual bool ItemIsDir(int index) const = 0;
 	virtual int64_t ItemGetSize(int index) const = 0;
 
-	std::map<wxString, std::wstring> m_fileTypeMap;
+	std::map<std::wstring, std::wstring> m_fileTypeMap;
 
 	// The .. item
 	bool m_hasParent{true};
@@ -537,10 +538,11 @@ protected:
 	void SortList(int column = -1, int direction = -1, bool updateSelections = true);
 	CFileListCtrlSortBase::DirSortMode GetDirSortMode();
 	NameSortMode GetNameSortMode();
-	virtual std::unique_ptr<CFileListCtrlSortBase> GetSortComparisonObject() = 0;
+	virtual void UpdateSortComparisonObject() = 0;
+	CFileListCtrlSortBase& GetSortComparisonObject();
 
 	// An empty path denotes a virtual file
-	std::wstring GetType(std::wstring name, bool dir, std::wstring const& path = std::wstring());
+	std::wstring GetType(std::wstring const& name, bool dir, std::wstring const& path = std::wstring());
 
 	// Comparison related
 	virtual void ScrollTopItem(int item);
@@ -611,6 +613,10 @@ private:
 	void OnProcessMouseEvent(wxCommandEvent& event);
 #endif
 	void OnKeyDown(wxKeyEvent& event);
+
+	COptionsBase& options_;
+
+	std::unique_ptr<CFileListCtrlSortBase> sortComparisonObject_;
 };
 
 class SortPredicate
@@ -618,15 +624,18 @@ class SortPredicate
 public:
 	SortPredicate() = delete;
 	SortPredicate(std::unique_ptr<CFileListCtrlSortBase> const& ref)
-		: p_(ref.get())
+		: p_(*ref)
+	{}
+	SortPredicate(CFileListCtrlSortBase const& ref)
+	    : p_(ref)
 	{}
 
 	inline bool operator()(int lhs, int rhs) {
-		return (*p_)(lhs, rhs);
+		return p_(lhs, rhs);
 	}
 
 public:
-	CFileListCtrlSortBase const* p_;
+	CFileListCtrlSortBase const& p_;
 };
 
 #ifdef FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
