@@ -6,9 +6,10 @@
 
 #include <vector>
 
-class CWrapEngine;
 class CCommandLine;
+class COptions;
 class CThemeProvider;
+class CWrapEngine;
 
 class CFileZillaApp : public wxApp
 {
@@ -21,7 +22,6 @@ public:
 
 	// Always (back)slash-terminated
 	CLocalPath GetResourceDir() const { return m_resourceDir; }
-	CLocalPath GetDefaultsDir() const { return m_defaultsDir; }
 	CLocalPath GetLocalesDir() const { return m_localesDir; }
 
 	std::wstring GetSettingsFile(std::wstring const& name) const;
@@ -48,7 +48,6 @@ public:
 protected:
 	void CheckExistsTool(std::wstring const& tool, std::wstring const& buildRelPath, char const* env, engineOptions setting, std::wstring const& description);
 
-	bool InitDefaultsDir();
 	bool LoadResourceFiles();
 	bool LoadLocales();
 	int ProcessCommandLine();
@@ -56,17 +55,13 @@ protected:
 	std::unique_ptr<wxLocale> m_pLocale;
 
 	CLocalPath m_resourceDir;
-	CLocalPath m_defaultsDir;
 	CLocalPath m_localesDir;
 
 #if wxUSE_DEBUGREPORT && wxUSE_ON_FATAL_EXCEPTION
 	virtual void OnFatalException();
 #endif
 
-	CLocalPath GetDataDir(std::vector<std::wstring> const& fileToFind, std::wstring const& prefixSub, bool searchSelfDir = true) const;
-
-	bool FileExists(std::wstring const& file) const;
-
+	std::unique_ptr<COptions> options_;
 	std::unique_ptr<CWrapEngine> m_pWrapEngine;
 
 	std::unique_ptr<CCommandLine> m_pCommandLine;
