@@ -1,13 +1,14 @@
 #include "filezilla.h"
 #include "filezillaapp.h"
 #include "import.h"
-#include "ipcmutex.h"
 #include "sitemanager.h"
 #include "xmlfunctions.h"
 #include "Options.h"
 #include "queue.h"
 #include "xrc_helper.h"
 
+#include "../commonui/ipcmutex.h"
+
 #include <wx/filedlg.h>
 
 CImportDialog::CImportDialog(wxWindow* parent, CQueueView* pQueueView)
