@@ -1,75 +1,15 @@
 #include "filezilla.h"
 #include "loginmanager.h"
-
 #include "dialogex.h"
 #include "filezillaapp.h"
 #include "Options.h"
 #include "textctrlex.h"
 
-#include <algorithm>
 
 CLoginManager CLoginManager::m_theLoginManager;
 
-std::list<CLoginManager::t_passwordcache>::iterator CLoginManager::FindItem(CServer const& server, std::wstring const& challenge)
-{
-	return std::find_if(m_passwordCache.begin(), m_passwordCache.end(), [&](t_passwordcache const& item)
-		{
-			return item.host == server.GetHost() && item.port == server.GetPort() && item.user == server.GetUser() && item.challenge == challenge;
-		}
-	);
-}
 
-bool CLoginManager::GetPassword(Site & site, bool silent)
-{
-	bool const needsUser = ProtocolHasUser(site.server.GetProtocol()) && site.server.GetUser().empty() && (site.credentials.logonType_ == LogonType::ask || site.credentials.logonType_ == LogonType::interactive);
-
-	if (site.credentials.logonType_ != LogonType::ask && !site.credentials.encrypted_ && !needsUser) {
-		return true;
-	}
-
-	if (site.credentials.encrypted_) {
-		auto priv = GetDecryptor(site.credentials.encrypted_);
-		if (priv) {
-			return site.credentials.Unprotect(priv);
-		}
-
-		if (!silent) {
-			return DisplayDialogForEncrypted(site);
-		}
-	}
-	else {
-		auto it = FindItem(site.server, std::wstring());
-		if (it != m_passwordCache.end()) {
-			site.credentials.SetPass(it->password);
-			return true;
-		}
-
-		if (!silent) {
-			return DisplayDialog(site, std::wstring(), true);
-		}
-	}
-
-	return false;
-}
-
-
-bool CLoginManager::GetPassword(Site & site, bool silent, std::wstring const& challenge, bool canRemember)
-{
-	if (canRemember) {
-		auto it = FindItem(site.server, challenge);
-		if (it != m_passwordCache.end()) {
-			site.credentials.SetPass(it->password);
-			return true;
-		}
-	}
-	if (silent) {
-		return false;
-	}
-
-	return DisplayDialog(site, challenge, canRemember);
-}
-
-bool CLoginManager::DisplayDialogForEncrypted(Site & site)
+bool CLoginManager::query_unprotect_site(Site & site)
 {
 	assert(site.credentials.encrypted_);
 
@@ -84,14 +24,14 @@ bool CLoginManager::DisplayDialogForEncrypted(Site & site)
 
 	auto* inner = lay.createFlex(2);
 	main->Add(inner);
-	
+
 	std::wstring const& name = site.GetName();
 	if (!name.empty()) {
 		inner->Add(new wxStaticText(&pwdDlg, -1, _("Name:")));
 		inner->Add(new wxStaticText(&pwdDlg, -1, LabelEscape(name)));
 	}
 
-	if (site.server.GetProtocol() == STORJ) {
+	if (site.server.GetProtocol() == STORJ || site.server.GetProtocol() == STORJ_GRANT) {
 		inner->Add(new wxStaticText(&pwdDlg, -1, _("Satellite:")));
 	}
 	else {
@@ -137,7 +77,7 @@ bool CLoginManager::DisplayDialogForEncrypted(Site & site)
 	buttons->AddButton(cancel);
 
 	buttons->Realize();
-	
+
 	pwdDlg.GetSizer()->Fit(&pwdDlg);
 	pwdDlg.GetSizer()->SetSizeHints(&pwdDlg);
 
@@ -154,7 +94,7 @@ bool CLoginManager::DisplayDialogForEncrypted(Site & site)
 			continue;
 		}
 
-		if (!site.credentials.Unprotect(key)) {
+		if (!unprotect(site.credentials, key)) {
 			wxMessageBoxEx(_("Failed to decrypt server password."), _("Invalid input"), wxICON_EXCLAMATION);
 			continue;
 		}
@@ -168,27 +108,41 @@ bool CLoginManager::DisplayDialogForEncrypted(Site & site)
 	return true;
 }
 
-bool CLoginManager::DisplayDialog(Site & site, std::wstring const& challenge, bool canRemember)
+bool CLoginManager::query_credentials(Site & site, std::wstring const& challenge, bool otp, bool canRemember)
 {
 	assert(!site.credentials.encrypted_);
 
+	bool needs_user{};
+	bool needs_pass{};
+	bool needs_otp{};
+
 	wxString title;
 	wxString header;
-	if (site.server.GetUser().empty()) {
+	if (site.server.GetUser().empty() && ProtocolHasUser(site.server.GetProtocol())) {
+		needs_user = true;
 		if (site.credentials.logonType_ == LogonType::interactive) {
 			title = _("Enter username");
 			header = _("Please enter a username for this server:");
-
 			canRemember = false;
 		}
 		else {
 			title = _("Enter username and password");
 			header = _("Please enter username and password for this server:");
+			needs_pass = true;
 		}
 	}
 	else {
-		title = _("Enter password");
-		header = _("Please enter a password for this server:");
+		if (otp) {
+			needs_otp = true;
+			title = _("Enter the 2FA code");
+			header = _("Please enter the 2FA code for this server:");
+			needs_pass = site.credentials.logonType_ == LogonType::interactive;
+		}
+		else {
+			needs_pass = true;
+			title = _("Enter password");
+			header = _("Please enter a password for this server:");
+		}
 	}
 
 	wxDialogEx pwdDlg;
@@ -197,7 +151,7 @@ bool CLoginManager::DisplayDialog(Site & site, std::wstring const& challenge, bo
 	}
 	auto& lay = pwdDlg.layout();
 	auto* main = lay.createMain(&pwdDlg, 1);
-	
+
 	main->Add(new wxStaticText(&pwdDlg, -1, header));
 
 	auto* inner = lay.createFlex(2);
@@ -217,15 +171,15 @@ bool CLoginManager::DisplayDialog(Site & site, std::wstring const& challenge, bo
 		inner->Add(new wxStaticText(&pwdDlg, -1, LabelEscape(site.server.GetUser())));
 	}
 
-	if (!challenge.empty()) {
-		std::wstring displayChallenge = LabelEscape(fz::trimmed(challenge));
+	if (!challenge.empty() && !otp) {
+		std::wstring displayChallenge = fz::trimmed(challenge);
 #ifdef FZ_WINDOWS
 		fz::replace_substrings(displayChallenge, L"\n", L"\r\n");
 #endif
 		main->AddSpacer(0);
 		main->Add(new wxStaticText(&pwdDlg, -1, _("Challenge:")));
 		auto* challengeText = new wxTextCtrlEx(&pwdDlg, -1, displayChallenge, wxDefaultPosition, wxDefaultSize, wxTE_MULTILINE | wxTE_READONLY);
-		challengeText->SetMinSize(wxSize(lay.dlgUnits(160), lay.dlgUnits(50)));
+		challengeText->SetMinSize(wxSize(lay.dlgUnits(240), lay.dlgUnits(60)));
 		challengeText->SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE));
 		main->Add(challengeText, lay.grow);
 	}
@@ -236,7 +190,7 @@ bool CLoginManager::DisplayDialog(Site & site, std::wstring const& challenge, bo
 	main->Add(inner);
 
 	wxTextCtrl* newUser{};
-	if (site.server.GetUser().empty()) {
+	if (needs_user) {
 		inner->Add(new wxStaticText(&pwdDlg, -1, _("&User:")), lay.valign);
 		newUser = new wxTextCtrlEx(&pwdDlg, -1, wxString());
 		newUser->SetMinSize(wxSize(150, -1));
@@ -246,7 +200,7 @@ bool CLoginManager::DisplayDialog(Site & site, std::wstring const& challenge, bo
 
 	wxTextCtrl* password{};
 	wxTextCtrl* key{}; // for storj
-	if (!site.server.GetUser().empty() || site.credentials.logonType_ != LogonType::interactive) {
+	if (needs_pass) {
 		inner->Add(new wxStaticText(&pwdDlg, -1, _("&Password:")), lay.valign);
 		password = new wxTextCtrlEx(&pwdDlg, -1, wxString(), wxDefaultPosition, wxDefaultSize, wxTE_PASSWORD);
 		password->SetMinSize(wxSize(150, -1));
@@ -263,8 +217,16 @@ bool CLoginManager::DisplayDialog(Site & site, std::wstring const& challenge, bo
 		}
 	}
 
+	wxTextCtrl* otpCode{};
+	if (needs_otp) {
+		inner->Add(new wxStaticText(&pwdDlg, -1, _("&Token code:")), lay.valign);
+		otpCode = new wxTextCtrlEx(&pwdDlg, -1, wxString());
+		otpCode->SetMinSize(wxSize(150, -1));
+		inner->Add(otpCode, lay.valign);
+	}
+
 	wxCheckBox* remember{};
-	if (canRemember) {
+	if (canRemember && needs_pass) {
 		main->AddSpacer(0);
 		remember = new wxCheckBox(&pwdDlg, -1, _("&Remember password until FileZilla is closed"));
 		remember->SetValue(true);
@@ -320,40 +282,22 @@ bool CLoginManager::DisplayDialog(Site & site, std::wstring const& challenge, bo
 		if (remember && remember->IsChecked()) {
 			RememberPassword(site, challenge);
 		}
+
+		if (otpCode) {
+			auto code = otpCode->GetValue().ToStdWstring();
+			if (code.empty()) {
+				wxMessageBoxEx(_("No code given."), _("Invalid input"), wxICON_EXCLAMATION);
+				continue;
+			}
+			site.credentials.SetExtraParameter(site.server.GetProtocol(), "otp_code", code);
+		}
+
 		break;
 	}
 
 	return true;
 }
 
-void CLoginManager::CachedPasswordFailed(CServer const& server, std::wstring const& challenge)
-{
-	auto it = FindItem(server, challenge);
-	if (it != m_passwordCache.end()) {
-		m_passwordCache.erase(it);
-	}
-}
-
-void CLoginManager::RememberPassword(Site & site, std::wstring const& challenge)
-{
-	if (site.credentials.logonType_ == LogonType::anonymous) {
-		return;
-	}
-
-	auto it = FindItem(site.server, challenge);
-	if (it != m_passwordCache.end()) {
-		it->password = site.credentials.GetPass();
-	}
-	else {
-		t_passwordcache entry;
-		entry.host = site.server.GetHost();
-		entry.port = site.server.GetPort();
-		entry.user = site.server.GetUser();
-		entry.password = site.credentials.GetPass();
-		entry.challenge = challenge;
-		m_passwordCache.push_back(entry);
-	}
-}
 
 bool CLoginManager::AskDecryptor(fz::public_key const& pub, bool allowForgotten, bool allowCancel)
 {
@@ -443,38 +387,3 @@ bool CLoginManager::AskDecryptor(fz::public_key const& pub, bool allowForgotten,
 
 	return true;
 }
-
-fz::private_key CLoginManager::GetDecryptor(fz::public_key const& pub, bool * forgotten)
-{
-	auto it = decryptors_.find(pub);
-	if (it != decryptors_.cend()) {
-		if (!it->second && forgotten) {
-			*forgotten = true;
-		}
-		return it->second;
-	}
-
-	for (auto const& pw : decryptorPasswords_) {
-		auto priv = fz::private_key::from_password(pw, pub.salt_);
-		if (priv && priv.pubkey() == pub) {
-			decryptors_[pub] = priv;
-			return priv;
-		}
-	}
-
-	return fz::private_key();
-}
-
-void CLoginManager::Remember(const fz::private_key &key, std::string_view const& pass)
-{
-	decryptors_[key.pubkey()] = key;
-
-	if (!pass.empty()) {
-		for (auto const& pw : decryptorPasswords_) {
-			if (pw == pass) {
-				return;
-			}
-		}
-		decryptorPasswords_.emplace_back(pass);
-	}
-}
