@@ -1,5 +1,6 @@
 #include "filezilla.h"
 #include "Options.h"
+#include "Mainfrm.h"
 #include "queue.h"
 #include "queueview_failed.h"
 #include "queueview_successful.h"
@@ -639,7 +640,7 @@ CFileItem* DoGetIdleChild(std::deque<CFileItem*> const* fileList, TransferDirect
 CFileItem* CServerItem::GetIdleChild(bool immediateOnly, TransferDirection direction)
 {
 	CFileItem* item = DoGetIdleChild(m_fileList[1], direction);
-	if( !item && !immediateOnly ) {
+	if ( !item && !immediateOnly ) {
 		item = DoGetIdleChild(m_fileList[0], direction);
 	}
 	return item;
@@ -846,8 +847,9 @@ EVT_KEY_DOWN(CQueueViewBase::OnKeyDown)
 EVT_MENU(XRCID("ID_EXPORT"), CQueueViewBase::OnExport)
 END_EVENT_TABLE()
 
-CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
+CQueueViewBase::CQueueViewBase(CQueue* parent, COptionsBase & options, int index, const wxString& title)
 	: wxListCtrlEx(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxCLIP_CHILDREN | wxLC_REPORT | wxLC_VIRTUAL | wxSUNKEN_BORDER | wxTAB_TRAVERSAL)
+	, options_(options)
 	, m_pageIndex(index)
 	, m_title(title)
 {
@@ -857,9 +859,9 @@ CQueueViewBase::CQueueViewBase(CQueue* parent, int index, const wxString& title)
 	wxSize s = CThemeProvider::GetIconSize(iconSizeSmall);
 	wxImageList* pImageList = new wxImageList(s.x, s.y);
 
-	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_SERVER"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
-	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FILE"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
-	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDER"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall)));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_SERVER"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall), true));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FILE"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall), true));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDER"), wxART_OTHER, CThemeProvider::GetIconSize(iconSizeSmall), true));
 
 	AssignImageList(pImageList, wxIMAGE_LIST_SMALL);
 
@@ -1568,29 +1570,48 @@ void CQueueViewBase::OnExport(wxCommandEvent&)
 
 	WriteToFile(exportRoot);
 
-	xml.Save(true);
+	SaveWithErrorDialog(xml);
 }
 
 // ------
 // CQueue
 // ------
 
-CQueue::CQueue(wxWindow* parent, CMainFrame *pMainFrame, CAsyncRequestQueue *pAsyncRequestQueue)
+CQueue::CQueue(wxWindow* parent, CMainFrame *pMainFrame, CAsyncRequestQueue *pAsyncRequestQueue, cert_store & certStore)
 {
 	Create(parent, -1, wxDefaultPosition, wxDefaultSize, wxNO_BORDER | wxAUI_NB_BOTTOM);
 	SetExArtProvider();
 
-	m_pQueueView = new CQueueView(this, 0, pMainFrame, pAsyncRequestQueue);
+	m_pQueueView = new CQueueView(this, 0, pMainFrame, pAsyncRequestQueue, certStore);
 	AddPage(m_pQueueView, m_pQueueView->GetTitle());
 
-	m_pQueueView_Failed = new CQueueViewFailed(this, 1);
+	m_pQueueView_Failed = new CQueueViewFailed(this, pMainFrame->GetOptions(), 1);
 	AddPage(m_pQueueView_Failed, m_pQueueView_Failed->GetTitle());
-	m_pQueueView_Successful = new CQueueViewSuccessful(this, 2);
+	m_pQueueView_Successful = new CQueueViewSuccessful(this, pMainFrame->GetOptions(), 2);
 	AddPage(m_pQueueView_Successful, m_pQueueView_Successful->GetTitle());
 
 	RemoveExtraBorders();
 
 	m_pQueueView->LoadQueue();
+
+	Bind(wxEVT_COMMAND_AUINOTEBOOK_TAB_RIGHT_UP, [this](wxAuiNotebookEvent& ev) {
+		int tab = ev.GetSelection();
+
+		wxContextMenuEvent cev(wxEVT_CONTEXT_MENU);
+		switch (tab) {
+		case 0:
+			m_pQueueView->HandleWindowEvent(cev);
+			break;
+		case 1:
+			m_pQueueView_Failed->HandleWindowEvent(cev);
+			break;
+		case 2:
+			m_pQueueView_Successful->HandleWindowEvent(cev);
+			break;
+		default:
+			break;
+		}
+	});
 }
 
 void CQueue::SetFocus()
