@@ -309,7 +309,7 @@ public:
 		colErrorReason
 	};
 
-	CQueueViewBase(CQueue* parent, int index, const wxString& title);
+	CQueueViewBase(CQueue* parent, COptionsBase & options, int index, wxString const& title);
 	virtual ~CQueueViewBase();
 
 	// Gets item for given server or creates new if it doesn't exist
@@ -350,6 +350,8 @@ protected:
 
 	void DisplayNumberQueuedFiles();
 
+	COptionsBase& options_;
+
 	// Position at which insertions start and number of insertions
 	int m_insertionStart{-1};
 	unsigned int m_insertionCount{};
@@ -393,10 +395,11 @@ class CQueueViewSuccessful;
 
 class CMainFrame;
 class CAsyncRequestQueue;
+class cert_store;
 class CQueue final : public wxAuiNotebookEx
 {
 public:
-	CQueue(wxWindow* parent, CMainFrame* pMainFrame, CAsyncRequestQueue* pAsyncRequestQueue);
+	CQueue(wxWindow* parent, CMainFrame* pMainFrame, CAsyncRequestQueue* pAsyncRequestQueue, cert_store & certStore);
 	virtual ~CQueue() {}
 
 	inline CQueueView* GetQueueView() { return m_pQueueView; }
