@@ -6,7 +6,6 @@
 #include "commandqueue.h"
 #include "filelistctrl.h"
 #include "file_utils.h"
-#include "ipcmutex.h"
 #include "Options.h"
 #include "queue.h"
 #include "remote_recursive_operation.h"
@@ -17,6 +16,8 @@
 #include "xrc_helper.h"
 #include "customheightlistctrl.h"
 
+#include "../commonui/ipcmutex.h"
+
 #include <libfilezilla/process.hpp>
 #include <libfilezilla/uri.hpp>
 #include <libfilezilla/translate.hpp>
@@ -106,7 +107,7 @@ class CSearchDialogFileList final : public CFileListCtrl<CGenericFileData>
 	friend class CSearchSortType;
 
 public:
-	CSearchDialogFileList(CSearchDialog* pParent, CQueueView* pQueue);
+	CSearchDialogFileList(CSearchDialog* pParent, CQueueView* pQueue, COptionsBase & options);
 
 	void clear();
 	void set_mode(CSearchDialog::search_mode mode);
@@ -116,7 +117,7 @@ protected:
 
 	virtual int64_t ItemGetSize(int index) const;
 
-	virtual std::unique_ptr<CFileListCtrlSortBase> GetSortComparisonObject() override;
+	virtual void UpdateSortComparisonObject() override;
 
 	CSearchDialog *m_searchDialog;
 
@@ -159,8 +160,8 @@ END_EVENT_TABLE()
 // Defined in RemoteListView.cpp
 std::wstring StripVMSRevision(std::wstring const& name);
 
-CSearchDialogFileList::CSearchDialogFileList(CSearchDialog* pParent, CQueueView* pQueue)
-	: CFileListCtrl<CGenericFileData>(pParent, pQueue, true),
+CSearchDialogFileList::CSearchDialogFileList(CSearchDialog* pParent, CQueueView* pQueue, COptionsBase & options)
+	: CFileListCtrl<CGenericFileData>(pParent, pQueue, options, true),
 	m_searchDialog(pParent)
 {
 	m_hasParent = false;
@@ -201,6 +202,7 @@ void CSearchDialogFileList::clear()
 void CSearchDialogFileList::set_mode(CSearchDialog::search_mode mode)
 {
 	mode_ = mode;
+	UpdateSortComparisonObject();
 }
 
 bool CSearchDialogFileList::ItemIsDir(int index) const
@@ -223,7 +225,7 @@ int64_t CSearchDialogFileList::ItemGetSize(int index) const
 	}
 }
 
-std::unique_ptr<CFileListCtrlSortBase> CSearchDialogFileList::GetSortComparisonObject()
+void CSearchDialogFileList::UpdateSortComparisonObject()
 {
 	CFileListCtrlSortBase::DirSortMode dirSortMode = GetDirSortMode();
 	NameSortMode nameSortMode = GetNameSortMode();
@@ -231,84 +233,84 @@ std::unique_ptr<CFileListCtrlSortBase> CSearchDialogFileList::GetSortComparisonO
 	if (mode_ == CSearchDialog::search_mode::local) {
 		if (!m_sortDirection) {
 			if (m_sortColumn == 1) {
-				return std::make_unique<CFileListCtrlSortPath<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CFileListCtrlSortPath<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 2) {
-				return std::make_unique<CFileListCtrlSortSize<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CFileListCtrlSortSize<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 3) {
-				return std::make_unique<CFileListCtrlSortType<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CFileListCtrlSortType<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 4) {
-				return std::make_unique<CFileListCtrlSortTime<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CFileListCtrlSortTime<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else {
-				return std::make_unique<CFileListCtrlSortNamePath<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CFileListCtrlSortNamePath<std::vector<CLocalSearchFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 		}
 		else {
 			if (m_sortColumn == 1) {
-				return std::make_unique<CReverseSort<CFileListCtrlSortPath<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortPath<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 2) {
-				return std::make_unique<CReverseSort<CFileListCtrlSortSize<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortSize<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 3) {
-				return std::make_unique<CReverseSort<CFileListCtrlSortType<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortType<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 4) {
-				return std::make_unique<CReverseSort<CFileListCtrlSortTime<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortTime<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else {
-				return std::make_unique<CReverseSort<CFileListCtrlSortNamePath<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortNamePath<std::vector<CLocalSearchFileData>, CGenericFileData>, CGenericFileData>>(localFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 		}
 	}
 	else {
 		if (!m_sortDirection) {
 			if (m_sortColumn == 1) {
-				return std::make_unique<CFileListCtrlSortPath<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CFileListCtrlSortPath<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 2) {
-				return std::make_unique<CFileListCtrlSortSize<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CFileListCtrlSortSize<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 3) {
-				return std::make_unique<CFileListCtrlSortType<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CFileListCtrlSortType<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 4) {
-				return std::make_unique<CFileListCtrlSortTime<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CFileListCtrlSortTime<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 5) {
-				return std::make_unique<CFileListCtrlSortPermissions<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CFileListCtrlSortPermissions<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 6) {
-				return std::make_unique<CFileListCtrlSortOwnerGroup<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CFileListCtrlSortOwnerGroup<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else {
-				return std::make_unique<CFileListCtrlSortNamePath<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CFileListCtrlSortNamePath<std::vector<CRemoteSearchFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 		}
 		else {
 			if (m_sortColumn == 1) {
-				return std::make_unique<CReverseSort<CFileListCtrlSortPath<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortPath<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 2) {
-				return std::make_unique<CReverseSort<CFileListCtrlSortSize<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortSize<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 3) {
-				return std::make_unique<CReverseSort<CFileListCtrlSortType<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortType<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 4) {
-				return std::make_unique<CReverseSort<CFileListCtrlSortTime<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortTime<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 5) {
-				return std::make_unique<CReverseSort<CFileListCtrlSortPermissions<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortPermissions<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else if (m_sortColumn == 6) {
-				return std::make_unique<CReverseSort<CFileListCtrlSortOwnerGroup<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortOwnerGroup<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 			else {
-				return std::make_unique<CReverseSort<CFileListCtrlSortNamePath<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
+				sortComparisonObject_ = std::make_unique<CReverseSort<CFileListCtrlSortNamePath<std::vector<CRemoteSearchFileData>, CGenericFileData>, CGenericFileData>>(remoteFileData_, m_fileData, dirSortMode, nameSortMode, this);
 			}
 		}
 	}
@@ -407,7 +409,10 @@ int CSearchDialogFileList::OnGetItemImage(long item) const
 	if (item < 0 || item >= (int)m_indexMapping.size()) {
 		return -1;
 	}
-	int index = m_indexMapping[item];
+	unsigned int index = m_indexMapping[item];
+	if (index >= pThis->m_fileData.size()) {
+		return -1;
+	}
 
 	int &icon = pThis->m_fileData[index].icon;
 
@@ -416,10 +421,16 @@ int CSearchDialogFileList::OnGetItemImage(long item) const
 	}
 
 	if (mode_ == CSearchDialog::search_mode::local) {
+		if (index >= localFileData_.size()) {
+			return -1;
+		}
 		auto const& file = localFileData_[index];
 		icon = pThis->GetIconIndex(iconType::file, file.path.GetPath() + file.name, true);
 	}
 	else {
+		if (index >= remoteFileData_.size()) {
+			return -1;
+		}
 		icon = pThis->GetIconIndex(iconType::file, remoteFileData_[index].name, false);
 	}
 
@@ -611,10 +622,11 @@ EVT_RADIOBUTTON(XRCID("ID_COMPARE_DATE"), CSearchDialog::OnChangeCompareOption)
 EVT_CHECKBOX(XRCID("ID_COMPARE_HIDEIDENTICAL"), CSearchDialog::OnChangeCompareOption)
 END_EVENT_TABLE()
 
-CSearchDialog::CSearchDialog(wxWindow* parent, CState& state, CQueueView* pQueue)
+CSearchDialog::CSearchDialog(wxWindow* parent, CState& state, CQueueView* pQueue, COptionsBase & options)
 	: CStateEventHandler(state)
 	, m_parent(parent)
 	, m_pQueue(pQueue)
+	, options_(options)
 {
 	m_pComparisonManager = new CComparisonManager(state);
 }
@@ -725,11 +737,11 @@ bool CSearchDialog::Load()
 		return false;
 	}
 
-	m_results = new CSearchDialogFileList(this, 0);
+	m_results = new CSearchDialogFileList(this, 0, options_);
 	ReplaceControl(XRCCTRL(*this, "ID_RESULTS", wxWindow), m_results);
 	m_results->SetFilelistStatusBar(pStatusBar);
 
-	m_remoteResults = new CSearchDialogFileList(this, 0);
+	m_remoteResults = new CSearchDialogFileList(this, 0, options_);
 	ReplaceControl(XRCCTRL(*this, "ID_REMOTE_RESULTS", wxWindow), m_remoteResults);
 	m_remoteResults->SetFilelistStatusBar(m_remoteStatusBar);
 	m_remoteResults->Show(false);
@@ -741,14 +753,14 @@ bool CSearchDialog::Load()
 
 	SetCtrlState();
 
-	const int mode = COptions::Get()->get_int(OPTION_COMPARISONMODE);
+	const int mode = options_.get_int(OPTION_COMPARISONMODE);
 	if (mode == 0) {
 		xrc_call(*this, "ID_COMPARE_SIZE", &wxRadioButton::SetValue, true);
 	}
 	else {
 		xrc_call(*this, "ID_COMPARE_DATE", &wxRadioButton::SetValue, true);
 	}
-	xrc_call(*this, "ID_COMPARE_HIDEIDENTICAL", &wxCheckBox::SetValue, COptions::Get()->get_int(OPTION_COMPARE_HIDEIDENTICAL) != 0);
+	xrc_call(*this, "ID_COMPARE_HIDEIDENTICAL", &wxCheckBox::SetValue, options_.get_int(OPTION_COMPARE_HIDEIDENTICAL) != 0);
 
 	LoadConditions();
 	EditFilter(m_search_filter);
@@ -817,7 +829,7 @@ void CSearchDialog::OnStateChange(t_statechange_notifications notification, std:
 	if (notification == STATECHANGE_REMOTE_DIR_OTHER && data2) {
 		if (searching_ && mode_ != search_mode::local) {
 			auto recursiveOperation = m_state.GetRemoteRecursiveOperation();
-			if (recursiveOperation && recursiveOperation->GetOperationMode() == CRecursiveOperation::recursive_list) {
+			if (recursiveOperation && recursiveOperation->GetOperationMode() == recursive_operation::recursive_list) {
 				std::shared_ptr<CDirectoryListing> const& listing = *reinterpret_cast<std::shared_ptr<CDirectoryListing> const*>(data2);
 
 				ProcessDirectoryListing(listing);
@@ -902,7 +914,7 @@ void CSearchDialog::ProcessDirectoryListing(std::shared_ptr<CDirectoryListing> c
 		added_indexes.reserve(listing->size());
 	}
 
-	std::unique_ptr<CFileListCtrlSortBase> compare = results->GetSortComparisonObject();
+	auto & compare = results->GetSortComparisonObject();
 	for (size_t i = 0; i < listing->size(); ++i) {
 		CDirentry const& entry = (*listing)[i];
 
@@ -956,7 +968,7 @@ void CSearchDialog::ProcessDirectoryListing(CLocalRecursiveOperation::listing co
 	int old_count = m_results->m_fileData.size();
 	int added_count = 0;
 
-	std::wstring const path = listing.localPath.GetPath();
+	std::wstring const& path = listing.localPath.GetPath();
 
 	bool const has_selections = m_results->GetSelectedItemCount() != 0;
 
@@ -965,7 +977,7 @@ void CSearchDialog::ProcessDirectoryListing(CLocalRecursiveOperation::listing co
 		added_indexes.reserve(listing.files.size() + listing.dirs.size());
 	}
 
-	std::unique_ptr<CFileListCtrlSortBase> compare = m_results->GetSortComparisonObject();
+	auto & compare = m_results->GetSortComparisonObject();
 
 	auto const& add_entry = [&](CLocalRecursiveOperation::listing::entry const& entry, bool dir) {
 		if (!CFilterManager::FilenameFilteredByFilter(m_search_filter, entry.name, path, dir, entry.size, entry.attributes, entry.time)) {
@@ -976,11 +988,11 @@ void CSearchDialog::ProcessDirectoryListing(CLocalRecursiveOperation::listing co
 		static_cast<CLocalRecursiveOperation::listing::entry&>(localData) = entry;
 		localData.path = listing.localPath;
 		localData.dir = dir;
-		m_results->localFileData_.push_back(localData);
+		m_results->localFileData_.emplace_back(std::move(localData));
 
 		CGenericFileData data;
 		data.icon = dir ? m_results->m_dirIcon : -2;
-		m_results->m_fileData.push_back(data);
+		m_results->m_fileData.emplace_back(std::move(data));
 
 		auto insertPos = std::lower_bound(m_results->m_indexMapping.begin(), m_results->m_indexMapping.end(), old_count + added_count, SortPredicate(compare));
 		int const added_index = insertPos - m_results->m_indexMapping.begin();
@@ -1119,7 +1131,7 @@ void CSearchDialog::OnSearch(wxCommandEvent&)
 		root.add_dir_to_visit(m_local_search_root);
 		m_state.GetLocalRecursiveOperation()->AddRecursionRoot(std::move(root));
 		ActiveFilters const filters; // Empty, recurse into everything
-		m_state.GetLocalRecursiveOperation()->StartRecursiveOperation(CRecursiveOperation::recursive_list, filters, true, false);
+		m_state.GetLocalRecursiveOperation()->StartRecursiveOperation(recursive_operation::recursive_list, filters, true, false);
 	}
 
 	if (mode_ != search_mode::local) {
@@ -1127,7 +1139,7 @@ void CSearchDialog::OnSearch(wxCommandEvent&)
 		root.add_dir_to_visit_restricted(m_remote_search_root, std::wstring(), true);
 		m_state.GetRemoteRecursiveOperation()->AddRecursionRoot(std::move(root));
 		ActiveFilters const filters; // Empty, recurse into everything
-		m_state.GetRemoteRecursiveOperation()->StartRecursiveOperation(CRecursiveOperation::recursive_list, filters);
+		m_state.GetRemoteRecursiveOperation()->StartRecursiveOperation(recursive_operation::recursive_list, filters);
 	}
 
 	SetCtrlState();
@@ -1480,7 +1492,7 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 
 		CServerPath remote_path = entry.path;
 		std::wstring localName = CQueueView::ReplaceInvalidCharacters(entry.name);
-		if (!entry.is_dir() && remote_path.GetType() == VMS && COptions::Get()->get_int(OPTION_STRIP_VMS_REVISION))
+		if (!entry.is_dir() && remote_path.GetType() == VMS && options_.get_int(OPTION_STRIP_VMS_REVISION))
 			localName = StripVMSRevision(localName);
 
 		m_pQueue->QueueFile(!start, true,
@@ -1489,7 +1501,7 @@ void CSearchDialog::OnDownload(wxCommandEvent&)
 	}
 	m_pQueue->QueueFile_Finish(start);
 
-	auto const mode = flatten ? CRecursiveOperation::recursive_transfer_flatten : CRecursiveOperation::recursive_transfer;
+	auto const mode = flatten ? recursive_operation::recursive_transfer_flatten : recursive_operation::recursive_transfer;
 
 	for (auto const& dir : selected_dirs) {
 		CLocalPath target_path = path;
@@ -1568,7 +1580,7 @@ void CSearchDialog::OnUpload(wxCommandEvent&)
 	}
 	m_pQueue->QueueFile_Finish(start);
 
-	auto const mode = flatten ? CRecursiveOperation::recursive_transfer_flatten : CRecursiveOperation::recursive_transfer;
+	auto const mode = flatten ? recursive_operation::recursive_transfer_flatten : recursive_operation::recursive_transfer;
 
 	for (auto const& dir : selected_dirs) {
 		CServerPath target_path = path;
@@ -1715,7 +1727,7 @@ void CSearchDialog::OnDeleteRemote(wxCommandEvent&)
 	}
 
 	ActiveFilters const filters; // Empty, recurse into everything
-	m_state.GetRemoteRecursiveOperation()->StartRecursiveOperation(CRecursiveOperation::recursive_delete, filters);
+	m_state.GetRemoteRecursiveOperation()->StartRecursiveOperation(recursive_operation::recursive_delete, filters);
 }
 
 void CSearchDialog::OnDeleteLocal(wxCommandEvent&)
@@ -1786,7 +1798,7 @@ void CSearchDialog::LoadConditions()
 		return;
 	}
 
-	if (!CFilterManager::LoadFilter(filter, m_search_filter)) {
+	if (!load_filter(filter, m_search_filter)) {
 		m_search_filter = CFilter();
 	}
 
@@ -1821,7 +1833,7 @@ void CSearchDialog::SaveConditions()
 	}
 	filter = document.append_child("Filter");
 
-	CFilterManager::SaveFilter(filter, m_search_filter);
+	save_filter(filter, m_search_filter);
 
 	pugi::xml_node comparative;
 	while ((comparative = document.child("Comparative"))) {
@@ -1832,7 +1844,7 @@ void CSearchDialog::SaveConditions()
 	AddTextElement(comparative, "CompareSizes", xrc_call(*this, "ID_COMPARE_SIZE", &wxRadioButton::GetValue) ? "1" : "0");
 	AddTextElement(comparative, "HideIdentical", xrc_call(*this, "ID_COMPARE_HIDEIDENTICAL", &wxCheckBox::GetValue) ? "1" : "0");
 
-	file.Save(true);
+	SaveWithErrorDialog(file);
 }
 
 void CSearchDialog::OnChangeSearchMode(wxCommandEvent&)
