@@ -1,4 +1,4 @@
-#include <filezilla.h>
+#include "../filezilla.h"
 #include "settingsdialog.h"
 #include "../Options.h"
 #include "optionspage.h"
@@ -27,14 +27,6 @@
 #include "../filezillaapp.h"
 #include "../Mainfrm.h"
 #include "../treectrlex.h"
-#include "../xrc_helper.h"
-
-BEGIN_EVENT_TABLE(CSettingsDialog, wxDialogEx)
-EVT_TREE_SEL_CHANGING(XRCID("ID_TREE"), CSettingsDialog::OnPageChanging)
-EVT_TREE_SEL_CHANGED(XRCID("ID_TREE"), CSettingsDialog::OnPageChanged)
-EVT_BUTTON(XRCID("wxID_OK"), CSettingsDialog::OnOK)
-EVT_BUTTON(XRCID("wxID_CANCEL"), CSettingsDialog::OnCancel)
-END_EVENT_TABLE()
 
 CSettingsDialog::CSettingsDialog(CFileZillaEngineContext & engine_context)
 	: m_engine_context(engine_context)
@@ -44,6 +36,14 @@ CSettingsDialog::CSettingsDialog(CFileZillaEngineContext & engine_context)
 
 CSettingsDialog::~CSettingsDialog()
 {
+	m_activePanel = nullptr;
+	m_pages.clear();
+
+	if (tree_) {
+		// Trees can generate these events during destruction, not good.
+		tree_->Unbind(wxEVT_TREE_SEL_CHANGING, &CSettingsDialog::OnPageChanging, this);
+		tree_->Unbind(wxEVT_TREE_SEL_CHANGED, &CSettingsDialog::OnPageChanged, this);
+	}
 }
 
 bool CSettingsDialog::Create(CMainFrame* pMainFrame)
@@ -51,7 +51,7 @@ bool CSettingsDialog::Create(CMainFrame* pMainFrame)
 	m_pMainFrame = pMainFrame;
 
 	SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
-	if (!wxDialogEx::Create(pMainFrame, -1, _("Settings"))) {
+	if (!wxDialogEx::Create(pMainFrame, nullID, _("Settings"))) {
 		return false;
 	}
 
@@ -63,19 +63,25 @@ bool CSettingsDialog::Create(CMainFrame* pMainFrame)
 	left->AddGrowableRow(1);
 	main->Add(left, 1, wxGROW);
 
-	left->Add(new wxStaticText(this, -1, _("Select &page:")));
+	left->Add(new wxStaticText(this, nullID, _("Select &page:")));
 
-	tree_ = new wxTreeCtrlEx(this, XRCID("ID_TREE"), wxDefaultPosition, wxDefaultSize, DEFAULT_TREE_STYLE | wxTR_HIDE_ROOT);
+	tree_ = new wxTreeCtrlEx(this, nullID, wxDefaultPosition, wxDefaultSize, DEFAULT_TREE_STYLE | wxTR_HIDE_ROOT);
 	tree_->SetFocus();
 	left->Add(tree_, 1, wxGROW);
 
 	auto ok = new wxButton(this, wxID_OK, _("OK"));
+	ok->Bind(wxEVT_BUTTON, &CSettingsDialog::OnOK, this);
 	ok->SetDefault();
 	left->Add(ok, lay.grow);
-	left->Add(new wxButton(this, wxID_CANCEL, _("&Cancel")), lay.grow);
+	auto cancel = new wxButton(this, wxID_CANCEL, _("Cancel"));
+	cancel->Bind(wxEVT_BUTTON, &CSettingsDialog::OnCancel, this);
+	left->Add(cancel, lay.grow);
 
 	pagePanel_ = new wxPanel(this);
-	main->Add(pagePanel_);
+	main->Add(pagePanel_, lay.grow);
+
+	tree_->Bind(wxEVT_TREE_SEL_CHANGING, &CSettingsDialog::OnPageChanging, this);
+	tree_->Bind(wxEVT_TREE_SEL_CHANGED, &CSettingsDialog::OnPageChanged, this);
 
 	if (!LoadPages()) {
 		return false;
@@ -104,8 +110,6 @@ void CSettingsDialog::AddPage(wxString const& name, COptionsPage* page, int nest
 
 bool CSettingsDialog::LoadPages()
 {
-	InitXrc(L"settings.xrc");
-
 	// Get the tree control.
 
 	if (!tree_) {
@@ -135,7 +139,7 @@ bool CSettingsDialog::LoadPages()
 	AddPage(_("File editing"), new COptionsPageEdit, 0);
 	AddPage(_("Filetype associations"), new COptionsPageEditAssociations, 1);
 #if FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
-	if (!COptions::Get()->GetOptionVal(OPTION_DEFAULT_DISABLEUPDATECHECK)) {
+	if (!m_pOptions->get_int(OPTION_DEFAULT_DISABLEUPDATECHECK)) {
 		AddPage(_("Updates"), new COptionsPageUpdateCheck, 0);
 	}
 #endif //FZ_MANUALUPDATECHECK && FZ_AUTOUPDATECHECK
@@ -190,10 +194,15 @@ bool CSettingsDialog::LoadPages()
 	Show();
 #endif
 
+	GetSizer()->Fit(this);
+
 	// Keep track of maximum page size
-	size = wxSize(0, 0);
+	size = pagePanel_->GetClientSize();
 	for (auto const& page : m_pages) {
-		size.IncTo(page.page->GetSizer()->GetMinSize());
+		auto sizer = page.page->GetSizer();
+		if (sizer) {
+			size.IncTo(sizer->GetMinSize());
+		}
 	}
 
 	wxSize panelSize = size;
@@ -204,16 +213,18 @@ bool CSettingsDialog::LoadPages()
 
 	// Adjust pages sizes according to maximum size
 	for (auto const& page : m_pages) {
-		page.page->GetSizer()->SetMinSize(size);
-		page.page->GetSizer()->Fit(page.page);
-		page.page->GetSizer()->SetSizeHints(page.page);
+		auto sizer = page.page->GetSizer();
+		if (sizer) {
+			sizer->SetMinSize(size);
+			sizer->Fit(page.page);
+			sizer->SetSizeHints(page.page);
+		}
 		if (GetLayoutDirection() == wxLayout_RightToLeft) {
 			page.page->Move(wxPoint(1, 0));
 		}
 	}
 
 	GetSizer()->Fit(this);
-	GetSizer()->SetSizeHints(this);
 
 	for (auto const& page : m_pages) {
 		page.page->Hide();
@@ -272,15 +283,21 @@ void CSettingsDialog::OnOK(wxCommandEvent&)
 		page.page->SavePage();
 	}
 
+	m_activePanel = nullptr;
+	m_pages.clear();
+
 	EndModal(wxID_OK);
 }
 
 void CSettingsDialog::OnCancel(wxCommandEvent&)
 {
+	m_activePanel = nullptr;
+	m_pages.clear();
+
 	EndModal(wxID_CANCEL);
 
 	for (auto const& saved : m_oldValues) {
-		m_pOptions->SetOption(saved.first, saved.second);
+		m_pOptions->set(saved.first, saved.second);
 	}
 }
 
@@ -295,7 +312,7 @@ void CSettingsDialog::OnPageChanging(wxTreeEvent& event)
 	}
 }
 
-void CSettingsDialog::RememberOldValue(int option)
+void CSettingsDialog::RememberOldValue(interfaceOptions option)
 {
-	m_oldValues[option] = m_pOptions->GetOption(option);
+	m_oldValues[option] = m_pOptions->get_string(option);
 }
