@@ -11,6 +11,9 @@
 #include "textctrlex.h"
 #include "xrc_helper.h"
 #include "wxext/spinctrlex.h"
+#if ENABLE_STORJ
+#include "storj_key_interface.h"
+#endif
 
 #include "../include/s3sse.h"
 
@@ -67,8 +70,9 @@ std::pair<std::array<ProtocolGroup, 2>::const_iterator, std::vector<std::pair<Se
 }
 }
 
-GeneralSiteControls::GeneralSiteControls(wxWindow & parent, DialogLayout const& lay, wxFlexGridSizer & sizer, std::function<void(ServerProtocol protocol, LogonType logon_type)> const& changeHandler)
+GeneralSiteControls::GeneralSiteControls(wxWindow & parent, DialogLayout const& lay, wxFlexGridSizer & sizer, COptionsBase & options, std::function<void(ServerProtocol protocol, LogonType logon_type)> const& changeHandler)
     : SiteControls(parent)
+	, options_(options)
     , changeHandler_(changeHandler)
 {
 	if (!sizer.IsColGrowable(0)) {
@@ -82,7 +86,7 @@ GeneralSiteControls::GeneralSiteControls(wxWindow & parent, DialogLayout const&
 	bag->SetEmptyCellSize(wxSize(-bag->GetHGap(), -bag->GetVGap()));
 
 	lay.gbNewRow(bag);
-	lay.gbAdd(bag, new wxStaticText(&parent, -1, _("Pro&tocol:")), lay.valign);
+	lay.gbAdd(bag, new wxStaticText(&parent, nullID, _("Pro&tocol:")), lay.valign);
 	auto protocols = new wxChoice(&parent, XRCID("ID_PROTOCOL"));
 	lay.gbAdd(bag, protocols, lay.valigng);
 
@@ -92,7 +96,7 @@ GeneralSiteControls::GeneralSiteControls(wxWindow & parent, DialogLayout const&
 	row->AddGrowableCol(0);
 	lay.gbAdd(bag, row, lay.valigng);
 	row->Add(new wxTextCtrlEx(&parent, XRCID("ID_HOST")), lay.valigng);
-	row->Add(new wxStaticText(&parent, -1, _("&Port:")), lay.valign);
+	row->Add(new wxStaticText(&parent, nullID, _("&Port:")), lay.valign);
 	auto* port = new wxTextCtrlEx(&parent, XRCID("ID_PORT"), wxString(), wxDefaultPosition, wxSize(lay.dlgUnits(27), -1));
 	port->SetMaxLength(5);
 	row->Add(port, lay.valign);
@@ -102,8 +106,8 @@ GeneralSiteControls::GeneralSiteControls(wxWindow & parent, DialogLayout const&
 	auto brow = new wxBoxSizer(wxHORIZONTAL);
 	lay.gbAdd(bag, brow, lay.valigng);
 	brow->Add(new wxChoice(&parent, XRCID("ID_ENCRYPTION")), 1);
-	brow->Add(new wxHyperlinkCtrl(&parent, XRCID("ID_DOCS"), _("Docs"), L"https://documentation.tardigrade.io/how-tos/set-up-filezilla-for-decentralized-file-transfer"), 0, wxRIGHT | wxALIGN_CENTER_VERTICAL, lay.dlgUnits(2))->Show(false);
-	brow->Add(new wxHyperlinkCtrl(&parent, XRCID("ID_SIGNUP"), _("Signup"), L"https://tardigrade.io/signup/?partner=filezilla"), lay.valign)->Show(false);
+	brow->Add(new wxHyperlinkCtrl(&parent, XRCID("ID_DOCS"), _("Docs"), L"https://docs.storj.io/how-tos/set-up-filezilla-for-decentralized-file-transfer"), 0, wxRIGHT | wxALIGN_CENTER_VERTICAL, lay.dlgUnits(2))->Show(false);
+	brow->Add(new wxHyperlinkCtrl(&parent, XRCID("ID_SIGNUP"), _("Signup"), L"https://storj.io/signup/?partner=filezilla"), lay.valign)->Show(false);
 	brow->AddSpacer(0);
 
 	lay.gbNewRow(bag);
@@ -113,7 +117,7 @@ GeneralSiteControls::GeneralSiteControls(wxWindow & parent, DialogLayout const&
 	lay.gbAddRow(bag, new wxStaticLine(&parent), lay.grow);
 
 	lay.gbNewRow(bag);
-	lay.gbAdd(bag, new wxStaticText(&parent, -1, _("&Logon Type:")), lay.valign);
+	lay.gbAdd(bag, new wxStaticText(&parent, nullID, _("&Logon Type:")), lay.valign);
 	auto logonTypes = new wxChoice(&parent, XRCID("ID_LOGONTYPE"));
 	lay.gbAdd(bag, logonTypes, lay.valigng);
 
@@ -286,7 +290,7 @@ void GeneralSiteControls::SetSite(Site const& site)
 		}
 		auto const& traits = ExtraServerParameterTraits(protocol);
 		for (auto const& trait : traits) {
-			if (trait.section_ == ParameterSection::credentials || trait.section_ == ParameterSection::custom) {
+			if (trait.section_ == ParameterSection::credentials || trait.flags_ & ParameterTraits::custom) {
 				continue;
 			}
 
@@ -310,8 +314,8 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 	xrc_call(parent_, "ID_ENCRYPTION_DESC", &wxStaticText::Show, group.first != protocolGroups().cend());
 	xrc_call(parent_, "ID_ENCRYPTION", &wxChoice::Show, group.first != protocolGroups().cend());
 
-	xrc_call(parent_, "ID_DOCS", &wxControl::Show, protocol == STORJ);
-	xrc_call(parent_, "ID_SIGNUP", &wxControl::Show, protocol == STORJ);
+	xrc_call(parent_, "ID_DOCS", &wxControl::Show, protocol == STORJ || protocol == STORJ_GRANT);
+	xrc_call(parent_, "ID_SIGNUP", &wxControl::Show, protocol == STORJ || protocol == STORJ_GRANT);
 
 	auto const supportedlogonTypes = GetSupportedLogonTypes(protocol);
 	assert(!supportedlogonTypes.empty());
@@ -331,11 +335,13 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 	}
 
 	bool const hasUser = ProtocolHasUser(protocol) && type != LogonType::anonymous;
+	bool const hasPw = type != LogonType::anonymous && type != LogonType::interactive && (protocol != SFTP || type != LogonType::key) && (protocol != S3 || type != LogonType::profile);
+
 
 	xrc_call(parent_, "ID_USER_DESC", &wxStaticText::Show, hasUser);
 	xrc_call(parent_, "ID_USER", &wxTextCtrl::Show, hasUser);
-	xrc_call(parent_, "ID_PASS_DESC", &wxStaticText::Show, type != LogonType::anonymous && type != LogonType::interactive  && (protocol != SFTP || type != LogonType::key));
-	xrc_call(parent_, "ID_PASS", &wxTextCtrl::Show, type != LogonType::anonymous && type != LogonType::interactive && (protocol != SFTP || type != LogonType::key));
+	xrc_call(parent_, "ID_PASS_DESC", &wxStaticText::Show, hasPw);
+	xrc_call(parent_, "ID_PASS", &wxTextCtrl::Show, hasPw);
 	xrc_call(parent_, "ID_ACCOUNT_DESC", &wxStaticText::Show, isFtp && type == LogonType::account);
 	xrc_call(parent_, "ID_ACCOUNT", &wxTextCtrl::Show, isFtp && type == LogonType::account);
 	xrc_call(parent_, "ID_KEYFILE_DESC", &wxStaticText::Show, protocol == SFTP && type == LogonType::key);
@@ -356,9 +362,18 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 		// @translator: Keep short
 		passLabel = _("Encr&yption Passphrase:");
 		break;
+	case STORJ_GRANT:
+		// @translator: Keep short
+		hostLabel = _("S&atellite:");
+		// @translator: Keep short
+		passLabel = _("Access &Grant:");
+		break;
 	case S3:
 		// @translator: Keep short
 		userLabel = _("&Access key ID:");
+		if (type == LogonType::profile) {
+			userLabel = _("&Profile");
+		}
 		// @translator: Keep short
 		passLabel = _("Secret Access &Key:");
 		break;
@@ -372,6 +387,7 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 		userLabel = _("Pro&ject ID:");
 		break;
 	case SWIFT:
+	case RACKSPACE:
 		// @translator: Keep short
 		hostLabel = _("Identity &host:");
 		// @translator: Keep short
@@ -441,7 +457,7 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 			label.SetLabel(_("&User:"));
 		}
 		else {
-			label.SetLabel(name);
+			label.SetLabel(fz::to_wstring_from_utf8(name));
 		}
 	};
 
@@ -465,7 +481,7 @@ void GeneralSiteControls::SetControlVisibility(ServerProtocol protocol, LogonTyp
 
 	std::vector<ParameterTraits> const& parameterTraits = ExtraServerParameterTraits(protocol);
 	for (auto const& trait : parameterTraits) {
-		if (trait.section_ == ParameterSection::custom) {
+		if (trait.flags_ & ParameterTraits::custom) {
 			continue;
 		}
 		auto & parameters = extraParameters_[trait.section_];
@@ -527,8 +543,8 @@ bool GeneralSiteControls::UpdateSite(Site & site, bool silent)
 	std::wstring pw = xrc_call(parent_, "ID_PASS", &wxTextCtrl::GetValue).ToStdWstring();
 
 	{
-		std::wstring const host = xrc_call(parent_, "ID_HOST", &wxTextCtrl::GetValue).ToStdWstring();
-		std::wstring const port = xrc_call(parent_, "ID_PORT", &wxTextCtrl::GetValue).ToStdWstring();
+		std::wstring host = xrc_call(parent_, "ID_HOST", &wxTextCtrl::GetValue).ToStdWstring();
+		std::wstring port = xrc_call(parent_, "ID_PORT", &wxTextCtrl::GetValue).ToStdWstring();
 
 		if (host.empty()) {
 			if (!silent) {
@@ -538,7 +554,6 @@ bool GeneralSiteControls::UpdateSite(Site & site, bool silent)
 			return false;
 		}
 
-
 		Site parsedSite;
 		std::wstring error;
 		site.m_default_bookmark.m_remoteDir = CServerPath();
@@ -568,13 +583,13 @@ bool GeneralSiteControls::UpdateSite(Site & site, bool silent)
 		logon_type = supportedlogonTypes.front();
 	}
 
-	if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) != 0 &&
+	if (options_.get_int(OPTION_DEFAULT_KIOSKMODE) != 0 &&
 			!predefined_ &&
 	        (logon_type == LogonType::account || logon_type == LogonType::normal))
 	{
 		if (!silent) {
 			wxString msg;
-			if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) != 0 && COptions::Get()->predefined(OPTION_DEFAULT_KIOSKMODE)) {
+			if (options_.get_int(OPTION_DEFAULT_KIOSKMODE) != 0 && options_.predefined(OPTION_DEFAULT_KIOSKMODE)) {
 				msg = _("Saving of password has been disabled by your system administrator.");
 			}
 			else {
@@ -671,7 +686,7 @@ bool GeneralSiteControls::UpdateSite(Site & site, bool silent)
 
 	std::vector<ParameterTraits> const& parameterTraits = ExtraServerParameterTraits(protocol);
 	for (auto const& trait : parameterTraits) {
-		if (trait.section_ == ParameterSection::custom || trait.section_ == ParameterSection::credentials) {
+		if (trait.section_ == ParameterSection::credentials || trait.flags_ & ParameterTraits::custom) {
 			continue;
 		}
 		for (auto const& row : extraParameters_[trait.section_]) {
@@ -686,13 +701,58 @@ bool GeneralSiteControls::UpdateSite(Site & site, bool silent)
 						return false;
 					}
 				}
-				site.server.SetExtraParameter(trait.name_, value);
+				if (trait.section_ == ParameterSection::credentials) {
+					site.credentials.SetExtraParameter(protocol, trait.name_, value);
+				}
+				else {
+					site.server.SetExtraParameter(trait.name_, value);
+				}
 				break;
 			}
 		}
-
 	}
 
+	if (protocol == S3 && logon_type == LogonType::profile) {
+		pw.clear();
+	}
+
+#if ENABLE_STORJ
+		if (protocol == STORJ_GRANT && logon_type == LogonType::normal) {
+			fz::trim(pw);
+			if (pw.empty() && !site.credentials.encrypted_) {
+				wxMessageBoxEx(_("You need to enter an access grant."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, wxGetTopLevelParent(&parent_));
+				xrc_call(parent_, "ID_PASS", &wxWindow::SetFocus);
+			}
+			else if (!pw.empty()) {
+				CStorjKeyInterface ki(options_, wxGetTopLevelParent(&parent_));
+				auto [valid, satellite] = ki.ValidateGrant(pw, silent);
+				if (valid) {
+					if (!satellite.empty()) {
+						size_t pos = satellite.rfind('@');
+						if (pos != std::wstring::npos) {
+							satellite = satellite.substr(pos + 1);
+						}
+						pos = satellite.rfind(':');
+						std::wstring port = L"7777";
+						if (pos != std::wstring::npos) {
+							port = satellite.substr(pos + 1);
+							satellite = satellite.substr(0, pos);
+						}
+						site.server.SetHost(satellite, fz::to_integral<unsigned short>(port, 7777));
+					}
+				}
+				else {
+					if (!silent) {
+						XRCCTRL(parent_, "ID_PASS", wxWindow)->SetFocus();
+						wxString msg = wxString::Format(_("You have to enter a valid access grant.\nError: %s"), satellite);
+						wxMessageBoxEx(msg, _("Site Manager - Invalid data"), wxICON_EXCLAMATION, wxGetTopLevelParent(&parent_));
+					}
+					return false;
+				}
+			}
+		}
+#endif
+
 	if (site.credentials.encrypted_) {
 		if (!pw.empty()) {
 			site.credentials.encrypted_ = fz::public_key();
@@ -789,7 +849,6 @@ ServerProtocol GeneralSiteControls::GetProtocol() const
 	return protocol;
 }
 
-
 void GeneralSiteControls::UpdateHostFromDefaults(ServerProtocol const newProtocol)
 {
 	if (newProtocol != protocol_) {
@@ -849,7 +908,7 @@ AdvancedSiteControls::AdvancedSiteControls(wxWindow & parent, DialogLayout const
 
 	sizer.Add(new wxStaticLine(&parent), lay.grow);
 
-	sizer.Add(new wxStaticText(&parent, -1, _("Default &local directory:")));
+	sizer.Add(new wxStaticText(&parent, nullID, _("Default &local directory:")));
 
 	row = lay.createFlex(0, 1);
 	sizer.Add(row, lay.grow);
@@ -867,7 +926,7 @@ AdvancedSiteControls::AdvancedSiteControls(wxWindow & parent, DialogLayout const
 	});
 
 	sizer.AddSpacer(0);
-	sizer.Add(new wxStaticText(&parent, -1, _("Default r&emote directory:")));
+	sizer.Add(new wxStaticText(&parent, nullID, _("Default r&emote directory:")));
 	sizer.Add(new wxTextCtrlEx(&parent, XRCID("ID_REMOTEDIR")), lay.grow);
 	sizer.AddSpacer(0);
 	sizer.Add(new wxCheckBox(&parent, XRCID("ID_SYNC"), _("&Use synchronized browsing")));
@@ -875,19 +934,19 @@ AdvancedSiteControls::AdvancedSiteControls(wxWindow & parent, DialogLayout const
 
 	sizer.Add(new wxStaticLine(&parent), lay.grow);
 
-	sizer.Add(new wxStaticText(&parent, -1, _("&Adjust server time, offset by:")));
+	sizer.Add(new wxStaticText(&parent, nullID, _("&Adjust server time, offset by:")));
 	row = lay.createFlex(0, 1);
 	sizer.Add(row);
 	auto* hours = new wxSpinCtrlEx(&parent, XRCID("ID_TIMEZONE_HOURS"), wxString(), wxDefaultPosition, wxSize(lay.dlgUnits(26), -1));
 	hours->SetRange(-24, 24);
 	hours->SetMaxLength(3);
 	row->Add(hours, lay.valign);
-	row->Add(new wxStaticText(&parent, -1, _("Hours,")), lay.valign);
+	row->Add(new wxStaticText(&parent, nullID, _("Hours,")), lay.valign);
 	auto* minutes = new wxSpinCtrlEx(&parent, XRCID("ID_TIMEZONE_MINUTES"), wxString(), wxDefaultPosition, wxSize(lay.dlgUnits(26), -1));
 	minutes->SetRange(-59, 59);
 	minutes->SetMaxLength(3);
 	row->Add(minutes, lay.valign);
-	row->Add(new wxStaticText(&parent, -1, _("Minutes")), lay.valign);
+	row->Add(new wxStaticText(&parent, nullID, _("Minutes")), lay.valign);
 }
 
 void AdvancedSiteControls::SetSite(Site const& site)
@@ -1001,10 +1060,10 @@ bool AdvancedSiteControls::UpdateSite(Site & site, bool silent)
 CharsetSiteControls::CharsetSiteControls(wxWindow & parent, DialogLayout const& lay, wxFlexGridSizer & sizer)
     : SiteControls(parent)
 {
-	sizer.Add(new wxStaticText(&parent, -1, _("The server uses following charset encoding for filenames:")));
+	sizer.Add(new wxStaticText(&parent, nullID, _("The server uses following charset encoding for filenames:")));
 	auto rbAuto = new wxRadioButton(&parent, XRCID("ID_CHARSET_AUTO"), _("&Autodetect"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
 	sizer.Add(rbAuto);
-	sizer.Add(new wxStaticText(&parent, -1, _("Uses UTF-8 if the server supports it, else uses local charset.")), 0, wxLEFT, 18);
+	sizer.Add(new wxStaticText(&parent, nullID, _("Uses UTF-8 if the server supports it, else uses local charset.")), 0, wxLEFT, 18);
 
 	auto rbUtf8 = new wxRadioButton(&parent, XRCID("ID_CHARSET_UTF8"), _("Force &UTF-8"));
 	sizer.Add(rbUtf8);
@@ -1012,12 +1071,12 @@ CharsetSiteControls::CharsetSiteControls(wxWindow & parent, DialogLayout const&
 	sizer.Add(rbCustom);
 
 	auto * row = lay.createFlex(0, 1);
-	row->Add(new wxStaticText(&parent, -1, _("&Encoding:")), lay.valign);
+	row->Add(new wxStaticText(&parent, nullID, _("&Encoding:")), lay.valign);
 	auto * encoding = new wxTextCtrlEx(&parent, XRCID("ID_ENCODING"));
 	row->Add(encoding, 0, wxLEFT | wxALIGN_CENTER_VERTICAL, 18);
 	sizer.Add(row);
 	sizer.AddSpacer(lay.dlgUnits(6));
-	sizer.Add(new wxStaticText(&parent, -1, _("Using the wrong charset can result in filenames not displaying properly.")));
+	sizer.Add(new wxStaticText(&parent, nullID, _("Using the wrong charset can result in filenames not displaying properly.")));
 
 	rbAuto->Bind(wxEVT_RADIOBUTTON, [encoding](wxEvent const&){ encoding->Disable(); });
 	rbUtf8->Bind(wxEVT_RADIOBUTTON, [encoding](wxEvent const&){ encoding->Disable(); });
@@ -1099,7 +1158,7 @@ TransferSettingsSiteControls::TransferSettingsSiteControls(wxWindow & parent, Di
 	sizer.Add(limit);
 	row = lay.createFlex(0, 1);
 	sizer.Add(row, 0, wxLEFT, lay.dlgUnits(10));
-	row->Add(new wxStaticText(&parent, -1, _("&Maximum number of connections:")), lay.valign);
+	row->Add(new wxStaticText(&parent, nullID, _("&Maximum number of connections:")), lay.valign);
 	auto * spin = new wxSpinCtrlEx(&parent, XRCID("ID_MAXMULTIPLE"), wxString(), wxDefaultPosition, wxSize(lay.dlgUnits(26), -1));
 	spin->SetMaxLength(2);
 	spin->SetRange(1, 10);
@@ -1188,7 +1247,6 @@ void TransferSettingsSiteControls::SetControlVisibility(ServerProtocol protocol,
 	transferModeLabel->GetContainingSizer()->Layout();
 }
 
-
 S3SiteControls::S3SiteControls(wxWindow & parent, DialogLayout const& lay, wxFlexGridSizer & sizer)
     : SiteControls(parent)
 {
@@ -1196,7 +1254,15 @@ S3SiteControls::S3SiteControls(wxWindow & parent, DialogLayout const& lay, wxFle
 		sizer.AddGrowableCol(0);
 	}
 
-	sizer.Add(new wxStaticText(&parent, -1, _("Server Side Encryption:")));
+	sizer.Add(new wxStaticText(&parent, nullID, _("Options:")));
+	auto * options_row = lay.createFlex(2);
+	options_row->AddGrowableCol(1);
+	sizer.Add(options_row, 0, wxLEFT|wxGROW, lay.indent);
+	options_row->Add(new wxStaticText(&parent, nullID, _("Re&gion:")), lay.valign);
+	options_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_REGION")), lay.valigng);
+
+	sizer.Add(new wxStaticLine(&parent), lay.grow);
+	sizer.Add(new wxStaticText(&parent, nullID, _("Server Side Encryption:")));
 
 	auto none = new wxRadioButton(&parent, XRCID("ID_S3_NOENCRYPTION"), _("N&o encryption"), wxDefaultPosition, wxDefaultSize, wxRB_GROUP);
 	sizer.Add(none);
@@ -1207,24 +1273,35 @@ S3SiteControls::S3SiteControls(wxWindow & parent, DialogLayout const& lay, wxFle
 	auto kms = new wxRadioButton(&parent, XRCID("ID_S3_AWSKMS"), _("AWS &KMS encryption"));
 	sizer.Add(kms);
 
-	auto * row = lay.createFlex(2);
-	row->AddGrowableCol(1);
-	sizer.Add(row, 0, wxLEFT|wxGROW, lay.indent);
-	row->Add(new wxStaticText(&parent, -1, _("&Select a key:")), lay.valign);
+	auto * key_row = lay.createFlex(2);
+	key_row->AddGrowableCol(1);
+	sizer.Add(key_row, 0, wxLEFT|wxGROW, lay.indent);
+	key_row->Add(new wxStaticText(&parent, nullID, _("&Select a key:")), lay.valign);
 	auto * choice = new wxChoice(&parent, XRCID("ID_S3_KMSKEY"));
 	choice->Append(_("Default (AWS/S3)"));
 	choice->Append(_("Custom KMS ARN"));
-	row->Add(choice, lay.valigng);
-	row->Add(new wxStaticText(&parent, -1, _("C&ustom KMS ARN:")), lay.valign);
-	row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_CUSTOM_KMS")), lay.valigng);
+	key_row->Add(choice, lay.valigng);
+	key_row->Add(new wxStaticText(&parent, nullID, _("C&ustom KMS ARN:")), lay.valign);
+	key_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_CUSTOM_KMS")), lay.valigng);
 
-	auto customer = new wxRadioButton(&parent, XRCID("ID_S3_CUSTOMER_ENCRYPTION"), _("Cu&stomer encryption"));
+	auto customer = new wxRadioButton(&parent, XRCID("ID_S3_CUSTOMER_ENCRYPTION"), _("Cus&tomer encryption"));
 	sizer.Add(customer);
-	row = lay.createFlex(2);
-	row->AddGrowableCol(1);
-	sizer.Add(row, 0, wxLEFT | wxGROW, lay.indent);
-	row->Add(new wxStaticText(&parent, -1, _("Cus&tomer Key:")), lay.valign);
-	row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_CUSTOMER_KEY")), lay.valigng);
+	key_row = lay.createFlex(2);
+	key_row->AddGrowableCol(1);
+	sizer.Add(key_row, 0, wxLEFT | wxGROW, lay.indent);
+	key_row->Add(new wxStaticText(&parent, nullID, _("Customer Ke&y:")), lay.valign);
+	key_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_CUSTOMER_KEY")), lay.valigng);
+
+	sizer.Add(new wxStaticLine(&parent), lay.grow);
+	sizer.Add(new wxStaticText(&parent, nullID, _("Security Token Service:")));
+
+	auto * sts_row = lay.createFlex(2);
+	sts_row->AddGrowableCol(1);
+	sizer.Add(sts_row, 0, wxLEFT|wxGROW, lay.indent);
+	sts_row->Add(new wxStaticText(&parent, nullID, _("Ro&le ARN:")), lay.valign);
+	sts_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_ROLE_ARN")), lay.valigng);
+	sts_row->Add(new wxStaticText(&parent, nullID, _("MFA D&evice Serial:")), lay.valign);
+	sts_row->Add(new wxTextCtrlEx(&parent, XRCID("ID_S3_MFA_SERIAL")), lay.valigng);
 
 	auto l = [this](wxEvent const&) { SetControlState(); };
 	none->Bind(wxEVT_RADIOBUTTON, l);
@@ -1262,30 +1339,38 @@ void S3SiteControls::SetSite(Site const& site)
 	xrc_call(parent_, "ID_S3_CUSTOMER_ENCRYPTION", &wxWindow::Enable, !predefined_);
 
 	if (site.server.GetProtocol() == S3) {
+		auto region = site.server.GetExtraParameter("region");
+		xrc_call(parent_, "ID_S3_REGION", &wxTextCtrl::ChangeValue, region);
+
 		xrc_call(parent_, "ID_S3_KMSKEY", &wxChoice::SetSelection, static_cast<int>(s3_sse::KmsKey::DEFAULT));
-		auto ssealgorithm = site.server.GetExtraParameter("ssealgorithm");
-		if (ssealgorithm.empty()) {
+		auto sse_algorithm = site.server.GetExtraParameter("ssealgorithm");
+		if (sse_algorithm.empty()) {
 			xrc_call(parent_, "ID_S3_NOENCRYPTION", &wxRadioButton::SetValue, true);
 		}
-		else if (ssealgorithm == "AES256") {
+		else if (sse_algorithm == "AES256") {
 			xrc_call(parent_, "ID_S3_AES256", &wxRadioButton::SetValue, true);
 		}
-		else if (ssealgorithm == "aws:kms") {
+		else if (sse_algorithm == "aws:kms") {
 			xrc_call(parent_, "ID_S3_AWSKMS", &wxRadioButton::SetValue, true);
-			auto sseKmsKey = site.server.GetExtraParameter("ssekmskey");
-			if (!sseKmsKey.empty()) {
+			auto sse_kms_key = site.server.GetExtraParameter("ssekmskey");
+			if (!sse_kms_key.empty()) {
 				xrc_call(parent_, "ID_S3_KMSKEY", &wxChoice::SetSelection, static_cast<int>(s3_sse::KmsKey::CUSTOM));
-				xrc_call(parent_, "ID_S3_CUSTOM_KMS", &wxTextCtrl::ChangeValue, sseKmsKey);
+				xrc_call(parent_, "ID_S3_CUSTOM_KMS", &wxTextCtrl::ChangeValue, sse_kms_key);
 			}
 			else {
 				xrc_call(parent_, "ID_S3_KMSKEY", &wxChoice::SetSelection, static_cast<int>(s3_sse::KmsKey::DEFAULT));
 			}
 		}
-		else if (ssealgorithm == "customer") {
+		else if (sse_algorithm == "customer") {
 			xrc_call(parent_, "ID_S3_CUSTOMER_ENCRYPTION", &wxRadioButton::SetValue, true);
-			auto customerKey = site.server.GetExtraParameter("ssecustomerkey");
-			xrc_call(parent_, "ID_S3_CUSTOMER_KEY", &wxTextCtrl::ChangeValue, customerKey);
+			auto customer_key = site.server.GetExtraParameter("ssecustomerkey");
+			xrc_call(parent_, "ID_S3_CUSTOMER_KEY", &wxTextCtrl::ChangeValue, customer_key);
 		}
+
+		auto value = site.server.GetExtraParameter("stsrolearn");
+		xrc_call(parent_, "ID_S3_ROLE_ARN", &wxTextCtrl::ChangeValue, value);
+		value = site.server.GetExtraParameter("stsmfaserial");
+		xrc_call(parent_, "ID_S3_MFA_SERIAL", &wxTextCtrl::ChangeValue, value);
 	}
 }
 
@@ -1293,6 +1378,8 @@ bool S3SiteControls::UpdateSite(Site & site, bool silent)
 {
 	CServer & server = site.server;
 	if (server.GetProtocol() == S3) {
+		server.SetExtraParameter("region", fz::to_wstring(xrc_call(parent_, "ID_S3_REGION", &wxTextCtrl::GetValue)));
+
 		if (xrc_call(parent_, "ID_S3_NOENCRYPTION", &wxRadioButton::GetValue)) {
 			server.ClearExtraParameter("ssealgorithm");
 		}
@@ -1340,6 +1427,124 @@ bool S3SiteControls::UpdateSite(Site & site, bool silent)
 			server.SetExtraParameter("ssealgorithm", L"customer");
 			server.SetExtraParameter("ssecustomerkey", fz::to_wstring(xrc_call(parent_, "ID_S3_CUSTOMER_KEY", &wxTextCtrl::GetValue)));
 		}
+
+		auto value = fz::to_wstring(xrc_call(parent_, "ID_S3_ROLE_ARN", &wxTextCtrl::GetValue));
+		if (!value.empty()) {
+			server.SetExtraParameter("stsrolearn", value);
+			value = fz::to_wstring(xrc_call(parent_, "ID_S3_MFA_SERIAL", &wxTextCtrl::GetValue));
+			if (!value.empty()) {
+				server.SetExtraParameter("stsmfaserial", value);
+			}
+		}
+	}
+
+	return true;
+}
+
+SwiftSiteControls::SwiftSiteControls(wxWindow & parent, DialogLayout const& lay, wxFlexGridSizer & sizer)
+	: SiteControls(parent)
+{
+	if (!sizer.IsColGrowable(0)) {
+		sizer.AddGrowableCol(0);
+	}
+
+	sizer.Add(new wxStaticText(&parent, nullID, _("Identity (Keystone):")));
+
+	auto keystone3 = new wxCheckBox(&parent, XRCID("ID_SWIFT_KEYSTONE_V3"), _("&Version 3"));
+	sizer.Add(keystone3);
+
+	auto *keyRow = lay.createFlex(2);
+	keyRow->AddGrowableCol(1);
+	sizer.Add(keyRow, 0, wxLEFT|wxGROW, lay.indent);
+	keyRow->Add(new wxStaticText(&parent, nullID, _("&Domain:")), lay.valign);
+	keyRow->Add(new wxTextCtrlEx(&parent, XRCID("ID_SWIFT_DOMAIN")), lay.valigng);
+
+	auto l = [this](wxEvent const&) { SetControlState(); };
+	keystone3->Bind(wxEVT_CHECKBOX, l);
+}
+
+void SwiftSiteControls::SetControlState()
+{
+	auto v3 = xrc_call(parent_, "ID_SWIFT_KEYSTONE_V3", &wxCheckBox::GetValue);
+
+	xrc_call(parent_, "ID_SWIFT_DOMAIN", &wxWindow::Enable, v3);
+}
+
+void SwiftSiteControls::SetSite(Site const& site)
+{
+	if (site.server.GetProtocol() == SWIFT) {
+		bool v3{};
+		auto pv3 = site.server.GetExtraParameter("keystone_version");
+		if (pv3.empty()) {
+			v3 = fz::starts_with(site.server.GetExtraParameter("identpath"), std::wstring(L"/v3"));
+		}
+		else {
+			v3 = pv3 == L"3";
+		}
+		xrc_call(parent_, "ID_SWIFT_KEYSTONE_V3", &wxCheckBox::SetValue, v3);
+		auto domain = site.server.GetExtraParameter("domain");
+		if (domain.empty()) {
+			domain = L"Default";
+		}
+		xrc_call(parent_, "ID_SWIFT_DOMAIN", &wxTextCtrl::ChangeValue, domain);
+	}
+}
+
+bool SwiftSiteControls::UpdateSite(Site & site, bool)
+{
+	CServer &server = site.server;
+	if (server.GetProtocol() == SWIFT) {
+		auto v3 = xrc_call(parent_, "ID_SWIFT_KEYSTONE_V3", &wxCheckBox::GetValue);
+		if (v3) {
+			server.SetExtraParameter("keystone_version", L"3");
+		}
+		else {
+			server.SetExtraParameter("keystone_version", L"2");
+		}
+
+		if (v3) {
+			auto domain = fz::to_wstring(xrc_call(parent_, "ID_SWIFT_DOMAIN", &wxTextCtrl::GetValue));
+			server.SetExtraParameter("domain", domain);
+		}
+		else {
+			server.ClearExtraParameter("domain");
+		}
+	}
+
+	return true;
+}
+
+DropboxSiteControls::DropboxSiteControls(wxWindow & parent, DialogLayout const&, wxFlexGridSizer & sizer)
+	: SiteControls(parent)
+{
+	if (!sizer.IsColGrowable(0)) {
+		sizer.AddGrowableCol(0);
+	}
+
+	sizer.Add(new wxStaticText(&parent, nullID, _("Dropbox for Business:")));
+
+	auto root_ns = new wxCheckBox(&parent, XRCID("ID_USE_ROOT_NS"), _("Use &team root namespace"));
+	sizer.Add(root_ns);
+}
+
+void DropboxSiteControls::SetSite(Site const& site)
+{
+	if (site.server.GetProtocol() == DROPBOX) {
+		auto root_ns = site.server.GetExtraParameter("root_namespace");
+		xrc_call(parent_, "ID_USE_ROOT_NS", &wxCheckBox::SetValue, root_ns == L"1");
+	}
+}
+
+bool DropboxSiteControls::UpdateSite(Site & site, bool)
+{
+	CServer & server = site.server;
+	if (server.GetProtocol() == DROPBOX) {
+		if (xrc_call(parent_, "ID_USE_ROOT_NS", &wxCheckBox::GetValue)) {
+			server.SetExtraParameter("root_namespace", L"1");
+		}
+		else {
+			server.ClearExtraParameter("root_namespace");
+		}
 	}
 
 	return true;
