@@ -7,7 +7,6 @@
 #include "drop_target_ex.h"
 #include "filezillaapp.h"
 #include "inputdialog.h"
-#include "ipcmutex.h"
 #include "Options.h"
 #include "sitemanager_site.h"
 #include "themeprovider.h"
@@ -18,6 +17,9 @@
 #include "xmlfunctions.h"
 #include "xrc_helper.h"
 
+#include "../commonui/fz_paths.h"
+#include "../commonui/ipcmutex.h"
+
 #include <wx/dirdlg.h>
 #include <wx/dnd.h>
 #include <wx/filedlg.h>
@@ -99,6 +101,11 @@ public:
 		m_pSiteManager = pSiteManager;
 	}
 
+	virtual wxTreeItemId DisplayDropHighlight(wxPoint const& p) override
+	{
+		return m_pSiteManager->tree_->DisplayDropHighlight(p);
+	}
+
 	bool IsValidDropLocation(wxTreeItemId const& hit, wxDragResult const& def)
 	{
 		if (!hit) {
@@ -139,7 +146,7 @@ public:
 			wxTreeItemId cur = hit;
 			while (cur && cur != tree->GetRootItem()) {
 				if (cur == item) {
-					ClearDropHighlight();
+					tree->ClearDropHighlight();
 					return wxDragNone;
 				}
 				cur = tree->GetItemParent(cur);
@@ -156,7 +163,8 @@ public:
 
 	virtual wxDragResult OnData(wxCoord x, wxCoord y, wxDragResult def)
 	{
-		ClearDropHighlight();
+		auto tree = m_pSiteManager->tree_;
+		tree->ClearDropHighlight();
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
@@ -164,7 +172,7 @@ public:
 			return def;
 		}
 
-		wxTreeItemId hit = GetHit(wxPoint(x, y));
+		wxTreeItemId hit = tree->GetHit(wxPoint(x, y));
 		if (!IsValidDropLocation(hit, def)) {
 			return wxDragNone;
 		}
@@ -185,9 +193,11 @@ public:
 	virtual bool OnDrop(wxCoord x, wxCoord y)
 	{
 		CScrollableDropTarget<wxTreeCtrlEx>::OnDrop(x, y);
-		ClearDropHighlight();
 
-		wxTreeItemId hit = GetHit(wxPoint(x, y));
+		auto tree = m_pSiteManager->tree_;
+		tree->ClearDropHighlight();
+
+		wxTreeItemId hit = tree->GetHit(wxPoint(x, y));
 		if (!IsValidDropLocation(hit, wxDragCopy)) {
 			return wxDragNone;
 		}
@@ -198,7 +208,7 @@ public:
 	virtual void OnLeave()
 	{
 		CScrollableDropTarget<wxTreeCtrlEx>::OnLeave();
-		ClearDropHighlight();
+		m_pSiteManager->tree_->ClearDropHighlight();
 	}
 
 	virtual wxDragResult OnEnter(wxCoord x, wxCoord y, wxDragResult def)
@@ -207,72 +217,36 @@ public:
 		return OnDragOver(x, y, def);
 	}
 
-	wxTreeItemId GetHit(const wxPoint& point)
-	{
-		int flags = 0;
-
-		wxTreeItemId hit = m_pSiteManager->tree_->HitTest(point, flags);
-
-		if (flags & (wxTREE_HITTEST_ABOVE | wxTREE_HITTEST_BELOW | wxTREE_HITTEST_NOWHERE | wxTREE_HITTEST_TOLEFT | wxTREE_HITTEST_TORIGHT)) {
-			return wxTreeItemId();
-		}
-
-		return hit;
-	}
-
 	virtual wxDragResult OnDragOver(wxCoord x, wxCoord y, wxDragResult def)
 	{
 		def = CScrollableDropTarget<wxTreeCtrlEx>::OnDragOver(x, y, def);
 
+		auto tree = m_pSiteManager->tree_;
 		if (def == wxDragError ||
 			def == wxDragNone ||
 			def == wxDragCancel)
 		{
-			ClearDropHighlight();
+			tree->ClearDropHighlight();
 			return def;
 		}
 
-		wxTreeItemId hit = GetHit(wxPoint(x, y));
+		wxTreeItemId hit = tree->GetHit(wxPoint(x, y));
 		if (!IsValidDropLocation(hit, def)) {
-			ClearDropHighlight();
+			m_pSiteManager->tree_->ClearDropHighlight();
+		}
+		else {
+			tree->DisplayDropHighlight(hit);
 		}
-
-		DisplayDropHighlight(wxPoint(x, y));
 
 		return def;
 	}
 
-	void ClearDropHighlight()
-	{
-		if (m_dropHighlight == wxTreeItemId()) {
-			return;
-		}
-
-		auto* tree = m_pSiteManager->tree_;
-		tree->SetItemDropHighlight(m_dropHighlight, false);
-		m_dropHighlight = wxTreeItemId();
-	}
-
-	wxTreeItemId DisplayDropHighlight(wxPoint p)
-	{
-		ClearDropHighlight();
-
-		wxTreeItemId hit = GetHit(p);
-		if (hit.IsOk()) {
-			auto* tree = m_pSiteManager->tree_;
-			tree->SetItemDropHighlight(hit, true);
-			m_dropHighlight = hit;
-		}
-
-		return hit;
-	}
-
 protected:
 	CSiteManagerDialog* m_pSiteManager;
-	wxTreeItemId m_dropHighlight;
 };
 
-CSiteManagerDialog::CSiteManagerDialog()
+CSiteManagerDialog::CSiteManagerDialog(COptionsBase & options)
+	: options_(options)
 {
 }
 
@@ -294,7 +268,7 @@ wxPanel * CreateBookmarkPanel(wxWindow* parent, DialogLayout const& lay)
 	main->AddGrowableCol(0);
 
 	main->Add(new wxStaticText(panel, -1, _("&Local directory:")));
-	
+
 	auto row = lay.createFlex(0, 1);
 	main->Add(row, lay.grow);
 	row->AddGrowableCol(0);
@@ -378,17 +352,17 @@ bool CSiteManagerDialog::Create(wxWindow* parent, std::vector<_connected_site>*
 	wxSize s = CThemeProvider::GetIconSize(iconSizeSmall);
 	wxImageList* pImageList = new wxImageList(s.x, s.y);
 
-	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDERCLOSED"), wxART_OTHER, s));
-	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDER"), wxART_OTHER, s));
-	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_SERVER"), wxART_OTHER, s));
-	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_BOOKMARK"), wxART_OTHER, s));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDERCLOSED"), wxART_OTHER, s, true));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_FOLDER"), wxART_OTHER, s, true));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_SERVER"), wxART_OTHER, s, true));
+	pImageList->Add(CThemeProvider::Get()->CreateBitmap(_T("ART_BOOKMARK"), wxART_OTHER, s, true));
 
 	tree_->AssignImageList(pImageList);
 
 	auto right = new wxBoxSizer(wxVERTICAL);
 	sides->Add(right, 1, wxLEFT|wxGROW, lay.gap);
 
-	m_pNotebook_Site = new CSiteManagerSite(*this);
+	m_pNotebook_Site = new CSiteManagerSite(*this, options_);
 	if (!m_pNotebook_Site->Load(this)) {
 		return false;
 	}
@@ -528,7 +502,7 @@ void CSiteManagerDialog::OnOK(wxCommandEvent&)
 
 	UpdateItem();
 
-	if (!CAskSavePasswordDialog::Run(this)) {
+	if (!CAskSavePasswordDialog::Run(this, options_)) {
 		return;
 	}
 
@@ -564,7 +538,7 @@ void CSiteManagerDialog::OnConnect(wxCommandEvent&)
 
 	UpdateItem();
 
-	if (!CAskSavePasswordDialog::Run(this)) {
+	if (!CAskSavePasswordDialog::Run(this, options_)) {
 		return;
 	}
 
@@ -578,15 +552,13 @@ void CSiteManagerDialog::OnConnect(wxCommandEvent&)
 class CSiteManagerXmlHandler_Tree : public CSiteManagerXmlHandler
 {
 public:
-	CSiteManagerXmlHandler_Tree(wxTreeCtrlEx* tree_, wxTreeItemId root, std::wstring const& lastSelection, bool predefined)
-		: m_tree_(tree_), m_item(root), m_predefined(predefined)
+	CSiteManagerXmlHandler_Tree(wxTreeCtrlEx* tree_, wxTreeItemId root, std::wstring const& lastSelection, bool predefined, int kiosk)
+		: m_tree_(tree_), m_item(root), m_predefined(predefined), m_kiosk(kiosk)
 	{
 		if (!CSiteManager::UnescapeSitePath(lastSelection, m_lastSelection)) {
 			m_lastSelection.clear();
 		}
 		m_lastSelectionIt = m_lastSelection.cbegin();
-
-		m_kiosk = COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE);
 	}
 
 	virtual ~CSiteManagerXmlHandler_Tree()
@@ -757,7 +729,7 @@ bool CSiteManagerDialog::Load()
 		return true;
 	}
 
-	std::wstring lastSelection = COptions::Get()->get_string(OPTION_SITEMANAGER_LASTSELECTED);
+	std::wstring lastSelection = options_.get_string(OPTION_SITEMANAGER_LASTSELECTED);
 	if (!lastSelection.empty() && lastSelection[0] == '0') {
 		if (lastSelection == _T("0")) {
 			tree_->SafeSelectItem(treeId);
@@ -769,7 +741,7 @@ bool CSiteManagerDialog::Load()
 	else {
 		lastSelection.clear();
 	}
-	CSiteManagerXmlHandler_Tree handler(tree_, treeId, lastSelection, false);
+	CSiteManagerXmlHandler_Tree handler(tree_, treeId, lastSelection, false, options_.get_int(OPTION_DEFAULT_KIOSKMODE));
 
 	bool res = CSiteManager::Load(element, handler);
 
@@ -818,8 +790,8 @@ bool CSiteManagerDialog::Save(pugi::xml_node element, wxTreeItemId treeId)
 
 		bool res = Save(element, m_ownSites);
 
-		if (!xml.Save(false)) {
-			if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
+		if (!xml.Save()) {
+			if (options_.get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
 				return res;
 			}
 			wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), xml.GetFileName(), xml.GetError());
@@ -1512,7 +1484,7 @@ void CSiteManagerDialog::OnCopySite(wxCommandEvent&)
 
 bool CSiteManagerDialog::LoadDefaultSites()
 {
-	CLocalPath const defaultsDir = wxGetApp().GetDefaultsDir();
+	CLocalPath const defaultsDir = GetDefaultsDir();
 	if (defaultsDir.empty()) {
 		return false;
 	}
@@ -1537,7 +1509,7 @@ bool CSiteManagerDialog::LoadDefaultSites()
 	tree_->SetItemImage(m_predefinedSites, 1, wxTreeItemIcon_Expanded);
 	tree_->SetItemImage(m_predefinedSites, 1, wxTreeItemIcon_SelectedExpanded);
 
-	std::wstring lastSelection = COptions::Get()->get_string(OPTION_SITEMANAGER_LASTSELECTED);
+	std::wstring lastSelection = options_.get_string(OPTION_SITEMANAGER_LASTSELECTED);
 	if (!lastSelection.empty() && lastSelection[0] == '1') {
 		if (lastSelection == _T("1")) {
 			tree_->SafeSelectItem(m_predefinedSites);
@@ -1549,7 +1521,7 @@ bool CSiteManagerDialog::LoadDefaultSites()
 	else {
 		lastSelection.clear();
 	}
-	CSiteManagerXmlHandler_Tree handler(tree_, m_predefinedSites, lastSelection, true);
+	CSiteManagerXmlHandler_Tree handler(tree_, m_predefinedSites, lastSelection, true, options_.get_int(OPTION_DEFAULT_KIOSKMODE));
 
 	CSiteManager::Load(element, handler);
 
@@ -1581,7 +1553,7 @@ void CSiteManagerDialog::OnBeginDrag(wxTreeEvent& event)
 	}
 #endif
 
-	if (COptions::Get()->get_int(OPTION_DND_DISABLED) != 0) {
+	if (options_.get_int(OPTION_DND_DISABLED) != 0) {
 		event.Veto();
 		return;
 	}
@@ -1902,8 +1874,8 @@ void CSiteManagerDialog::RememberLastSelected()
 	if (sel) {
 		path = GetSitePath(sel);
 	}
-	
-	COptions::Get()->set(OPTION_SITEMANAGER_LASTSELECTED, path);
+
+	options_.set(OPTION_SITEMANAGER_LASTSELECTED, path);
 }
 
 void CSiteManagerDialog::OnContextMenu(wxTreeEvent&)
@@ -1959,7 +1931,7 @@ void CSiteManagerDialog::OnExportSelected(wxCommandEvent&)
 		}
 	}
 
-	if (!xml.Save(false)) {
+	if (!xml.Save()) {
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), xml.GetFileName(), xml.GetError());
 		wxMessageBoxEx(msg, _("Error writing xml file"), wxICON_ERROR);
 	}
