@@ -4,10 +4,12 @@
 #include "filezillaapp.h"
 #include "inputdialog.h"
 
+#include <libfilezilla/local_filesys.hpp>
 #include <libfilezilla/process.hpp>
 
-CStorjKeyInterface::CStorjKeyInterface(wxWindow* parent)
-	: m_parent(parent)
+CStorjKeyInterface::CStorjKeyInterface(COptionsBase & options, wxWindow* parent)
+	: options_(options)
+	, m_parent(parent)
 {
 }
 
@@ -15,37 +17,26 @@ CStorjKeyInterface::~CStorjKeyInterface()
 {
 }
 
-std::wstring CStorjKeyInterface::GenerateKey()
+namespace {
+std::wstring quote(std::wstring_view const& arg)
 {
-	LoadProcess(false);
-
-	if (!Send(L"genkey")) {
-		return std::wstring();
-	}
-
-	std::wstring reply;
-	ReplyCode code = GetReply(reply);
-	if (code != success) {
-		return std::wstring();
-	}
-
-	return reply;
+	return L"\"" + fz::replaced_substrings(arg, L"\"", L"\"\"") + L"\"";
 }
-
-bool CStorjKeyInterface::ValidateKey(std::wstring const& key, bool silent)
+}
+std::tuple<bool, std::wstring> CStorjKeyInterface::ValidateGrant(std::wstring const& grant, bool silent)
 {
-	if (key.empty()) {
-		return false;
+	if (grant.empty()) {
+		return {false, std::wstring()};
 	}
 	LoadProcess(silent);
 
-	if (!Send(L"validatekey " + key)) {
-		return false;
+	if (!Send(L"validate grant " + quote(grant))) {
+		return {false, std::wstring()};
 	}
 
 	std::wstring reply;
 	ReplyCode code = GetReply(reply);
-	return code == success;
+	return {code == success, reply};
 }
 
 bool CStorjKeyInterface::LoadProcess(bool silent)
@@ -55,8 +46,8 @@ bool CStorjKeyInterface::LoadProcess(bool silent)
 	}
 	m_initialized = true;
 
-	std::wstring executable = COptions::Get()->get_string(OPTION_FZSTORJ_EXECUTABLE);
-	size_t pos = executable.rfind(wxFileName::GetPathSeparator());
+	std::wstring executable = options_.get_string(OPTION_FZSTORJ_EXECUTABLE);
+	size_t pos = executable.rfind(fz::local_filesys::path_separator);
 	if (pos == std::wstring::npos) {
 		if (!silent) {
 			wxMessageBoxEx(_("fzstorj could not be started.\nPlease make sure this executable exists in the same directory as the main FileZilla executable."), _("Error starting program"), wxICON_EXCLAMATION);
@@ -114,7 +105,7 @@ CStorjKeyInterface::ReplyCode CStorjKeyInterface::GetReply(std::wstring & reply)
 		if (pos == std::string::npos) {
 			int read = m_process->read(buffer, 100);
 			if (read <= 0) {
-				wxMessageBoxEx(_("Could not get reply from fzputtygen."), _("Command failed"), wxICON_EXCLAMATION);
+				wxMessageBoxEx(_("Could not get reply from fzstorj."), _("Command failed"), wxICON_EXCLAMATION);
 				m_process.reset();
 				return error;
 			}
