@@ -7,6 +7,9 @@
 
 #include <libfilezilla/time.hpp>
 
+#include <wx/animate.h>
+#include <wx/artprov.h>
+
 enum iconSize
 {
 	iconSizeTiny,
@@ -17,13 +20,20 @@ enum iconSize
 	iconSizeHuge
 };
 
+struct wxSize_cmp final
+{
+	bool operator()(wxSize const& a, wxSize const& b) const {
+		return a.x < b.x || (a.x == b.x && a.y < b.y);
+	}
+};
+
 class CTheme final
 {
 public:
 	bool Load(std::wstring const& theme);
 	bool Load(std::wstring const& theme, std::vector<wxSize> sizes);
 
-	wxBitmap const& LoadBitmap(std::wstring const& name, wxSize const& size);
+	wxBitmap const& LoadBitmap(CLocalPath const& cacheDir, std::wstring const& name, wxSize const& size);
 	wxAnimation LoadAnimation(std::wstring const& name, wxSize const& size);
 
 	static wxSize StringToSize(std::wstring const&);
@@ -33,25 +43,18 @@ public:
 	std::wstring get_author() const { return author_; }
 	std::wstring get_mail() const { return mail_; }
 
-	std::vector<wxBitmap> GetAllImages(wxSize const& size);
+	std::vector<wxBitmap> GetAllImages(CLocalPath const& cacheDir, wxSize const& size);
 private:
-	struct size_cmp final
-	{
-		bool operator()(wxSize const& a, wxSize const&b) const {
-			return a.x < b.x || (a.x == b.x && a.y < b.y);
-		}
-	};
-
 	struct cacheEntry
 	{
 		// Converting from wxImage to wxBitmap to wxImage is quite slow, so cache the images as well.
-		std::map<wxSize, wxBitmap, size_cmp> bitmaps_;
-		std::map<wxSize, wxImage, size_cmp> images_;
+		std::map<wxSize, wxBitmap, wxSize_cmp> bitmaps_;
+		std::map<wxSize, wxImage, wxSize_cmp> images_;
 	};
 
-	wxBitmap const& DoLoadBitmap(std::wstring const& name, wxSize const& size, cacheEntry & cache);
+	wxBitmap const& DoLoadBitmap(CLocalPath const& cacheDir, std::wstring const& name, wxSize const& size, cacheEntry & cache);
 
-	wxBitmap const& LoadBitmapWithSpecificSizeAndScale(std::wstring const& name, wxSize const& size, wxSize const& scale, cacheEntry & cache);
+	wxBitmap const& LoadBitmapWithSpecificSizeAndScale(CLocalPath const& cacheDir, std::wstring const& name, wxSize const& size, wxSize const& scale, cacheEntry & cache);
 
 	wxImage const& LoadImageWithSpecificSize(std::wstring const& file, wxSize const& size, cacheEntry & cache);
 
@@ -64,15 +67,16 @@ private:
 
 	fz::datetime timestamp_;
 
-	std::map<wxSize, bool, size_cmp> sizes_;
+	std::map<wxSize, bool, wxSize_cmp> sizes_;
 
 	std::map<std::wstring, cacheEntry> cache_;
 };
 
+class COptions;
 class CThemeProvider final : public wxArtProvider, public wxEvtHandler, public COptionChangeEventHandler
 {
 public:
-	CThemeProvider();
+	CThemeProvider(COptions& options);
 	virtual ~CThemeProvider();
 
 	static std::vector<std::wstring> GetThemes();
@@ -89,13 +93,20 @@ public:
 
 	wxAnimation CreateAnimation(wxArtID const& id, wxSize const& size);
 
-	virtual wxBitmap CreateBitmap(wxArtID const& id, wxArtClient const& client, wxSize const& size);
+	virtual wxBitmap CreateBitmap(wxArtID const& id, wxArtClient const& client, wxSize const& size) override {
+		return CreateBitmap(id, client, size, false);
+	}
+	wxBitmap CreateBitmap(wxArtID const& id, wxArtClient const& client, wxSize const& size, bool allowDummy);
 
-protected:
+private:
+	wxBitmap const& GetEmpty(wxSize const& size);
 
 	virtual void OnOptionsChanged(watched_options const& options);
 
+	COptions& options_;
+	CLocalPath cacheDir_;
 	std::map<std::wstring, CTheme> themes_;
+	std::map<wxSize, wxBitmap, wxSize_cmp> emptyBitmaps_;
 };
 
 #endif
