@@ -4,6 +4,7 @@
 #include "Options.h"
 #include "themeprovider.h"
 #include "xrc_helper.h"
+#include "../commonui/updater.h"
 #include "../include/version.h"
 #include <wx/hyperlink.h>
 #include <wx/statbmp.h>
@@ -13,42 +14,46 @@ BEGIN_EVENT_TABLE(CWelcomeDialog, wxDialogEx)
 EVT_TIMER(wxID_ANY, CWelcomeDialog::OnTimer)
 END_EVENT_TABLE()
 
-void CWelcomeDialog::RunDelayed(wxWindow* parent)
+CWelcomeDialog::CWelcomeDialog(COptionsBase & options, wxWindow* parent)
+    : options_(options)
+    , parent_(parent)
 {
-	CWelcomeDialog * dlg = new CWelcomeDialog;
-	dlg->parent_ = parent;
-	dlg->m_delayedShowTimer.SetOwner(dlg);
-	dlg->m_delayedShowTimer.Start(10, true);
 }
 
-bool CWelcomeDialog::Run(wxWindow* parent, bool force)
+void CWelcomeDialog::RunDelayed()
 {
-	const wxString ownVersion = GetFileZillaVersion();
-	wxString greetingVersion = COptions::Get()->get_string(OPTION_GREETINGVERSION);
+	m_delayedShowTimer.SetOwner(this);
+	m_delayedShowTimer.Start(10, true);
+}
 
-	wxString const resources = COptions::Get()->get_string(OPTION_GREETINGRESOURCES);
-	COptions::Get()->set(OPTION_GREETINGRESOURCES, _T(""));
+bool CWelcomeDialog::Run(bool force)
+{
+	auto const ownVersion = GetFileZillaVersion();
+	auto const greetingVersion = options_.get_string(OPTION_GREETINGVERSION);
+
+	auto const resources = options_.get_string(OPTION_GREETINGRESOURCES);
+	options_.set(OPTION_GREETINGRESOURCES, L"");
 
 	if (!force) {
-		if (COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
+		if (options_.get_int(OPTION_DEFAULT_KIOSKMODE) == 2) {
 			return true;
 		}
 
-		if (!greetingVersion.empty() &&
-			CBuildInfo::ConvertToVersionNumber(ownVersion.c_str()) <= CBuildInfo::ConvertToVersionNumber(greetingVersion.c_str()))
+		if (!ownVersion.empty() && !greetingVersion.empty() &&
+			ConvertToVersionNumber(ownVersion.c_str()) <= ConvertToVersionNumber(greetingVersion.c_str()))
 		{
 			// Been there done that
 			return true;
 		}
-		COptions::Get()->set(OPTION_GREETINGVERSION, ownVersion.ToStdWstring());
+		options_.set(OPTION_GREETINGVERSION, ownVersion);
 
-		if (greetingVersion.empty() && !COptions::Get()->get_int(OPTION_DEFAULT_KIOSKMODE)) {
-			COptions::Get()->set(OPTION_PROMPTPASSWORDSAVE, 1);
+		if (greetingVersion.empty() && !options_.get_int(OPTION_DEFAULT_KIOSKMODE)) {
+			options_.set(OPTION_PROMPTPASSWORDSAVE, 1);
 		}
 	}
 
 
-	Create(parent, -1, _("Welcome to FileZilla"));
+	Create(parent_, -1, _("Welcome to FileZilla"));
 
 	auto const& lay = layout();
 	auto outer = new wxBoxSizer(wxVERTICAL);
@@ -79,11 +84,11 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 
 	main->Add(new wxPanel(this, XRCID("ID_HEADERMESSAGE_PANEL")), lay.halign)->Show(false);
 
-	if (!greetingVersion.empty()) {
+	if (!greetingVersion.empty() && !ownVersion.empty()) {
 		auto news = new wxStaticText(this, -1, _("What's new"));
 		news->SetFont(news->GetFont().Bold());
 		main->Add(news);
-		main->Add(new wxHyperlinkCtrl(this, -1, wxString::Format(_("New features and improvements in %s"), GetFileZillaVersion()), wxString::Format(url, _T("news")) + _T("&oldversion=") + greetingVersion), 0, wxLEFT, lay.indent);
+		main->Add(new wxHyperlinkCtrl(this, -1, wxString::Format(_("New features and improvements in %s"), ownVersion), wxString::Format(url, _T("news")) + _T("&oldversion=") + greetingVersion), 0, wxLEFT, lay.indent);
 	}
 
 	main->AddSpacer(0);
@@ -118,7 +123,7 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 
 	buttons->Realize();
 
-	InitFooter(force ? wxString() : resources);
+	InitFooter(force ? std::wstring() : resources);
 
 	Layout();
 
@@ -132,7 +137,7 @@ bool CWelcomeDialog::Run(wxWindow* parent, bool force)
 void CWelcomeDialog::OnTimer(wxTimerEvent&)
 {
 	if (CanShowPopupDialog()) {
-		Run(parent_, false);
+		Run(false);
 	}
 	Destroy();
 }
@@ -160,10 +165,10 @@ void CreateMessagePanel(wxWindow& dlg, char const* ctrl, wxXmlResource& resource
 }
 #endif
 
-void CWelcomeDialog::InitFooter(wxString const& resources)
+void CWelcomeDialog::InitFooter(std::wstring const& resources)
 {
 #if FZ_WINDOWS && FZ_MANUALUPDATECHECK
-	if (CBuildInfo::GetBuildType() == _T("official") && !COptions::Get()->get_bool(OPTION_DISABLE_UPDATE_FOOTER)) {
+	if (CBuildInfo::GetBuildType() == _T("official") && !options_.get_bool(OPTION_DISABLE_UPDATE_FOOTER)) {
 		if (!resources.empty()) {
 			wxLogNull null;
 
